	/**
	 * Marshals this object to an instance document and outputs it to the specified output stream.
	 * @param out	OutputStream object to receive the instance document.
	 * @throws	java.io.IOException	An IO error occurs when writing out the instance document.
	 */
	public void marshal(java.io.OutputStream out)
	throws java.io.IOException
	{
		com.borland.xml.toolkit.Element elem = marshal();
		com.borland.xml.toolkit.XmlUtil.writeDocument( out, elem, getPublicId(), getSystemId(), _outputter );
	}

	/**
	 * Marshals this object to an instance document and outputs it to the specified output stream.
	 * The XML output has the following attributes:<br>
	 * <li>using the specified indent</li><br>
	 * <li>If the parameter <code>indent</code> is not <code>null</code>, elements
	 * are separated by newlines ("\r\n"). If <code>indent</code> is <code>null</code>,
	 * no newline is generated between elements.</li><br>
	 * <li>using the default encoding specified in XmlUtil class </li><br>
	 * <li>printing the "encoding" attribute in the XML declaration</li><br>
	 * <li>printing the XML declaration</li><br>
	 * @param out	OutputStream object to receive the instance document.
	 * @param indent    the string (usually white spaces) used to indent elements.
	 * @see com.borland.xml.toolkit.Outputter#Outputter(String indent)
	 * @see com.borland.xml.toolkit.XmlUtil
	 * @throws	java.io.IOException	An IO error occurs when writing out the instance document.
	 */
	public void marshal(java.io.OutputStream out, String indent)
	throws java.io.IOException
	{
		com.borland.xml.toolkit.Element elem = marshal();
		com.borland.xml.toolkit.XmlUtil.writeDocument( out, elem, getPublicId(), getSystemId(),
													   new com.borland.xml.toolkit.Outputter(indent) );
	}

	/**
	 * Marshals this object to an instance document and outputs it to the specified output stream.
	 * The XML output has the following attributes:<br>
	 * <li>using the specified indent</li><br>
	 * <li>If the parameter <code>indent</code> is not <code>null</code>, elements
	 * are separated by newlines ("\r\n"). If <code>indent</code> is <code>null</code>,
	 * no newline is generated between elements.</li><br>
	 * <li>using the specified encoding</li><br>
	 * <li>printing the "encoding" attribute in the XML declaration</li><br>
	 * <li>printing the XML declaration</li><br>
	 * @param out	OutputStream object to receive the instance document.
	 * @param indent    the string (usually white spaces) used to indent elements.
	 * @param encoding    an encoding format, such as UTF-8.
	 * @see com.borland.xml.toolkit.Outputter#Outputter(String indent, String encoding)
	 * @throws	java.io.IOException	An IO error occurs when writing out the instance document.
	 */
	public void marshal(java.io.OutputStream out, String indent, String encoding)
	throws java.io.IOException
	{
		com.borland.xml.toolkit.Element elem = marshal();
		com.borland.xml.toolkit.XmlUtil.writeDocument( out, elem, getPublicId(), getSystemId(),
													   new com.borland.xml.toolkit.Outputter(indent, encoding) );
	}

	/**
	 * Marshals this object to an instance document and outputs it to the specified output stream.
	 * The XML output has the following attributes:<br>
	 * <li>using the specified indent</li><br>
	 * <li>If the parameter <code>newlines</code> is <code>true</code>, elements
	 * are separated by newlines ("\r\n"). If <code>newlines</code> is <code>false</code>,
	 * no newline is generated between elements.</li><br>
	 * <li>using the specified encoding</li><br>
	 * <li>printing the "encoding" attribute in the XML declaration if
	 * <code>printEncoding</code> is <code>true</code>; no printing, otherwise.</li><br>
	 * <li>printing the XML declaration if <code>printDeclaration</code> is
	 * <code>true</code>; no printing, otherwise.</li><br>
	 * @param out	OutputStream object to receive the instance document.
	 * @param indent    the string (usually white spaces) used to indent elements.
	 * @param newlines  boolean value to specify whether you want to separate
	 *                  elements with carriage return and line feed ("\r\n").
	 * @param encoding    an encoding format, such as UTF-8.
	 * @param printEncoding <code>true</code> to print the "encoding" attribute;
	 *                      <code>false</code>, otherwise.
	 * @param printDeclaration <code>true</code> to print the XML declaration;
	 *                      <code>false</code>, otherwise.
	 * @see com.borland.xml.toolkit.Outputter#Outputter(String indent, boolean newlines, String encoding,
	 * boolean printEncoding, boolean printDeclaration)
	 * @throws	java.io.IOException	An IO error occurs when writing out the instance document.
	 */
	public void marshal(java.io.OutputStream out, String indent, boolean newlines,
						String encoding, boolean printEncoding, boolean printDeclaration)
	throws java.io.IOException
	{
		com.borland.xml.toolkit.Element elem = marshal();
		com.borland.xml.toolkit.XmlUtil.writeDocument( out, elem, getPublicId(), getSystemId(),
													   new com.borland.xml.toolkit.Outputter(indent, newlines, encoding,
																							 printEncoding,
																							 printDeclaration) );
	}

	/**
	 * Marshals this object to an instance document and outputs it to the specified output stream.
	 * The format of XML output is controlled by the specified outputter.
	 * @param out	OutputStream object to receive the instance document.
	 * @param outputter  Outputter object to control the format of XML output.
	 * @see com.borland.xml.toolkit.Outputter
	 * @throws	java.io.IOException	An IO error occurs when writing out the instance document.
	 */
	public void marshal(java.io.OutputStream out, com.borland.xml.toolkit.Outputter outputter)
	throws java.io.IOException
	{
		com.borland.xml.toolkit.Element elem = marshal();
		com.borland.xml.toolkit.XmlUtil.writeDocument( out, elem, getPublicId(), getSystemId(),
													   outputter );
	}

	/**
	 * Marshals this object to an instance document and stores it in the specified file.
	 * @param fileName	file to store the instance document.
	 * @throws	java.io.IOException	An IO error occurs when writing out the instance document.
	 */
	public void marshal(String fileName)
	throws java.io.IOException
	{
		com.borland.xml.toolkit.Element elem = marshal();
		com.borland.xml.toolkit.XmlUtil.writeDocument( fileName, elem, getPublicId(),
													   getSystemId(), _outputter );
	}

	/**
	 * Marshals this object to an instance document and stores it in the specified file.
	 * The XML output has the following attributes:<br>
	 * <li>using the specified indent</li><br>
	 * <li>If the parameter <code>indent</code> is not <code>null</code>, elements
	 * are separated by newlines ("\r\n"). If <code>indent</code> is <code>null</code>,
	 * no newline is generated between elements.</li><br>
	 * <li>using the default encoding specified in XmlUtil class </li><br>
	 * <li>printing the "encoding" attribute in the XML declaration</li><br>
	 * <li>printing the XML declaration</li><br>
	 * @param fileName	file to store the instance document.
	 * @param indent    the string (usually white spaces) used to indent elements.
	 * @see com.borland.xml.toolkit.Outputter#Outputter(String indent)
	 * @see com.borland.xml.toolkit.XmlUtil
	 * @throws	java.io.IOException	An IO error occurs when writing out the instance document.
	 */
	public void marshal(String fileName, String indent)
	throws java.io.IOException
	{
		com.borland.xml.toolkit.Element elem = marshal();
		com.borland.xml.toolkit.XmlUtil.writeDocument( fileName, elem, getPublicId(), getSystemId(),
													   new com.borland.xml.toolkit.Outputter(indent) );
	}

	/**
	 * Marshals this object to an instance document and stores it in the specified file.
	 * The XML output has the following attributes:<br>
	 * <li>using the specified indent</li><br>
	 * <li>If the parameter <code>indent</code> is not <code>null</code>, elements
	 * are separated by newlines ("\r\n"). If <code>indent</code> is <code>null</code>,
	 * no newline is generated between elements.</li><br>
	 * <li>using the specified encoding</li><br>
	 * <li>printing the "encoding" attribute in the XML declaration</li><br>
	 * <li>printing the XML declaration</li><br>
	 * @param fileName	file to store the instance document.
	 * @param indent    the string (usually white spaces) used to indent elements.
	 * @param encoding    an encoding format, such as UTF-8.
	 * @see com.borland.xml.toolkit.Outputter#Outputter(String indent, String encoding)
	 * @throws	java.io.IOException	An IO error occurs when writing out the instance document.
	 */
	public void marshal(String fileName, String indent, String encoding)
	throws java.io.IOException
	{
		com.borland.xml.toolkit.Element elem = marshal();
		com.borland.xml.toolkit.XmlUtil.writeDocument( fileName, elem, getPublicId(), getSystemId(),
													   new com.borland.xml.toolkit.Outputter(indent, encoding) );
	}

	/**
	 * Marshals this object to an instance document and stores it in the specified file.
	 * The XML output has the following attributes:<br>
	 * <li>using the specified indent</li><br>
	 * <li>If the parameter <code>newlines</code> is <code>true</code>, elements
	 * are separated by newlines ("\r\n"). If <code>newlines</code> is <code>false</code>,
	 * no newline is generated between elements.</li><br>
	 * <li>using the specified encoding</li><br>
	 * <li>printing the "encoding" attribute in the XML declaration if
	 * <code>printEncoding</code> is <code>true</code>; no printing, otherwise.</li><br>
	 * <li>printing the XML declaration if <code>printDeclaration</code> is
	 * <code>true</code>; no printing, otherwise.</li><br>
	 * @param fileName	file to store the instance document.
	 * @param indent    the string (usually white spaces) used to indent elements.
	 * @param newlines  boolean value to specify whether you want to separate
	 *                  elements with carriage return and line feed ("\r\n").
	 * @param encoding    an encoding format, such as UTF-8.
	 * @param printEncoding <code>true</code> to print the "encoding" attribute;
	 *                      <code>false</code>, otherwise.
	 * @param printDeclaration <code>true</code> to print the XML declaration;
	 *                      <code>false</code>, otherwise.
	 * @see com.borland.xml.toolkit.Outputter#Outputter(String indent, boolean newlines, String encoding,
	 * boolean printEncoding, boolean printDeclaration)
	 * @throws	java.io.IOException	An IO error occurs when writing out the instance document.
	 */
	public void marshal(String fileName, String indent, boolean newlines,
						String encoding, boolean printEncoding, boolean printDeclaration)
	throws java.io.IOException
	{
		com.borland.xml.toolkit.Element elem = marshal();
		com.borland.xml.toolkit.XmlUtil.writeDocument( fileName, elem, getPublicId(), getSystemId(),
													   new com.borland.xml.toolkit.Outputter(indent, newlines, encoding,
																							 printEncoding,
																							 printDeclaration) );
	}

	/**
	 * Marshals this object to an instance document and stores it in the specified file.
	 * The format of XML output is controlled by the specified outputter.
	 * @param fileName	file to store the instance document.
	 * @param outputter  Outputter object to control the format of XML output.
	 * @see com.borland.xml.toolkit.Outputter
	 * @throws	java.io.IOException	An IO error occurs when writing out the instance document.
	 */
	public void marshal(String fileName, com.borland.xml.toolkit.Outputter outputter)
	throws java.io.IOException
	{
		com.borland.xml.toolkit.Element elem = marshal();
		com.borland.xml.toolkit.XmlUtil.writeDocument( fileName, elem, getPublicId(),
													   getSystemId(), outputter );
	}

	/**
	 * Marshals this object to an instance document and stores it in the specified file.
	 * @param file	File to store the instance document.
	 * @throws	java.io.IOException	An IO error occurs when writing out the instance document.
	 */
	public void marshal(java.io.File file)
	throws java.io.IOException
	{
		com.borland.xml.toolkit.Element elem = marshal();
		com.borland.xml.toolkit.XmlUtil.writeDocument( file, elem, getPublicId(),
													   getSystemId(), _outputter );
	}

	/**
	 * Marshals this object to an instance document and stores it in the specified file.
	 * The XML output has the following attributes:<br>
	 * <li>using the specified indent</li><br>
	 * <li>If the parameter <code>indent</code> is not <code>null</code>, elements
	 * are separated by newlines ("\r\n"). If <code>indent</code> is <code>null</code>,
	 * no newline is generated between elements.</li><br>
	 * <li>using the default encoding specified in XmlUtil class </li><br>
	 * <li>printing the "encoding" attribute in the XML declaration</li><br>
	 * <li>printing the XML declaration</li><br>
	 * @param file	File to store the instance document.
	 * @param indent    the string (usually white spaces) used to indent elements.
	 * @see com.borland.xml.toolkit.Outputter#Outputter(String indent)
	 * @see com.borland.xml.toolkit.XmlUtil
	 * @throws	java.io.IOException	An IO error occurs when writing out the instance document.
	 */
	public void marshal(java.io.File file, String indent)
	throws java.io.IOException
	{
		com.borland.xml.toolkit.Element elem = marshal();
		com.borland.xml.toolkit.XmlUtil.writeDocument( file, elem, getPublicId(), getSystemId(),
													   new com.borland.xml.toolkit.Outputter(indent) );
	}

	/**
	 * Marshals this object to an instance document and stores it in the specified file.
	 * The XML output has the following attributes:<br>
	 * <li>using the specified indent</li><br>
	 * <li>If the parameter <code>indent</code> is not <code>null</code>, elements
	 * are separated by newlines ("\r\n"). If <code>indent</code> is <code>null</code>,
	 * no newline is generated between elements.</li><br>
	 * <li>using the specified encoding</li><br>
	 * <li>printing the "encoding" attribute in the XML declaration</li><br>
	 * <li>printing the XML declaration</li><br>
	 * @param file	File to store the instance document.
	 * @param indent    the string (usually white spaces) used to indent elements.
	 * @param encoding    an encoding format, such as UTF-8.
	 * @see com.borland.xml.toolkit.Outputter#Outputter(String indent, String encoding)
	 * @throws	java.io.IOException	An IO error occurs when writing out the instance document.
	 */
	public void marshal(java.io.File file, String indent, String encoding)
	throws java.io.IOException
	{
		com.borland.xml.toolkit.Element elem = marshal();
		com.borland.xml.toolkit.XmlUtil.writeDocument( file, elem, getPublicId(), getSystemId(),
													   new com.borland.xml.toolkit.Outputter(indent, encoding) );
	}

	/**
	 * Marshals this object to an instance document and stores it in the specified file.
	 * The XML output has the following attributes:<br>
	 * <li>using the specified indent</li><br>
	 * <li>If the parameter <code>newlines</code> is <code>true</code>, elements
	 * are separated by newlines ("\r\n"). If <code>newlines</code> is <code>false</code>,
	 * no newline is generated between elements.</li><br>
	 * <li>using the specified encoding</li><br>
	 * <li>printing the "encoding" attribute in the XML declaration if
	 * <code>printEncoding</code> is <code>true</code>; no printing, otherwise.</li><br>
	 * <li>printing the XML declaration if <code>printDeclaration</code> is
	 * <code>true</code>; no printing, otherwise.</li><br>
	 * @param file	File to store the instance document.
	 * @param indent    the string (usually white spaces) used to indent elements.
	 * @param newlines  boolean value to specify whether you want to separate
	 *                  elements with carriage return and line feed ("\r\n").
	 * @param encoding    an encoding format, such as UTF-8.
	 * @param printEncoding <code>true</code> to print the "encoding" attribute;
	 *                      <code>false</code>, otherwise.
	 * @param printDeclaration <code>true</code> to print the XML declaration;
	 *                      <code>false</code>, otherwise.
	 * @see com.borland.xml.toolkit.Outputter#Outputter(String indent, boolean newlines, String encoding,
	 * boolean printEncoding, boolean printDeclaration)
	 * @throws	java.io.IOException	An IO error occurs when writing out the instance document.
	 */
	public void marshal(java.io.File file, String indent, boolean newlines,
						String encoding, boolean printEncoding, boolean printDeclaration)
	throws java.io.IOException
	{
		com.borland.xml.toolkit.Element elem = marshal();
		com.borland.xml.toolkit.XmlUtil.writeDocument( file, elem, getPublicId(), getSystemId(),
													   new com.borland.xml.toolkit.Outputter(indent, newlines, encoding,
																							 printEncoding,
																							 printDeclaration) );
	}

	/**
	 * Marshals this object to an instance document and stores it in the specified file.
	 * The format of XML output is controlled by the specified outputter.
	 * @param file	File to store the instance document.
	 * @param outputter  Outputter object to control the format of XML output.
	 * @see com.borland.xml.toolkit.Outputter
	 * @throws	java.io.IOException	An IO error occurs when writing out the instance document.
	 */
	public void marshal(java.io.File file, com.borland.xml.toolkit.Outputter outputter)
	throws java.io.IOException
	{
		com.borland.xml.toolkit.Element elem = marshal();
		com.borland.xml.toolkit.XmlUtil.writeDocument( file, elem, getPublicId(), getSystemId(),
													   outputter );
	}

	/**
	 * Marshals this object to an instance document and outputs it to the specified writer.
	 * @param writer	Writer object to receive the instance document.
	 * @throws	java.io.IOException	An IO error occurs when writing out the instance document.
	 */
	public void marshal(java.io.Writer writer)
	throws java.io.IOException
	{
		com.borland.xml.toolkit.Element elem = marshal();
		com.borland.xml.toolkit.XmlUtil.writeDocument( writer, elem, getPublicId(),
													   getSystemId(), _outputter );
	}

	/**
	 * Marshals this object to an instance document and outputs it to the specified writer.
	 * The XML output has the following attributes:<br>
	 * <li>using the specified indent</li><br>
	 * <li>If the parameter <code>indent</code> is not <code>null</code>, elements
	 * are separated by newlines ("\r\n"). If <code>indent</code> is <code>null</code>,
	 * no newline is generated between elements.</li><br>
	 * <li>using the default encoding specified in XmlUtil class </li><br>
	 * <li>printing the "encoding" attribute in the XML declaration</li><br>
	 * <li>printing the XML declaration</li><br>
	 * @param writer	Writer object to receive the instance document.
	 * @param indent    the string (usually white spaces) used to indent elements.
	 * @see com.borland.xml.toolkit.Outputter#Outputter(String indent)
	 * @see com.borland.xml.toolkit.XmlUtil
	 * @throws	java.io.IOException	An IO error occurs when writing out the instance document.
	 */
	public void marshal(java.io.Writer writer, String indent)
	throws java.io.IOException
	{
		com.borland.xml.toolkit.Element elem = marshal();
		com.borland.xml.toolkit.XmlUtil.writeDocument( writer, elem, getPublicId(), getSystemId(),
													   new com.borland.xml.toolkit.Outputter(indent) );
	}

	/**
	 * Marshals this object to an instance document and outputs it to the specified writer.
	 * The XML output has the following attributes:<br>
	 * <li>using the specified indent</li><br>
	 * <li>If the parameter <code>indent</code> is not <code>null</code>, elements
	 * are separated by newlines ("\r\n"). If <code>indent</code> is <code>null</code>,
	 * no newline is generated between elements.</li><br>
	 * <li>using the specified encoding</li><br>
	 * <li>printing the "encoding" attribute in the XML declaration</li><br>
	 * <li>printing the XML declaration</li><br>
	 * @param writer	Writer object to receive the instance document.
	 * @param indent    the string (usually white spaces) used to indent elements.
	 * @param encoding    an encoding format, such as UTF-8.
	 * @see com.borland.xml.toolkit.Outputter#Outputter(String indent, String encoding)
	 * @throws	java.io.IOException	An IO error occurs when writing out the instance document.
	 */
	public void marshal(java.io.Writer writer, String indent, String encoding)
	throws java.io.IOException
	{
		com.borland.xml.toolkit.Element elem = marshal();
		com.borland.xml.toolkit.XmlUtil.writeDocument( writer, elem, getPublicId(), getSystemId(),
													   new com.borland.xml.toolkit.Outputter(indent, encoding) );
	}

	/**
	 * Marshals this object to an instance document and outputs it to the specified writer.
	 * The XML output has the following attributes:<br>
	 * <li>using the specified indent</li><br>
	 * <li>If the parameter <code>newlines</code> is <code>true</code>, elements
	 * are separated by newlines ("\r\n"). If <code>newlines</code> is <code>false</code>,
	 * no newline is generated between elements.</li><br>
	 * <li>using the specified encoding</li><br>
	 * <li>printing the "encoding" attribute in the XML declaration if
	 * <code>printEncoding</code> is <code>true</code>; no printing, otherwise.</li><br>
	 * <li>printing the XML declaration if <code>printDeclaration</code> is
	 * <code>true</code>; no printing, otherwise.</li><br>
	 * @param writer	Writer object to receive the instance document.
	 * @param indent    the string (usually white spaces) used to indent elements.
	 * @param newlines  boolean value to specify whether you want to separate
	 *                  elements with carriage return and line feed ("\r\n").
	 * @param encoding    an encoding format, such as UTF-8.
	 * @param printEncoding <code>true</code> to print the "encoding" attribute;
	 *                      <code>false</code>, otherwise.
	 * @param printDeclaration <code>true</code> to print the XML declaration;
	 *                      <code>false</code>, otherwise.
	 * @see com.borland.xml.toolkit.Outputter#Outputter(String indent, boolean newlines, String encoding,
	 * boolean printEncoding, boolean printDeclaration)
	 * @throws	java.io.IOException	An IO error occurs when writing out the instance document.
	 */
	public void marshal(java.io.Writer writer, String indent, boolean newlines,
						String encoding, boolean printEncoding, boolean printDeclaration)
	throws java.io.IOException
	{
		com.borland.xml.toolkit.Element elem = marshal();
		com.borland.xml.toolkit.XmlUtil.writeDocument( writer, elem, getPublicId(), getSystemId(),
													   new com.borland.xml.toolkit.Outputter(indent, newlines, encoding,
																							 printEncoding,
																							 printDeclaration) );
	}

	/**
	 * Marshals this object to an instance document and outputs it to the specified writer.
	 * The format of XML output is controlled by the specified outputter.
	 * @param writer	Writer object to receive the instance document.
	 * @param outputter  Outputter object to control the format of XML output.
	 * @see com.borland.xml.toolkit.Outputter
	 * @throws	java.io.IOException	An IO error occurs when writing out the instance document.
	 */
	public void marshal(java.io.Writer writer, com.borland.xml.toolkit.Outputter outputter)
	throws java.io.IOException
	{
		com.borland.xml.toolkit.Element elem = marshal();
		com.borland.xml.toolkit.XmlUtil.writeDocument( writer, elem, getPublicId(), getSystemId(),
													   outputter );
	}

	/**
	 * Unmarshals an instance document back to a <<_className_>> object from an input stream.
	 * @param in	The InputStream object to read the instance document.
	 * @param saxParserClass    fully qualified class name to identify a SAX parser.
	 *                          If this parameter is null, the default parser
	 *                          <code>org.apache.xerces.parsers.SAXParser</code> is used.
	 * @param validation  <code>true</code> to turn on the validation feature;
	 *                    <code>false</code> to turn it off.
	 * @param entityResolver EntityResolver object to provide customized handling for
	 *                       external entities.
	 * @param dtdHandler    DtdHandler object to receive notification of notation
	 *                      declaration and unparsed entity declaration events.
	 * @param errorHandler  ErrorHandler object to provide customized error handling.
	 */
	public static <<_className_>> unmarshal(java.io.InputStream in, String saxParserClass,
										    boolean validation,
											org.xml.sax.EntityResolver entityResolver,
											org.xml.sax.DTDHandler dtdHandler,
											org.xml.sax.ErrorHandler errorHandler)
	{
		com.borland.xml.toolkit.Element elem;
		elem = com.borland.xml.toolkit.XmlUtil.getDocRootElement(in, saxParserClass,
															     validation, entityResolver,
															     dtdHandler, errorHandler);
		return <<_className_>>.unmarshal(elem);
	}

	/**
	 * Unmarshals an instance document back to a <<_className_>> object from an input stream.
	 * @param in	The InputStream object to read the instance document.
	 * @param domParserClass    fully qualified class name to identify a DOM parser.
	 *                          If this parameter is null, the default parser
	 *                          <code>org.apache.xerces.parsers.DOMParser#xerces</code>
	 *                          is used.
	 * @param validation  <code>true</code> to turn on the validation feature;
	 *                    <code>false</code> to turn it off.
	 */
	public static <<_className_>> unmarshal(java.io.InputStream in, String domParserClass,
										    boolean validation)
	throws com.borland.xml.toolkit.DOMAdapterNotFoundException
	{
		com.borland.xml.toolkit.Element elem;
		elem = com.borland.xml.toolkit.XmlUtil.getDocRootElement(in, domParserClass,
															     validation);
		return <<_className_>>.unmarshal(elem);
	}

	/**
	 * Unmarshals an instance document back to a <<_className_>> object from an input stream.
	 * @param in	The InputStream object to read the instance document.
	 */
	public static <<_className_>> unmarshal(java.io.InputStream in)
	{
		return <<_className_>>.unmarshal(com.borland.xml.toolkit.XmlUtil.getDocRootElement(in));
	}

	/**
	 * Unmarshals an instance document back to a <<_className_>> object from a file.
	 * @param file	A File object, which contains the instance document.
	 * @param saxParserClass    fully qualified class name to identify a SAX parser.
	 *                          If this parameter is null, the default parser
	 *                          <code>org.apache.xerces.parsers.SAXParser</code> is used.
	 * @param validation  <code>true</code> to turn on the validation feature;
	 *                    <code>false</code> to turn it off.
	 * @param entityResolver EntityResolver object to provide customized handling for
	 *                       external entities.
	 * @param dtdHandler    DtdHandler object to receive notification of notation
	 *                      declaration and unparsed entity declaration events.
	 * @param errorHandler  ErrorHandler object to provide customized error handling.
	 */
	public static <<_className_>> unmarshal(java.io.File file, String saxParserClass,
										    boolean validation,
											org.xml.sax.EntityResolver entityResolver,
											org.xml.sax.DTDHandler dtdHandler,
											org.xml.sax.ErrorHandler errorHandler)
	{
		com.borland.xml.toolkit.Element elem;
		elem = com.borland.xml.toolkit.XmlUtil.getDocRootElement(file, saxParserClass,
															     validation, entityResolver,
															     dtdHandler, errorHandler);
		return <<_className_>>.unmarshal(elem);
	}

	/**
	 * Unmarshals an instance document back to a <<_className_>> object from a file.
	 * @param file	A File object, which contains the instance document.
	 * @param domParserClass    fully qualified class name to identify a DOM parser.
	 *                          If this parameter is null, the default parser
	 *                          <code>org.apache.xerces.parsers.DOMParser#xerces</code>
	 *                          is used.
	 * @param validation  <code>true</code> to turn on the validation feature;
	 *                    <code>false</code> to turn it off.
	 */
	public static <<_className_>> unmarshal(java.io.File file, String domParserClass,
										    boolean validation)
	throws com.borland.xml.toolkit.DOMAdapterNotFoundException
	{
		com.borland.xml.toolkit.Element elem;
		elem = com.borland.xml.toolkit.XmlUtil.getDocRootElement(file, domParserClass,
															     validation);
		return <<_className_>>.unmarshal(elem);
	}

	/**
	 * Unmarshals an instance document back to a <<_className_>> object from a file.
	 * @param file	A File object, which contains the instance document.
	 */
	public static <<_className_>> unmarshal(java.io.File file)
	{
		return <<_className_>>.unmarshal(com.borland.xml.toolkit.XmlUtil.getDocRootElement(file));
	}

	/**
	 * Unmarshals an instance document back to a <<_className_>> object from a file.
	 * @param fileName	File name of XML document.
	 * @param saxParserClass    fully qualified class name to identify a SAX parser.
	 *                          If this parameter is null, the default parser
	 *                          <code>org.apache.xerces.parsers.SAXParser</code> is used.
	 * @param validation  <code>true</code> to turn on the validation feature;
	 *                    <code>false</code> to turn it off.
	 * @param entityResolver EntityResolver object to provide customized handling for
	 *                       external entities.
	 * @param dtdHandler    DtdHandler object to receive notification of notation
	 *                      declaration and unparsed entity declaration events.
	 * @param errorHandler  ErrorHandler object to provide customized error handling.
	 */
	public static <<_className_>> unmarshal(String fileName, String saxParserClass,
										    boolean validation,
											org.xml.sax.EntityResolver entityResolver,
											org.xml.sax.DTDHandler dtdHandler,
											org.xml.sax.ErrorHandler errorHandler)
	{
		com.borland.xml.toolkit.Element elem;
		elem = com.borland.xml.toolkit.XmlUtil.getDocRootElement(fileName, saxParserClass,
															     validation, entityResolver,
															     dtdHandler, errorHandler);
		return <<_className_>>.unmarshal(elem);
	}

	/**
	 * Unmarshals an instance document back to a <<_className_>> object from a file.
	 * @param fileName	File name of XML document.
	 * @param domParserClass    fully qualified class name to identify a DOM parser.
	 *                          If this parameter is null, the default parser
	 *                          <code>org.apache.xerces.parsers.DOMParser#xerces</code>
	 *                          is used.
	 * @param validation  <code>true</code> to turn on the validation feature;
	 *                    <code>false</code> to turn it off.
	 */
	public static <<_className_>> unmarshal(String fileName, String domParserClass,
										    boolean validation)
	throws com.borland.xml.toolkit.DOMAdapterNotFoundException
	{
		com.borland.xml.toolkit.Element elem;
		elem = com.borland.xml.toolkit.XmlUtil.getDocRootElement(fileName, domParserClass,
															     validation);
		return <<_className_>>.unmarshal(elem);
	}

	/**
	 * Unmarshals an instance document back to a <<_className_>> object from a file.
	 * @param fileName	File name of XML document.
	 */
	public static <<_className_>> unmarshal(String fileName)
	{
		return <<_className_>>.unmarshal(com.borland.xml.toolkit.XmlUtil.getDocRootElement(fileName));
	}

	/**
	 * Unmarshals an instance document back to a <<_className_>> object from a URL.
	 * @param url	the URL of the instance document.
	 * @param saxParserClass    fully qualified class name to identify a SAX parser.
	 *                          If this parameter is null, the default parser
	 *                          <code>org.apache.xerces.parsers.SAXParser</code> is used.
	 * @param validation  <code>true</code> to turn on the validation feature;
	 *                    <code>false</code> to turn it off.
	 * @param entityResolver EntityResolver object to provide customized handling for
	 *                       external entities.
	 * @param dtdHandler    DtdHandler object to receive notification of notation
	 *                      declaration and unparsed entity declaration events.
	 * @param errorHandler  ErrorHandler object to provide customized error handling.
	 */
	public static <<_className_>> unmarshal(java.net.URL url, String saxParserClass,
										    boolean validation,
											org.xml.sax.EntityResolver entityResolver,
											org.xml.sax.DTDHandler dtdHandler,
											org.xml.sax.ErrorHandler errorHandler)
	{
		com.borland.xml.toolkit.Element elem;
		elem = com.borland.xml.toolkit.XmlUtil.getDocRootElement(url, saxParserClass,
															     validation, entityResolver,
															     dtdHandler, errorHandler);
		return <<_className_>>.unmarshal(elem);
	}

	/**
	 * Unmarshals an instance document back to a <<_className_>> object from a URL.
	 * @param url	the URL of the instance document.
	 * @param domParserClass    fully qualified class name to identify a DOM parser.
	 *                          If this parameter is null, the default parser
	 *                          <code>org.apache.xerces.parsers.DOMParser#xerces</code>
	 *                          is used.
	 * @param validation  <code>true</code> to turn on the validation feature;
	 *                    <code>false</code> to turn it off.
	 */
	public static <<_className_>> unmarshal(java.net.URL url, String domParserClass,
										    boolean validation)
	throws com.borland.xml.toolkit.DOMAdapterNotFoundException
	{
		com.borland.xml.toolkit.Element elem;
		elem = com.borland.xml.toolkit.XmlUtil.getDocRootElement(url, domParserClass,
															     validation);
		return <<_className_>>.unmarshal(elem);
	}

	/**
	 * Unmarshals an instance document back to a <<_className_>> object from a URL.
	 * @param url	the URL of the instance document.
	 */
	public static <<_className_>> unmarshal(java.net.URL url)
	{
		return <<_className_>>.unmarshal(com.borland.xml.toolkit.XmlUtil.getDocRootElement(url));
	}

	/**
	 * Unmarshals an instance document back to a <<_className_>> object from a reader.
	 * @param reader	The Reader object to read the instance document.
	 * @param saxParserClass    fully qualified class name to identify a SAX parser.
	 *                          If this parameter is null, the default parser
	 *                          <code>org.apache.xerces.parsers.SAXParser</code> is used.
	 * @param validation  <code>true</code> to turn on the validation feature;
	 *                    <code>false</code> to turn it off.
	 * @param entityResolver EntityResolver object to provide customized handling for
	 *                       external entities.
	 * @param dtdHandler    DtdHandler object to receive notification of notation
	 *                      declaration and unparsed entity declaration events.
	 * @param errorHandler  ErrorHandler object to provide customized error handling.
	 */
	public static <<_className_>> unmarshal(java.io.Reader reader, String saxParserClass,
										    boolean validation,
											org.xml.sax.EntityResolver entityResolver,
											org.xml.sax.DTDHandler dtdHandler,
											org.xml.sax.ErrorHandler errorHandler)
	{
		com.borland.xml.toolkit.Element elem;
		elem = com.borland.xml.toolkit.XmlUtil.getDocRootElement(reader, saxParserClass,
															     validation, entityResolver,
															     dtdHandler, errorHandler);
		return <<_className_>>.unmarshal(elem);
	}

	/**
	 * Unmarshals an instance document back to a <<_className_>> object from a reader.
	 * @param reader	The Reader object to read the instance document.
	 */
	public static <<_className_>> unmarshal(java.io.Reader reader)
	{
		return <<_className_>>.unmarshal(com.borland.xml.toolkit.XmlUtil.getDocRootElement(reader));
	}

	/**
	 * Sets the outputter to control the format of XML output.
	 */
	public void _setOutputter(com.borland.xml.toolkit.Outputter outputter)
	{
		_outputter = outputter;
	}

	/**
	 * Gets the outputter, which controls the format of XML output.
	 */
	public com.borland.xml.toolkit.Outputter _getOutputter()
	{
		return _outputter;
	}

	/**
	 * Sets the indent string that is used to indent XML output. By default, the
	 * indent is set to four white spaces. If you pass a null or "" to this method,
	 * no indentation occurs.
	 */
	public void _setIndent(String indent)
	{
		if( _outputter == null )
			_outputter = new com.borland.xml.toolkit.Outputter();

		_outputter.setIndent(indent);
	}

	/**
	 * Sets the number of white spaces used to indent XML output. By default,
	 * the indent is set to four white spaces.
	 */
	public void _setIndentSize(int indentSize)
	{
		if( _outputter == null )
			_outputter = new com.borland.xml.toolkit.Outputter();

	    _outputter.setIndentSize(indentSize);
	}

	/**
	 * Declares whether you want to separate elements with carriage return and
	 * line feed ("\r\n") in the XML output.
	 * @param newlines  <code>true</code> to print newlines; <code>false</code> otherwise.
	 */
	public void _setNewlines(boolean newlines)
	{
		if( _outputter == null )
			_outputter = new com.borland.xml.toolkit.Outputter();

		_outputter.setNewlines(newlines);
	}

	/**
	 * Replaces the existing encoding in use for XML output with a new encoding.
	 */
	public void _setEncoding(String encoding)
	{
		if( _outputter == null )
			_outputter = new com.borland.xml.toolkit.Outputter();

		_outputter.setEncoding(encoding);
	}

	/**
	 * Declares whether you to want to print the "encoding" attribute in the
	 * XML declaration.
	 */
	public void _setPrintEncoding(boolean printEncoding)
	{
		if( _outputter == null )
			_outputter = new com.borland.xml.toolkit.Outputter();

		_outputter.setPrintEncoding(printEncoding);
	}

	/**
	 * Declares whether you to want to print the XML declaration.
	 */
	public void _setPrintXMLDeclaration(boolean printXMLDeclaration)
	{
		if( _outputter == null )
			_outputter = new com.borland.xml.toolkit.Outputter();

		_outputter.setPrintXMLDeclaration(printXMLDeclaration);
	}

	/**
	 * Gets public identifier.
	 */
	public String getPublicId()
	{
		return publicId;
	}

	/**
	 * Sets public identifier.
	 */
	public void setPublicId(String publicId)
	{
		this.publicId = publicId;
	}

	/**
	 * Gets system identifier.
	 */
	public String getSystemId()
	{
		return systemId;
	}

	/**
	 * Sets system identifier.
	 */
	public void setSystemId(String systemId)
	{
		this.systemId = systemId;
	}
