/**
 * Copyright (c) 2001-2006 eXo Platform SARL, http://www.exoplatform.com/
 *
 * All rights reserved. Please look at license.txt in info directory for
 * more license detail.
 *
 * -------------------------------------------------------------------------
 * $Id: SignerParameters.java 9137 2006-09-27 01:41:57Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.exoplatform.applications.signerclient;

import java.applet.Applet;
import java.util.Collections;
import java.util.Vector;

import org.exoplatform.applet.launcher.AppletParamHelper;

/**
 * @version $Rev: 9137 $ $Date: 2006-09-27 08:41:57 +0700 (Wed, 27 Sep 2006) $
 * @author <a href="mailto:rafael.marins@exoplatform.com">Rafael Marins</a>
 */
public class SignerParameters extends AppletParamHelper {

	public static final String SIGNATURE_FIELD_PARAM = "signatureField";
	public static final String CERT_CHAIN_FIELD_PARAM = "certificationChainField";
	public static final String ENCODED_DIGEST_FIELD_PARAM = "encodedDigestField";
	public static final String SIGNING_FORM_NAME_PARAM = "signingFormName";
	public static final String ALGORITHM_PARAM = "algorithm";

	private static String[][] signerParameterInfo;

    public SignerParameters(Applet applet) {
	    super(applet);
    }

    public String getSignatureFieldNameParam() {
    	return getParameter(SIGNATURE_FIELD_PARAM);
    }

    public String getCerticateChainFieldNameParam() {
    	return getParameter(CERT_CHAIN_FIELD_PARAM);
    }

    public String getEncodedDigestFieldNameParam() {
    	return getParameter(ENCODED_DIGEST_FIELD_PARAM);
    }

    public String getSigningFormNameParam() {
    	return getParameter(SIGNING_FORM_NAME_PARAM);
    }

    public String getAlgorithmParam() {
    	return getParameter(ALGORITHM_PARAM);
    }

    /* (non-Javadoc)
     * @see org.exoplatform.applet.launcher.AppletArguments#getParameterInfo()
     */
    public String[][] getParameterInfo() {

		if (signerParameterInfo == null) {

			Vector params = new Vector();
	    	Collections.addAll(params, super.getParameterInfo());

	    	String[][] info = new String[][] {
					{SIGNING_FORM_NAME_PARAM, 			"text", 	"HTML form name of the containing fields to serve as signing communication"},
					{ENCODED_DIGEST_FIELD_PARAM, 		"text", 	"Form field name that holds the digest/hash to sign (process' first phase)"},
					{SIGNATURE_FIELD_PARAM, 			"text", 	"(Output) Form field name to give encoded (base64) signature"},
					{CERT_CHAIN_FIELD_PARAM, 			"text", 	"(Output) Form field name to write encoded (base64) certification chain"},
					{ALGORITHM_PARAM, 					"text", 	"Signing algorithm (eg: SHA1withRSA, SHA256withRSA, ...)"}
			};

	    	Collections.addAll(params, info);

	    	signerParameterInfo = (String[][]) params.toArray(new String[params.size()]);
		}

		return signerParameterInfo;
    }
}
