/**
 * Copyright (c) 2001-2006 eXo Platform SARL, http://www.exoplatform.com/
 *
 * All rights reserved. Please look at license.txt in info directory for
 * more license detail.
 *
 * -------------------------------------------------------------------------
 * $Id: JCECryptoPlugin.java 9137 2006-09-27 01:41:57Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.exoplatform.applications.signerclient.crypto.jce;

import java.io.File;
import java.io.FileInputStream;
import java.security.InvalidKeyException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.SignatureException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertPath;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.List;

import org.exoplatform.applet.plugin.AppletPlugin;
import org.exoplatform.applet.security.RestrictedAction;
import org.exoplatform.applet.security.RestrictedActor;
import org.exoplatform.applications.signerclient.crypto.CryptoException;
import org.exoplatform.applications.signerclient.crypto.CryptoPlugin;

/**
 * @version $Rev: 9137 $ $Date: 2006-09-27 08:41:57 +0700 (Wed, 27 Sep 2006) $
 * @author <a href="mailto:rafael.marins@exoplatform.com">Rafael Marins</a>
 */
public class JCECryptoPlugin implements AppletPlugin, CryptoPlugin, RestrictedActor {

	private RestrictedAction access;

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.AppletPlugin#getName()
	 */
	public String getName() {
	    return "JCECrytpoPlugin";
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.AppletPlugin#getDescription()
	 */
	public String getDescription() {
	    return "eXo Applet plugin providing crytpo functionalities.";
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.security.TrustedActor#setTrustAccess(org.exoplatform.applet.security.TrustedAccess)
	 */
	public void setTrustAccess(RestrictedAction access) {
	    this.access = access;
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applications.signerclient.CryptoPlugin#sign(byte[], java.security.PrivateKey, java.lang.String)
	 */
	public byte[] sign(byte[] hash, String algorithm, Object privateKey) throws CryptoException {

		byte[] signature = null;

		// instantiate Signer using a hash as the enclosing digest
        Signer signer = SignerFactory.getSigner(hash, algorithm);

        if (signer == null)
        	throw new CryptoException("No signer found for algorithm: " + algorithm);

        try {
        	// provide the given private key
	        signer.initSign((PrivateKey) privateKey);

	        // encrypt and encode the given digest using signature algorithm
	        signature = signer.sign();

        } catch (InvalidKeyException e) {
        	throw new CryptoException(e.getMessage(), e);
        } catch (SignatureException e) {
        	throw new CryptoException(e.getMessage(), e);
        }

        return signature;
	}

	/* (non-Javadoc)
     * @see org.exoplatform.applications.signerclient.crypto.CryptoPlugin#getCertificationChain(java.lang.Object)
     */
    public Object[] getCertificationChain(Object keyStore) throws CryptoException {

    	Certificate[] certs = null;

    	try {
        	KeyStore store = (KeyStore) keyStore;

        	// retrieve certificate list using first alias found
        	certs = store.getCertificateChain(getFirstAlias(store));

        } catch (KeyStoreException e) {
        	throw new CryptoException(
        			"Failed to retrieve certificate chain from keystore.", e);
        }

	    return certs;
    }

    /* (non-Javadoc)
     * @see org.exoplatform.applications.signerclient.crypto.CryptoPlugin#getCertPath(java.lang.Object[], java.lang.String, java.lang.String)
     */
    public byte[] getCertPath(Object[] certChain, String type) throws CryptoException {

    	byte[] encodedCertPath = null;

    	try {

        	Certificate[] certs = (Certificate[]) certChain;
        	List inChain = Arrays.asList(certs);

        	CertificateFactory cfactory = CertificateFactory.getInstance(type);

	        CertPath cp = cfactory.generateCertPath(inChain);

	        encodedCertPath = cp.getEncoded();
	        
        } catch (CertificateException e) {
        	throw new CryptoException(e.getMessage(), e);
        }

    	return encodedCertPath;
    }

	/* (non-Javadoc)
     * @see org.exoplatform.applications.signerclient.crypto.CryptoPlugin#getPrivateKey(java.lang.Object, byte[])
     */
    public Object getPrivateKey(Object keyStore, String password) throws CryptoException {

    	PrivateKey privateKey = null;

    	KeyStore store = (KeyStore) keyStore;

    	try {
	        privateKey = (PrivateKey) store.getKey(getKeyAlias(store),
	        		password.toCharArray());

        } catch (KeyStoreException e) {
        	throw new CryptoException(
        			"Failed to retrieve certificate chain from keystore.", e);

        } catch (NoSuchAlgorithmException e) {
	        throw new CryptoException(e.getMessage(), e);

        } catch (UnrecoverableKeyException e) {
        	throw new CryptoException(e.getMessage(), e);
        }

	    return privateKey;
    }

    private String getKeyAlias(KeyStore store) throws KeyStoreException {

    	String keyAlias = null;

    	Enumeration aliases = store.aliases();
    	while (aliases.hasMoreElements()) {
    		String aliasName = (String) aliases.nextElement();
    		if (store.isKeyEntry(aliasName))
    			keyAlias = aliasName;
    		break;
    	}

	    return keyAlias;
    }

	/* (non-Javadoc)
     * @see org.exoplatform.applications.signerclient.crypto.CryptoPlugin#loadKeyStore(java.io.File, byte[], java.lang.String)
     */
    public Object loadKeyStore(File ksFile, String ksPassword, String ksType) throws CryptoException {

    	KeyStore keyStore = null;

        try {
        	keyStore = KeyStore.getInstance(ksType);
			FileInputStream keyStoreStream = access.getFileInputStream(ksFile);
			keyStore.load(keyStoreStream, ksPassword.toCharArray());
        } catch (Exception e) {
        	throw new CryptoException(e.getMessage(), e);
        }

		return keyStore;
    }

    private static String getFirstAlias(KeyStore keyStore) throws KeyStoreException {

    	String firstAlias = null;

    	// retrieve the first alias from keystore
    	Enumeration aliases = keyStore.aliases();
    	if (aliases.hasMoreElements())
    		firstAlias = (String) aliases.nextElement();

    	return firstAlias;
    }
}
