/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.faces.cms;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Set;

import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.RepositoryException;

import org.exoplatform.commons.utils.MimeTypeResolver;
import org.exoplatform.container.SessionContainer;
import org.exoplatform.faces.core.component.UICalendarInput;
import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UIInput;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UIUploadInput;
import org.exoplatform.portal.session.PortalResources;
import org.exoplatform.services.cms.JcrInputProperty;

/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Feb 23, 2005
 * @version $Id$
 */
public class CMSUtils {

  static private Map smallNodeTypeIconMapping_;
  static private Map largeNodeTypeIconMapping_;
  static private Map propertyTypeIconMapping_;
  static private String defaultSmallNodeIcon_;
  static private String defaultLargeNodeIcon_;
  static private String defaultPropertyIcon_; 
  static private boolean isReferenceNode_ = false ;
  
  public static final String JCR_INFO = "jcrInfo".intern();;  
  public static final String JCR_CONTENT = "jcr:content".intern();
  public static final String JCR_DATA = "jcr:data".intern();
  public static final String JCR_MIMETYPE = "jcr:mimeType".intern();
  public static final String JCR_LASTMODIFIED = "jcr:lastModified".intern();

  public static Map prepareMap(UISimpleForm uiForm) {
    Map rawinputs = new HashMap();
    Map inputs = uiForm.getUIInputs();
    Set keys = inputs.keySet();
    Object value = null;
    for (Iterator iter = keys.iterator(); iter.hasNext();) {
      String key = (String) iter.next();
      UIInput input = (UIInput) inputs.get(key);
      JcrInputProperty property = (JcrInputProperty) input.getProperty(JCR_INFO);
      if (property != null) {
        if (uiForm.isMultiValue(key)) {
          List<UIInput> multiInputs = uiForm.getMultipleValuesUIInput(key);
          List values = new ArrayList();
          for (UIInput multiInput : multiInputs) {
            values.add(multiInput.getValue());
          }
          property.setValue((String[]) values.toArray(new String[values.size()]));
        } else {
          if (input instanceof UIUploadInput) {
            UIUploadInput upload = (UIUploadInput) input;
            value = upload.getContent();
            if (upload.isRemoved()) {
              value = null;
            } else if (!upload.hasContent()) {
              continue; // ignore if upload has not been explicitly removed
            }
            String jcrPath = property.getJcrPath();
            int idx = jcrPath.indexOf(JCR_DATA);
            if (idx > 0) {
              String resourcePath = jcrPath.substring(0, idx) ;
              String mimePath = resourcePath + JCR_MIMETYPE;
              if (rawinputs.get(mimePath) == null) {
                JcrInputProperty mimeTypeProperty = new JcrInputProperty();
                mimeTypeProperty.setJcrPath(mimePath);
                String fileName = upload.getFileName();

                String mimeType = new MimeTypeResolver().getMimeType(fileName);
                mimeTypeProperty.setValue(upload.isRemoved() ? null : mimeType);
                rawinputs.put(mimePath, mimeTypeProperty);
              }

              String lastModifiedPath = resourcePath + JCR_LASTMODIFIED;
              if (rawinputs.get(lastModifiedPath) == null) {
                JcrInputProperty lastModifiedProperty = new JcrInputProperty();
                lastModifiedProperty.setJcrPath(lastModifiedPath);
                lastModifiedProperty.setValue(upload.isRemoved() ? null : new GregorianCalendar());
                rawinputs.put(lastModifiedPath, lastModifiedProperty);
              }
            }
          } else {
            value = input.getValue();
          }
          property.setValue(value);
        }
        rawinputs.put(property.getJcrPath(), property);
      }
    }
    return rawinputs;
  }    

  public static void updateFormValues(UISimpleForm uiForm,Map jcrPropertiesMap) {
    if(jcrPropertiesMap == null) return ;    
    Map inputs = uiForm.getUIInputs();
    Set keys = inputs.keySet();
    for (Iterator iter = keys.iterator(); iter.hasNext();) {
      String key = (String) iter.next();
      UIInput input = (UIInput) inputs.get(key);
      JcrInputProperty property = (JcrInputProperty) input.getProperty(JCR_INFO);
      if(property == null) continue ;      
      JcrInputProperty  existedProperty = (JcrInputProperty)jcrPropertiesMap.get(property.getJcrPath()) ;
      if(existedProperty == null) continue ;
      Object valueObj = existedProperty.getValue() ;
      if(valueObj == null) continue ;
      if(uiForm.isMultiValue(key)) {
        if(valueObj instanceof String) 
          input.setValue((String)valueObj) ;
        else if(valueObj instanceof String[]) {
          String[] values = (String[]) valueObj ;
          input.setValue(values[0]) ;
          for(int i = 1;i<values.length;i ++) {
            String name = input.getName() + "["+i+"]" ;              
            if (input instanceof UIUploadInput) {
              uiForm.addValue(new UIUploadInput(name));
            } else if (input instanceof UICheckBox) {
              uiForm.addValue(new UICheckBox(name, name, Boolean.getBoolean(values[i])));
            } else if (input instanceof UICalendarInput) {
              uiForm.addValue(new UICalendarInput(name, values[i]));
            } else if (input instanceof UIUploadInput) {
              System.out.println("TODO : upload input");
            } else {
              uiForm.addValue(new UIStringInput(name, values[i]));
            }

          }
        }            

      } else {
        input.setValue((String)valueObj) ;
      }                    
    }    
  }

  static public void setReference(boolean b) {
    isReferenceNode_ = b ;
  }

  static public String getIconByNodeTypeName(String nodeType) {   
    if (smallNodeTypeIconMapping_ == null) {
      smallNodeTypeIconMapping_ = getSubProperties("NodeType.icon.small.");
      defaultSmallNodeIcon_ = (String) smallNodeTypeIconMapping_.get("default");
    }
    String icon ;
    icon = (String) smallNodeTypeIconMapping_.get(nodeType);
    if (icon == null)
      icon = defaultSmallNodeIcon_;
    return icon;
  }

  static public String getSmallNodeTypeIcon(Node node) {
    String nodeType = getNodeType(node);
    if (smallNodeTypeIconMapping_ == null) {
      smallNodeTypeIconMapping_ = getSubProperties("NodeType.icon.small.");
      defaultSmallNodeIcon_ = (String) smallNodeTypeIconMapping_.get("default");
    }
    String icon = getFileIcon(node, nodeType, true);
    if (icon != null)
      return icon;
    //icon = (String) smallNodeTypeIconMapping_.get(nodeType);
    if(isReferenceNode_){ 
      icon = (String) smallNodeTypeIconMapping_.get(nodeType + "_reference");
      isReferenceNode_ = false ;
    }else
      icon = (String) smallNodeTypeIconMapping_.get(nodeType);
    if (icon == null)
      icon = defaultSmallNodeIcon_;
    return icon;
  }

  static public String getLargeNodeTypeIcon(Node node) {
    String nodeType = getNodeType(node);
    if (largeNodeTypeIconMapping_ == null) {
      largeNodeTypeIconMapping_ = getSubProperties("NodeType.icon.large.");
      defaultLargeNodeIcon_ = (String) largeNodeTypeIconMapping_.get("default");
    }
    String icon = getFileIcon(node, nodeType, false);   
    if (icon != null)
      return icon;
    //icon = (String) largeNodeTypeIconMapping_.get(nodeType);
    if(isReferenceNode_){ 
      icon = (String) largeNodeTypeIconMapping_.get(nodeType + "_reference");
      isReferenceNode_ = false ;
    }else
      icon = (String) largeNodeTypeIconMapping_.get(nodeType);
    if (icon == null)
      icon = defaultLargeNodeIcon_;
    return icon;
  }

  static private String getFileIcon(Node node, String nodeType, boolean isSmall) {
    Node jcrContentNode = null;
    try {
      if ("nt:file".equals(nodeType))
        jcrContentNode = node.getNode("jcr:content");
      else if ("nt:resource".equals(nodeType))
        jcrContentNode = node;
      if (jcrContentNode != null) {        
        String mimeType = null;
        if (jcrContentNode.hasProperty("jcr:mimeType"))
          mimeType = jcrContentNode.getProperty("jcr:mimeType").getString();        
        if (isSmall)
          return (String) smallNodeTypeIconMapping_.get(nodeType + "."
              + mimeType);
        return (String) largeNodeTypeIconMapping_
        .get(nodeType + "." + mimeType);
      }
    } catch (RepositoryException e) {
    }
    return null;
  }

  static public String getFileIcon(String fileName,String nodeType,boolean isSmall) {
    MimeTypeResolver resolver = new MimeTypeResolver() ;
    String mimeType = resolver.getMimeType(fileName) ;
    try{
      if(isSmall)
        return (String)smallNodeTypeIconMapping_.get(nodeType +"."+ mimeType) ;
      return (String)largeNodeTypeIconMapping_.get(nodeType +"."+ mimeType) ;
    }catch (Exception e) {
    }
    return "" ;
  }
  public static String toHTML(String str) {
    str = str.replaceAll("<", "&lt;") ;
    str = str.replaceAll(">", "&gt;") ;
    str = str.replaceAll("\n", "<br>") ;
    str = str.replaceAll("--", "&ndash; &ndash;") ;
    str = str.replaceAll("\t", "&#09;");
    str = str.replaceAll(" ", "&nbsp;");
    return str ;
  }
  private static String getNodeType(Node node) {
    String nodeType = "";
    try {      
      nodeType = node.getPrimaryNodeType().getName();     
    } catch (Exception ex) {
      nodeType = "default";
    }
    return nodeType;
  }

  static public String getPropertyTypeIcon(Property property) {
    String name = "";
    try {
      name = property.getName();
    } catch (Exception ex) {
    }
    if (propertyTypeIconMapping_ == null) {
      propertyTypeIconMapping_ = getSubProperties("PropertyType.icon.");
      defaultPropertyIcon_ = (String) propertyTypeIconMapping_.get("default");
    }
    String icon = (String) propertyTypeIconMapping_.get(name);
    if (icon == null)
      icon = defaultPropertyIcon_;
    return icon;
  }

  static public String getPropertyIconByName(String propertyName) {
    if (propertyTypeIconMapping_ == null) {
      propertyTypeIconMapping_ = getSubProperties("PropertyType.icon.");
      defaultPropertyIcon_ = (String) propertyTypeIconMapping_.get("default");
    }    
    String icon = (String) propertyTypeIconMapping_.get(propertyName);
    if (icon == null)
      icon = defaultPropertyIcon_;
    return icon;
  }

  private static Map getSubProperties(String prefix){
    Map<String,String> map = new HashMap<String,String>();
    ResourceBundle res = getPortalBundle();
    Enumeration e = res.getKeys();
    while (e.hasMoreElements()) {      
      String key = (String) e.nextElement();          
      if (key.startsWith(prefix)) {   
        String newkey = key.substring(prefix.length(), key.length());
        map.put(newkey, res.getString(key));
      }
    }    
    return map;    
  }

  public static ResourceBundle getPortalBundle() {
    PortalResources portalResources = (PortalResources)SessionContainer.getComponent(PortalResources.class) ;
    return portalResources.getApplicationResource() ;
  }

  public static String getAttachmentSize(Node node){
    Node jcrContentNode = null;
    long size = 0;
    String sSize = "0";
    String nodeType = "";
    try {      
      nodeType = node.getPrimaryNodeType().getName();     
    } catch (Exception ex) {
      nodeType = "default";
    }
    try {
      if ("nt:file".equals(nodeType))
        jcrContentNode = node.getNode("jcr:content");
      else if ("nt:resource".equals(nodeType))
        jcrContentNode = node;
      if (jcrContentNode != null) {
        Property data = null;
        if (jcrContentNode.hasProperty("jcr:data"))
          data = jcrContentNode.getProperty("jcr:data");
        size = data.getLength();
      }
    } catch (Exception e) {
    }

    if(size >0){
      sSize = String.valueOf((((float)(size/100))/10));
    }
    sSize += " Kb";
    return sSize;
  }

}
