package org.exoplatform.portlets.content.jcr.component;

import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.jcr.Node;
import javax.jcr.Session;
import javax.jcr.Value;
import javax.portlet.PortletPreferences;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.SessionContainer;
import org.exoplatform.faces.FacesUtil;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.content.JCRUtil;
import org.exoplatform.portlets.content.jcr.component.model.Preference;
import org.exoplatform.services.cms.JCRSessionManager;
import org.exoplatform.services.cms.drives.ManageDriveService;
import org.exoplatform.services.jcr.RepositoryService;

public class UIDriveBrowser extends UIExoCommand{
  private static final String SELECT_WORKSPACE_ACTION = "selectWorkspace".intern() ;
  public static final int NB_PER_ROW = 4;

  static Parameter[] SELECT_WORKSPACE_PARAMS = { new Parameter(ACTION , SELECT_WORKSPACE_ACTION)} ;

  private String userName_ ;
  private ManageDriveService dservice_ ;
  private RepositoryService repositoryService_ ;
  public UIDriveBrowser(ManageDriveService dservice,RepositoryService repositoryService) throws Exception {  	
    setRendererType("VelocityRenderer") ;
    setId("UIDriveBrowser");   
    dservice_ = dservice ;
    repositoryService_ = repositoryService ;
    FacesContext context  = FacesContext.getCurrentInstance() ;
    ExternalContext econtext = context.getExternalContext()  ;
    userName_ = econtext.getRemoteUser() ;
    addActionListener(SelectWorkspaceActionListener.class, SELECT_WORKSPACE_ACTION);
  }

  public Parameter[] getSelectWorkspaceParams() { return SELECT_WORKSPACE_PARAMS ; }  

  public String[] getWorkspaces() throws Exception { 
    return repositoryService_.getRepository().getWorkspaceNames(); 
  }

  public List<Node> getDrives() throws Exception {
    return dservice_.getAllDriveByPermission(userName_) ;
  }

  public String getActionIcon(Node drive) throws Exception {
    String icon = null ;
    Session session = null ;
    String path = drive.getProperty("exo:icon").getString() ;
    if(path != null && path.length() > 0) {      
      try{
        session = repositoryService_.getRepository().getSystemSession("digital-assets") ;
        Node file = (Node)session.getItem(path) ;
        if(file.hasNode("jcr:content")){
          String portalName = PortalContainer.getInstance().getPortalContainerInfo().getContainerName() ;
          icon = "<img src=\"/"+portalName+"/jcr?workspace=digital-assets&path=" + file.getPath() +
          "&relativePath=jcr:content\" width=\"64\" height=\"64\" />" ;
          session.logout() ;
          return icon ;
        }
      }catch (Exception ex) {
        if(session !=null ) session.logout() ;
      }      
    }    
    ResourceBundle res = JCRUtil.getPortletBundle();
    icon = res.getString("UIDriveBrowser.icon.workspace." + drive.getProperty("exo:workspace").getString());
    return icon;
  }

  static public class SelectWorkspaceActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event)  throws Exception {			
      UIDriveBrowser uicomp = (UIDriveBrowser) event.getComponent() ;			    						
      String driveName = event.getParameter(OBJECTID) ;
      Node drive = uicomp.dservice_.getDriveByName(driveName) ;      
      String workspaceName = drive.getProperty("exo:workspace").getString() ;
      String path = drive.getProperty("exo:path").getString() ;
      List<String> folderTypes = new ArrayList<String>() ;
      for(Value v:drive.getProperty("exo:folderTypes").getValues()) {
        folderTypes.add(v.getString()) ; 
      }        
      UIJCRExplorer explorer = (UIJCRExplorer) uicomp.getSibling(UIJCRExplorer.class);	               
      PortletPreferences prefs = FacesUtil.getPortletPreferences(); 
      List<String> views = new ArrayList<String>() ;
      for(Value v:drive.getProperty("exo:views").getValues())
        views.add(v.getString()) ;      
      prefs.setValue(Utils.WORKSPACE_NAME, workspaceName) ;
      prefs.setValue(Utils.JCR_PATH, path) ;
      prefs.setValues("views", views.<String>toArray(new String[views.size()])) ;
      prefs.setValue("drive", driveName) ;
      prefs.setValues("folderTypes",folderTypes.toArray(new String[folderTypes.size()])) ;
      prefs.store() ;
      JCRSessionManager sessionManager = 
        (JCRSessionManager)PortalContainer.getComponent(JCRSessionManager.class) ;
      String appSessionId = SessionContainer.getInstance().getSessionId() ;
      Session session = sessionManager.openSession(workspaceName,appSessionId) ; 
      Node node = (Node)session.getItem(path) ;
      explorer.getAllClipBoard().clear() ;
      explorer.setSession(session);				
      explorer.setSelectNode(node) ;
      Preference pref = explorer.getPreference() ;
      pref.setShowPreferenceDocuments(drive.getProperty("exo:viewPreferences").getBoolean()) ;
      pref.setShowNonDocumentType(drive.getProperty("exo:viewNonDocument").getBoolean()) ;
      pref.setLeftExplorer(drive.getProperty("exo:viewExplorer").getBoolean()) ;
      pref.setClipBoardListState(drive.getProperty("exo:viewClipboard").getBoolean()) ;
      pref.setEmpty(false) ;
      UIViewController viewController = (UIViewController)explorer.getChildComponentOfType(UIViewController.class) ;            
      viewController.renderTabs(views.get(0)) ;
      explorer.broadcastOnChange(node) ;
      uicomp.setRenderedSibling(UIJCRExplorer.class) ;						   			
    }
  }  
}
