/***************************************************************************
 * Copyright 2001-2007 The eXo Platform SAS         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.jcr.component;

import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.Item;
import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.PropertyType;
import javax.jcr.Value;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.PropertyDefinition;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.cms.CMSDialogComponent;
import org.exoplatform.faces.cms.CMSUtils;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.component.UIViewController.CancelActionListener;
import org.exoplatform.services.cms.metadata.MetadataService;
import org.exoplatform.services.jcr.core.nodetype.ExtendedNodeType;

/**
 * Created by The eXo Platform SAS
 * Author : Pham Xuan Hoa
 *          hoa.pham@exoplatform.com
 * May 14, 2007  
 */
public class UIMetadataInfo extends UIExoCommand implements ChangeNodeListener, CMSDialogComponent {

  public static final String EXO_METADATA  = "exo:metadata".intern() ;
  public static final String JCR_PRIMARY_NODE = "jcrPrimaryNode:".intern() ;
  public static final String INTERNAL_USE = "exo:internalUse".intern() ;  

  static Parameter[] EDIT_PARAMS = { new Parameter(ACTION , EDIT_ACTION)} ;
  static Parameter[] CANCEL_PARAMS = { new Parameter(ACTION , CANCEL_ACTION)} ;

  private MetadataService mservice_ ;
  private Node node_ ;  
  private Node viewNode_ ;

  public UIMetadataInfo(MetadataService metaService) {    
    setId("UIMetadataInfo") ;
    setRendererType("VelocityRenderer") ;

    mservice_ = metaService ;
    addActionListener(EditMetadataActionListener.class,EDIT_ACTION) ;
    addActionListener(CancelActionListener.class,CANCEL_ACTION) ;
  }

  public Parameter[] getEditParams() { return EDIT_PARAMS ; }
  public Parameter[] getCancelParams() { return CANCEL_PARAMS ; }

  public boolean isInternalUse(String metadataType) throws Exception {
    ExtendedNodeType nodeType = (ExtendedNodeType)node_.getSession().getWorkspace().getNodeTypeManager().getNodeType(metadataType) ;
    PropertyDefinition def = nodeType.getPropertyDefinitions(INTERNAL_USE).getAnyDefinition() ;
    return def.getDefaultValues()[0].getBoolean() ;
  }

  public List<String> getMetadataNames() throws Exception {       
    List<String> metaNames = new ArrayList<String>() ;
    for(NodeType mixin:node_.getMixinNodeTypes()) {
      if(mixin.isNodeType(EXO_METADATA)) 
        metaNames.add(mixin.getName()) ;
    }    
    return metaNames ;
  }

  public List<String> getViewsPath(List<String> metaList) throws Exception {
    List<String> list = new ArrayList<String>() ;
    for(String metaType:metaList) {
      String viewPath = mservice_.getMetadataTemplatePath(metaType,false) ;
      if(viewPath == null || viewPath.length() ==0) continue ;
      list.add("jcr:"+viewPath) ;
    }
    return list ;
  }  

  public String getMetaIcon(String metaType) {
    return CMSUtils.getIconByNodeTypeName(metaType) ;
  }

  public List<String> getMultiValues(Node node, String name) throws Exception {
    List<String> list = new ArrayList<String>();
    if (!node.hasProperty(name))
      return list;
    Property property = node.getProperty(name) ;
    if(property.getType() == PropertyType.REFERENCE ||
        property.getType() == PropertyType.BINARY || property.getType() == PropertyType.REFERENCE)
      return list ;    
    if (property.getDefinition().isMultiple()) {        
      for (Value value : property.getValues()) {
        list.add(value.getString());
      }
    }else
      list.add(property.getString());               
    return list;
  }

  public Node getCurrentNode() { return node_ ; }

  public Node getNode() {  return viewNode_ ; }
  public void setViewNode(Node viewNode) { this.viewNode_ = viewNode ; } 

  public List<String> getPrimaryItemMetaNames() throws Exception {
    List<String> metaNames = new ArrayList<String>() ;
    Node primaryNode = getPrimaryItemNode() ;
    if(primaryNode == null ) return metaNames ;
    for(NodeType  mixin:primaryNode.getMixinNodeTypes()) 
      if(mixin.isNodeType(EXO_METADATA))
        metaNames.add(mixin.getName()) ;    
    return metaNames ;
  }


  public Node getPrimaryItemNode() {
    try{
      Item primaryItem = node_.getPrimaryItem();
      if(primaryItem != null && primaryItem.isNode())
        return (Node)primaryItem ;
    }catch (Exception e) {      
    }
    return null ;
  }


  public void onChange(UIJCRExplorer uiExplorer, Node node) throws Exception {
    node_ = node ;
    this.setRendered(false) ;
  }

  public static class EditMetadataActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIMetadataInfo metaInfo = (UIMetadataInfo)event.getComponent() ;
      if(!metaInfo.getCurrentNode().isCheckedOut()) {
        Information iprovider = findInformationProvider(metaInfo) ;
        ResourceBundle res = getApplicationResourceBundle() ;
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN,
            res.getString("UIMetadataInfo.msg.node-is-checkedin"))) ;
        return ;
      }
      String objId = event.getParameter(OBJECTID) ;
      UIMetadataForm form = (UIMetadataForm)metaInfo.getSibling(UIMetadataForm.class) ;
      form.clearInputs() ;
      if(objId.startsWith("primaryNode:")) {   
        String metaName = objId.substring("primaryNode:".length()) ;
        form.update(metaInfo.getPrimaryItemNode(),metaName) ;
      }else {
        form.update(metaInfo.getCurrentNode(),objId) ;
      }                         
      metaInfo.setRenderedSibling(UIMetadataForm.class) ;
    }    
  }

}
