package org.exoplatform.portlets.content.jcr.component;

import java.util.ArrayList;
import java.util.List;

import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.content.jcr.component.model.Preference;

public class UIPreferencesForm extends UISimpleForm {
  
  public static final String CHECKBOX = "checkbox";
  public static final String SORT_TYPE = "sortType";
  public static final String ORDER = "order";
  public static final String PROPERTY = "property";
  public static final String LEFT_EXPLORER = "leftExplorer" ;
  public static final String SHOW_CLIPBOARD = "showClipBoard" ;
  public static final String SHOW_NON_DOCUMENTTYPE = "showNonDocumentType" ;
  public static final String SHOW_PREFERENCES = "showPreferences" ;
  public static final String PAGE_SIZE = "pageSize" ;
  
  protected UICheckBox leftExplorerBox_ ;
  protected UICheckBox clipboardBox_ ;
  protected UICheckBox showNonDocumentTypeBox_ ;
  protected UICheckBox showPreferencesBox_ ;
  private UISelectBox uiSelectSortOptions_ ;
  private UISelectBox uiSelectOrderOptions_ ;
  private UIStringInput uiProperty_ ;
  private UISelectBox uiPageSize_ ;
  public UIPreferencesForm() {
    super("preferencesForm", "post", null) ;    
    setRendererType("SimpleFormVelocityRenderer");   
    
    add(new UICheckBox(CHECKBOX, "false"));
    leftExplorerBox_ = new UICheckBox(LEFT_EXPLORER,"false").setSelect(false) ;
    add(leftExplorerBox_) ;
    clipboardBox_ = new UICheckBox(SHOW_CLIPBOARD,"false").setSelect(false) ;    
    add(clipboardBox_) ;
    showNonDocumentTypeBox_ = new UICheckBox(SHOW_NON_DOCUMENTTYPE, "false").setSelect(false) ;
    add(showNonDocumentTypeBox_) ;
    showPreferencesBox_ = new UICheckBox(SHOW_PREFERENCES, "false").setSelect(false) ;
    add(showPreferencesBox_) ;
    uiProperty_ = new UIStringInput(PROPERTY,"") ;
    add(uiProperty_) ;
    
    List<SelectItem> sortOptions = new ArrayList<SelectItem>() ;
    sortOptions.add(new SelectItem(Preference.ALPHABETICAL_SORT, Preference.ALPHABETICAL_SORT));
    sortOptions.add(new SelectItem(Preference.TYPE_SORT, Preference.TYPE_SORT));
    sortOptions.add(new SelectItem(Preference.PROPERTY_SORT, Preference.PROPERTY_SORT));
    uiSelectSortOptions_ = new  UISelectBox(SORT_TYPE, "", sortOptions) ;
    add(uiSelectSortOptions_);
    
    List<SelectItem> orderOptions = new ArrayList<SelectItem>() ;
    orderOptions.add(new SelectItem(Preference.ASCENDING_ORDER, Preference.ASCENDING_ORDER));
    orderOptions.add(new SelectItem(Preference.DESCENDING_ORDER, Preference.DESCENDING_ORDER));
    uiSelectOrderOptions_ = new  UISelectBox(ORDER, "", orderOptions) ;
    add(uiSelectOrderOptions_);
    
    List<SelectItem> pageSizeOptions = new ArrayList<SelectItem>() ;
    pageSizeOptions.add(new SelectItem("5","5")) ;
    pageSizeOptions.add(new SelectItem("10","10")) ;
    pageSizeOptions.add(new SelectItem("20","20")) ;
    pageSizeOptions.add(new SelectItem("50","50")) ;
    pageSizeOptions.add(new SelectItem("100","100")) ;
    pageSizeOptions.add(new SelectItem("All",Integer.toString(Integer.MAX_VALUE))) ;
    uiPageSize_ = new UISelectBox(PAGE_SIZE,"",pageSizeOptions) ;
    add(uiPageSize_) ;
    
    addActionListener(SaveActionListener.class, SAVE_ACTION) ;
    addActionListener(CancelActionListener.class, CANCEL_ACTION) ;
  }
  
  public void update(Preference pref) {
    leftExplorerBox_.setSelect(pref.showLeftExplorer()) ;
    clipboardBox_.setSelect(pref.isShowClipBoardList()) ;
    showNonDocumentTypeBox_.setSelect(pref.isShowNonDocumentType()) ;
    showPreferencesBox_.setSelect(pref.isShowPreferenceDocuments()) ;
    uiSelectSortOptions_.setValue(pref.getSort()) ;
    uiSelectOrderOptions_.setValue(pref.getOrder()) ;
    uiProperty_.setValue(pref.getProperty()) ;
    uiPageSize_.setValue(Integer.toString(pref.getNodesPerPage())) ;
  }
  
  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPreferencesForm uicomp = (UIPreferencesForm) event.getSource() ;
      UIJCRExplorer explorer = (UIJCRExplorer) uicomp.getSibling(UIJCRExplorer.class);
      
      UICheckBox box = (UICheckBox) uicomp.getUIInput(CHECKBOX);
      boolean jcrEnable = new Boolean(box.getSelect()).booleanValue();
      UICheckBox leftExplorerBox = (UICheckBox)uicomp.getUIInput(LEFT_EXPLORER);
      boolean showLeftExplorer = new Boolean(leftExplorerBox.getSelect()).booleanValue() ;
      UICheckBox showClipBoardBox = (UICheckBox)uicomp.getUIInput(SHOW_CLIPBOARD) ;
      boolean showClipBoard = new Boolean(showClipBoardBox.getSelect()).booleanValue() ;
      boolean showNonDocument = uicomp.getUICheckBox(SHOW_NON_DOCUMENTTYPE).getSelect() ;
      boolean showPreferences = uicomp.getUICheckBox(SHOW_PREFERENCES).getSelect() ;
      String sort = uicomp.getUIInput(SORT_TYPE).getValue();
      String order = uicomp.getUIInput(ORDER).getValue();
      String property = uicomp.getUIStringInput(PROPERTY).getValue();
      int pazeSize = Integer.parseInt(uicomp.getUIInput(PAGE_SIZE).getValue()) ;
      
      Preference pref = explorer.getPreference();      
      pref.setJcrEnable(jcrEnable);
      pref.setLeftExplorer(showLeftExplorer);
      pref.setClipBoardListState(showClipBoard) ;
      pref.setSort(sort);
      pref.setOrder(order);
      pref.setProperty(property);
      pref.setShowNonDocumentType(showNonDocument) ;
      pref.setShowPreferenceDocuments(showPreferences) ;
      pref.setNodesPerPage(pazeSize) ;
      explorer.broadcastOnChange(explorer.currentNode_) ;
      uicomp.setRenderedSibling(UIJCRExplorer.class);
    }
  }  
  
  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPreferencesForm uicomp = (UIPreferencesForm) event.getSource() ;
      uicomp.setRenderedSibling(UIJCRExplorer.class);      
    }
  }    

}
