/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.jcr.component;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.PropertyType;
import javax.jcr.Value;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.PropertyDefinition;

import org.exoplatform.faces.cms.CMSUtils;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;

/**
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIContentDisplayer.java,v 1.2 2004/08/07 18:11:26 tuan08 Exp $
 */
public class UIPropertiesInfo extends UIExoCommand implements ChangeNodeListener {
      
  private static String ADD_PROPERTY_ACTION = "addProperty" ;
  private static String SET_PROPERTY_ACTION = "setProperty" ;  
  
  static Parameter[] BACK_PARAMS = { new Parameter(ACTION , CANCEL_ACTION)} ;
  static Parameter[] ADD_PROPERTY_PARAMS = { new Parameter(ACTION , ADD_PROPERTY_ACTION)} ;  
  static Parameter[] SET_PROPERTY_PARAMS = { new Parameter(ACTION , SET_PROPERTY_ACTION)} ;
  
  private Node node_ ;
  private List propertyList_ = new ArrayList() ;
  
  public UIPropertiesInfo() {
    setRendererType("VelocityRenderer") ;
    
    addActionListener(AddPropertyActionListener.class, ADD_PROPERTY_ACTION);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
    addActionListener(SetPropertyActionListener.class, SET_PROPERTY_ACTION);
  }
  public Parameter[] getBackParams() { return BACK_PARAMS ; }
  public void  onChange(UIJCRExplorer explorer, Node node)  {
    node_ = node ;
    setRendered(false) ;
  }
  
  public Parameter[] getAddPropertyParams() { return ADD_PROPERTY_PARAMS; }  
  public Parameter[] getSetPropertyParams() { return SET_PROPERTY_PARAMS; }
  
  public Node getNode() { return node_ ; }
  
  public Iterator getProperties() throws Exception { return node_.getProperties() ; }
  
  public boolean canAddProperty() throws Exception {   
    try {
      NodeType nodeType = node_.getPrimaryNodeType() ;            
      if(nodeType.isNodeType("nt:unstructured")) return true ;
      return false ;
    }catch(Exception e) {
      return false ;
    }    
  }
  
  public boolean canSetProperty() throws Exception {
    try {
      propertyList_ = new ArrayList() ;
      NodeType nodeType = node_.getPrimaryNodeType() ;
      PropertyDefinition[] propertiesDef = nodeType.getPropertyDefinitions() ;
      for(int i = 0; i < propertiesDef.length ; i ++) {
        try {
          Property pro = node_.getProperty(propertiesDef[i].getName()) ;
          try{
            Value vl = pro.getValue() ;
            if(nodeType.canSetProperty(pro.getName(), vl))
              propertyList_.add(pro.getName()) ;
          }catch (Exception e){
            Value[] vls = pro.getValues() ;
            if(nodeType.canSetProperty(pro.getName(), vls))
              propertyList_.add(pro.getName()) ;
          }
        }catch (Exception e) {}
      }
      if (propertyList_.size() > 0 ) return true ;
      return false ;
    }catch(Exception e) {      
      return false ;
    }    
  }
  
  public boolean isMultiValue(Property prop) throws Exception{
    return prop.getDefinition().isMultiple() ;    
  }
  
  public String getPropertyIcon(Property prop) { return CMSUtils.getPropertyTypeIcon(prop) ; }
  
  public String getPropertyValue(Property prop) throws Exception {    
    if(prop.getType() == PropertyType.BINARY) 
      return "jcr:data is binary data";
    try {            
      if(prop.getDefinition().isMultiple()) {
        Value[] values =  prop.getValues();
        StringBuffer sB = new StringBuffer();
        for (int i = 0; i < values.length; i++) {
          if(i > 0) sB.append("; ") ;
          sB.append(values[i].getString());
        }
        return sB.toString();
      }
      return prop.getValue().getString() ;
    }catch(Exception e) {
      return "Can not get values!" ;
    }
  }
  
  static public class AddPropertyActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPropertiesInfo propertiesInfo = (UIPropertiesInfo) event.getSource();
      propertiesInfo.setRenderedSibling(UIPropertyForm.class);
    }
  }      
  
  static public class SetPropertyActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPropertiesInfo propertiesInfo = (UIPropertiesInfo) event.getSource();
      UISetPropertyForm uiSetProperty = 
        (UISetPropertyForm)propertiesInfo.getSibling(UISetPropertyForm.class) ;
      uiSetProperty.update(propertiesInfo.propertyList_) ;
      propertiesInfo.setRenderedSibling(UISetPropertyForm.class); 
            
    }
  }
  
  static public class CancelActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIPropertiesInfo uiForm = (UIPropertiesInfo) event.getComponent();
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uiForm.getAncestorOfType(UIJCRExplorer.class);
      uiExplorer.broadcastOnChange(uiExplorer.getSelectNode());       
    }
  }
}