package org.exoplatform.portlets.content.taxonomy.component;

import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.Node;
import javax.jcr.ReferentialIntegrityException;
import javax.jcr.RepositoryException;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.admin.component.UIECMAdminSummary;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.jcr.component.model.ClipboardCommand;
import org.exoplatform.services.cms.categories.CategoriesService;

public class UITaxonomyManager extends UIExoCommand {
	final static public String SELECT_TAXONOMY_ACTION = "selectTaxonomy".intern() ;
	final static public String ADD_TAXONOMY_ACTION = "addTaxonomy".intern() ;		
	final static public String REMOVE_TAXONOMY_ACTION = "removeTaxonomy".intern();
	public static final String PASTE_ACTION = "paste".intern();
	public static final String COPY_ACTION = "copy".intern();
	public static final String CUT_ACTION = "cut".intern();	     
	
	static Parameter[] SELECT_TAXONOMY_PARAMS = { new Parameter(ACTION , SELECT_TAXONOMY_ACTION)} ;
	static Parameter[] ADD_TAXONOMY_PARAMS = { new Parameter(ACTION , ADD_TAXONOMY_ACTION)} ;  	
	static Parameter[] REMOVE_TAXONOMY_PARAMS = { new Parameter(ACTION,REMOVE_TAXONOMY_ACTION) };	
	static Parameter[] COPY_NODE_PARAMS     = { new Parameter(ACTION,COPY_ACTION) };
	static Parameter[] CUT_NODE_PARAMS      = { new Parameter(ACTION,CUT_ACTION) };	
	static Parameter[] PASTE_NODE_PARAMS = { new Parameter(ACTION, PASTE_ACTION) };
	static Parameter[] BACK_PARAMS     = { new Parameter(ACTION,BACK_ACTION) };
	
	protected CategoriesService categoriesService_ ;
	private TaxonomyNode currentNode_ ;
	protected TaxonomyNode rootTaxonnomy_ ;
	protected ClipboardCommand clipboard_ = new ClipboardCommand() ;
	
	public UITaxonomyManager(CategoriesService categoriesService) throws Exception {
		setId("UITaxonomyExplorer") ;
		setRendererType("VelocityRenderer") ;
		
		categoriesService_ = categoriesService ; 								
		
		addActionListener(SelectTaxonomyActionListener.class,SELECT_TAXONOMY_ACTION) ;
		addActionListener(AddTaxonomyActionListener.class,ADD_TAXONOMY_ACTION) ;		
		addActionListener(RemoveTaxonomyActionListener.class,REMOVE_TAXONOMY_ACTION) ;
		addActionListener(CopyTaxonomyActionListener.class,COPY_ACTION) ;
		addActionListener(CutTaxonomyActionListener.class,CUT_ACTION) ;
		addActionListener(PasteTaxonomyActionListener.class,PASTE_ACTION);
		addActionListener(BackActionListener.class,BACK_ACTION) ;
	}
	
	public Parameter[] getSelectTaxonomyParams() { return SELECT_TAXONOMY_PARAMS ;}
	public Parameter[] getAddTaxonomyParams() { return ADD_TAXONOMY_PARAMS ;}			        					  	
	public Parameter[] getCopyTaxonomyParams() { return COPY_NODE_PARAMS ; }
	public Parameter[] getCutTaxonomyParams() { return CUT_NODE_PARAMS ; }
	public Parameter[] getPasteTaxonomyParams() { return PASTE_NODE_PARAMS ; }
	public Parameter[] getRemoveTaxonomyParams() {return REMOVE_TAXONOMY_PARAMS ; }
	public Parameter[] getBackParams() {return BACK_PARAMS ; }
	
	public ClipboardCommand getClipBoard() { return clipboard_ ; }
	
	public TaxonomyNode getRootTaxonomyNode() throws Exception {  
		if(rootTaxonnomy_==null) {
			try {
				rootTaxonnomy_ = new TaxonomyNode(categoriesService_.getTaxonomyHomeNode()) ;
			}catch (RepositoryException e) {
				return null ;
			}
		}							  	
		return rootTaxonnomy_ ;
	}
	
	public void update() throws Exception {
		try {
			rootTaxonnomy_ = new TaxonomyNode(categoriesService_.getTaxonomyHomeNode()) ;
		}catch (RepositoryException e) {			
		}
	}
	public void setSelectedTaxonomy(TaxonomyNode selectedTaxonomy) throws Exception{  	
		currentNode_ = selectedTaxonomy ;  	  	  	
	}
	
	public TaxonomyNode getSelectedTaxonomy() {
		return currentNode_ ;
	}		
	
	public static class SelectTaxonomyActionListener extends ExoActionListener {		
		public void execute(ExoActionEvent event) throws Exception {
			UITaxonomyManager uimanager = (UITaxonomyManager)event.getComponent() ;
			TaxonomyNode root = uimanager.getRootTaxonomyNode() ;
			String path = event.getParameter(OBJECTID) ;
			TaxonomyNode selectedTaxonomy= root.findTaxonomyNode(path) ;
			selectedTaxonomy.setExpanded(!selectedTaxonomy.isExpanded()) ;
		}		 
	}	
	
	public static class AddTaxonomyActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {	
			UITaxonomyManager uimanager = (UITaxonomyManager)event.getComponent() ;
			TaxonomyNode root = uimanager.getRootTaxonomyNode() ;
			String realPath = event.getParameter(OBJECTID) ;				
			TaxonomyNode selectedTaxonomy= root.findTaxonomyNode(realPath) ;
			uimanager.setSelectedTaxonomy(selectedTaxonomy) ;		
			((UITaxonomyForm)uimanager.getSibling(UITaxonomyForm.class)).reset() ;
			uimanager.setRenderedSibling(UITaxonomyForm.class) ;
		}  	
	}		
	
	public static class RemoveTaxonomyActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UITaxonomyManager uimanager = (UITaxonomyManager)event.getComponent() ;			
			Information iprovider = findInformationProvider(uimanager);
      ResourceBundle res = getApplicationResourceBundle() ;
			String path = event.getParameter(OBJECTID) ;						
			try {
				uimanager.categoriesService_.removeTaxonomyNode(path) ;
			}catch (ReferentialIntegrityException e) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR,
              res.getString("UITaxonomyManager.msg.referential-integrity")));
      }
      catch (Exception ex) {
//				ex.printStackTrace() ;
				JCRExceptionManager.process(iprovider, ex);
			}
			uimanager.update() ;
		}		
	}
	
	static public class CopyTaxonomyActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {    				
			UITaxonomyManager uimanager = (UITaxonomyManager)event.getComponent() ;
			String realPath = event.getParameter(OBJECTID);            
			Information iprovider = findInformationProvider(uimanager);
      ResourceBundle res = getApplicationResourceBundle() ;
			try {      	
				Node node = uimanager.categoriesService_.getTaxonomyNode(realPath) ;
				uimanager.getClipBoard().setType(ClipboardCommand.COPY) ;
				uimanager.getClipBoard().setSrcPath(node.getPath());				
				Object[] args = { node.getPath() };
				iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_INFO,
                             res.getString("UITaxonomyManager.msg.taxonomy-copied"), args));
			} catch (Exception e) {
				JCRExceptionManager.process(iprovider, e);
			}
		}		                                   
	}
	
	static public class CutTaxonomyActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			ResourceBundle res = getApplicationResourceBundle() ;
			UITaxonomyManager uimanager = (UITaxonomyManager)event.getComponent() ;
			String realPath = event.getParameter(OBJECTID);            
			Information iprovider = findInformationProvider(uimanager);
			try {      	
				Node node = uimanager.categoriesService_.getTaxonomyNode(realPath) ;
				uimanager.getClipBoard().setType(ClipboardCommand.CUT) ;
				uimanager.getClipBoard().setSrcPath(node.getPath());								
				Object[] args = { node.getPath() };				
				iprovider.addMessage( new ExoFacesMessage(FacesMessage.SEVERITY_INFO,
              res.getString("UITaxonomyManager.msg.taxonomy-cut"), args));
			} catch (Exception e) {
				JCRExceptionManager.process(iprovider, e);
			}
		}
		
	}
	
	static public class PasteTaxonomyActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			ResourceBundle res = getApplicationResourceBundle() ;
			UITaxonomyManager uimanager = (UITaxonomyManager)event.getComponent() ;
			String realPath = event.getParameter(OBJECTID);            
			Information iprovider = findInformationProvider(uimanager);	      	      	      
			String type = uimanager.getClipBoard().getType();
			String srcPath = uimanager.getClipBoard().getSrcPath();      
			if(srcPath == null){
				iprovider.addMessage(new ExoFacesMessage(
						res.getString("UITaxonomyManager.msg.no-taxonomy-selected")));
				return ;
			}
			try {      	
				Node destNode = uimanager.categoriesService_.getTaxonomyNode(realPath) ;
				String destPath = destNode.getPath() + srcPath.substring(srcPath.lastIndexOf("/"));				
				uimanager.categoriesService_.moveTaxonomyNode(srcPath,destPath,type) ;												
				Object[] args = { srcPath, destPath };
				iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_INFO,
                           res.getString("UITaxonomyManager.msg.taxonomy-pasted"), args));
				
			}catch (ReferentialIntegrityException e) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR,
                       res.getString("UITaxonomyManager.msg.referential-integrity")));
      } catch (Exception e) {				
				JCRExceptionManager.process(iprovider, e);
			}
			uimanager.update() ;
			uimanager.setRenderedSibling(UITaxonomyManager.class) ;
		}
	}
	
	public static class BackActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
			UITaxonomyManager uiexplorer = (UITaxonomyManager)event.getComponent() ;
			uiexplorer.setRenderedSibling(UIECMAdminSummary.class) ;
		}		
	}
}
