/***************************************************************************
 * Copyright 2001-2007 The eXo Platform SAS         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.uploader.component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.Node;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.PropertyDefinition;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.cms.CMSUtils;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.jcr.component.ChangeNodeListener;
import org.exoplatform.portlets.content.jcr.component.UIJCRExplorer;
import org.exoplatform.portlets.content.jcr.component.UIViewController.CancelActionListener;
import org.exoplatform.services.cms.CmsService;
import org.exoplatform.services.jcr.core.nodetype.ExtendedNodeType;

/**
 * Created by The eXo Platform SAS
 * Author : Pham Xuan Hoa
 *          hoa.pham@exoplatform.com
 * May 10, 2007  
 */
public class UIContentUploader extends UIExoCommand implements ChangeNodeListener {

  private final static String SELECT_METATYPE_ACTION = "selectMetaType" ;
  public final static String METADATA_TYPES = "metadataTypes" ;
  public final static String ADD_METADATA_ACTION = "addMetadata".intern() ;  
  
  static Parameter[] CANCEL_PARAMS = { new Parameter(ACTION , CANCEL_ACTION)} ;
  static Parameter[] SAVE_PARAMS = { new Parameter(ACTION , SAVE_ACTION)} ;  
  static Parameter[] SELECT_PARAMS = { new Parameter(ACTION , SELECT_METATYPE_ACTION)} ;
  static Parameter[] REMOVE_PARAMS = { new Parameter(ACTION , REMOVE_ACTION)} ;
  static Parameter[] ADD_PARAMS = { new Parameter(ACTION , ADD_METADATA_ACTION)} ;

  private FileInfo fileInfo_ ;    
  protected Node contentNode_ ;

  protected List<String> selectedMetaTypes_ = new ArrayList<String>() ;
  protected HashMap<String,Map> metaPropertiesMap_ = new HashMap<String,Map>() ;

  public UIContentUploader() {    
    setId("contentUploader") ;
    setRendererType("VelocityRenderer");       

    addActionListener(SaveActionListener.class,SAVE_ACTION) ;
    addActionListener(AddActionListener.class,ADD_METADATA_ACTION) ;
    addActionListener(CancelActionListener.class,CANCEL_ACTION) ;
    addActionListener(RemoveActionListener.class,REMOVE_ACTION) ;
    addActionListener(SelectMetaTypeActionListener.class,SELECT_METATYPE_ACTION) ;
  }

  public Parameter[] getSaveParams() { return SAVE_PARAMS ; }
  public Parameter[] getCancelParams() { return CANCEL_PARAMS ; }  
  public Parameter[] getSelectParams() { return SELECT_PARAMS ; }
  public Parameter[] getAddParams() { return ADD_PARAMS ; }
  public Parameter[] getRemoveParams() { return REMOVE_PARAMS ; }

  public List<String> getSelectedMetaTypes() { return selectedMetaTypes_ ; }
  public void setSelectedMetaTypes(List<String> list) { this.selectedMetaTypes_ = list ; }
  
  public HashMap<String,Map> getMetaPropertiesMap() { return metaPropertiesMap_ ; }

  public String getMetaIcon(String metaTypeName) {
    return CMSUtils.getIconByNodeTypeName(metaTypeName) ;
  }

  public FileInfo getFileInfo() { return this.fileInfo_ ; }
  
  public String getFileIcon() {
    return CMSUtils.getFileIcon(fileInfo_.getFileName(),"nt:file",true) ;
  }  
  
  public Node getContentNode() { return this.contentNode_ ; }

  public void setSelectedMetaContent(String metaName,Map propertiesMap) {
    this.metaPropertiesMap_.put(metaName,propertiesMap) ;
  }      
  
  public void setUploadInfo( FileInfo fileInfo, Node contentNode) throws Exception {
    this.fileInfo_ = fileInfo ;
    this.contentNode_ = contentNode ;
    List<String> list = new ArrayList<String>() ;
    for(NodeType mixin:contentNode_.getMixinNodeTypes()) {
      if(mixin.isNodeType("exo:metadata") && isExternalUse(mixin)) {
        list.add(mixin.getName()) ;
      }
    }
    this.selectedMetaTypes_ = list ;

  }  

  private boolean isExternalUse(NodeType nodeType) throws Exception{
    PropertyDefinition def = 
      ((ExtendedNodeType)nodeType).getPropertyDefinitions("exo:internalUse").getAnyDefinition() ;    
    return !def.getDefaultValues()[0].getBoolean() ;
  }

  public void onChange(UIJCRExplorer uiExplorer, Node node) throws Exception {
    setRendered(false) ;
  }

  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIContentUploader uiForm = (UIContentUploader)event.getSource() ;                                    
      UIJCRExplorer uiExplorer = (UIJCRExplorer)uiForm.getAncestorOfType(UIJCRExplorer.class) ;
      Information iprovider = findInformationProvider(uiForm) ;
      ResourceBundle res = getApplicationResourceBundle() ;      
      CmsService cmsService = (CmsService)PortalContainer.getComponent(CmsService.class) ;
      HashMap<String,Map> metaPropertiesMap = uiForm.getMetaPropertiesMap() ;
      try {
        for(String metadata:uiForm.getSelectedMetaTypes()) {
          Map inputValues = metaPropertiesMap.get(metadata) ;                    
          cmsService.storeNodeContent(uiForm.contentNode_,metadata,inputValues) ;                    
        }
        uiForm.contentNode_.save() ;
        uiExplorer.getSession().save() ;
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_INFO,
            res.getString("UIContentUploader.msg.save-successfully"))) ;
        uiExplorer.setSelectNode(uiForm.contentNode_.getParent()) ;               
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider,e) ;
      }
      
    }
  }

  static public class AddActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIContentUploader uiComp = (UIContentUploader)event.getSource() ;
      UIExternalMetadataList metadataList = 
        (UIExternalMetadataList)uiComp.getSibling(UIExternalMetadataList.class) ;
      metadataList.update(uiComp.getSelectedMetaTypes(),uiComp.contentNode_) ;
      uiComp.setRenderedSibling(UIExternalMetadataList.class) ;
    }
  }   

  static public class RemoveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIContentUploader uiComp = (UIContentUploader)event.getSource() ;
      String metaName = event.getParameter(OBJECTID) ;      
      uiComp.getSelectedMetaTypes().remove(metaName) ;
    }
  }
  
  static public class SelectMetaTypeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIContentUploader uiComp = (UIContentUploader)event.getSource() ;     
      String selectedMetaName = event.getParameter(OBJECTID) ;
      UIMetadataDialogForm metaDialogForm = 
        (UIMetadataDialogForm)uiComp.getSibling(UIMetadataDialogForm.class) ;      
      metaDialogForm.setMetaTypeName(selectedMetaName) ;      
      if(uiComp.metaPropertiesMap_.containsKey(selectedMetaName)) {
        Map inputValues = uiComp.metaPropertiesMap_.get(selectedMetaName) ;
        CMSUtils.updateFormValues(metaDialogForm,inputValues) ;
      } 
      metaDialogForm.setNode(uiComp.contentNode_) ;
      uiComp.setRenderedSibling(UIMetadataDialogForm.class) ;
    }
  }    
}
