package org.exoplatform.portlets.content.uploader.component;

import java.io.ByteArrayInputStream;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Map;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.Node;

import org.exoplatform.commons.utils.IOUtil;
import org.exoplatform.commons.utils.MimeTypeResolver;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.cms.CMSUtils;
import org.exoplatform.faces.core.component.UISimpleMulltipartForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UIUploadInputStream;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.jcr.component.ChangeNodeListener;
import org.exoplatform.portlets.content.jcr.component.UIJCRExplorer;
import org.exoplatform.portlets.content.jcr.component.UIViewController.CancelActionListener;
import org.exoplatform.services.cms.CmsService;
import org.exoplatform.services.cms.JcrInputProperty;

public class UIUploadForm extends UISimpleMulltipartForm implements ChangeNodeListener {

  final public static String MIX_VERSION = "mix:versionable".intern() ; 
  final public static String NT_FILE = "nt:file".intern() ;
  final public static String NT_RESOURCE = "nt:resource".intern() ;
  final public static String JCR_CONTENT = "jcr:content".intern() ;
  final public static String JCR_DATA = "jcr:data".intern() ;
  final public static String JCR_MIMETYPE = "jcr:mimeType".intern() ;
  final public static String JCR_LASTMODIFIED = "jcr:lastModified".intern() ;

  final public static String UPLOAD_ACTION = "uploadAction".intern() ;        

  static Parameter[] CANCEL_PARAMS = { new Parameter(ACTION , CANCEL_ACTION)} ;  
  static Parameter[] UPLOAD_PARAMS = { new Parameter(ACTION , UPLOAD_ACTION)} ;

  private FileInfo fileInfo_ ;
  private byte[] fileData_ ;  
  private String fileIcon_ ="";
  private boolean success_ = false ;      
  private Node contentNode_ ; 

  public UIUploadForm() {
    super("uploadForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");     
    add(new UIUploadInputStream("upload")); 
    add( new UIStringInput("nodeName","")) ;

    addActionListener(UploadActionListener.class, UPLOAD_ACTION);
    addActionListener(SaveActionListener.class,SAVE_ACTION) ;
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
  }

  public Parameter[] getCancelParams() { return CANCEL_PARAMS ; }
 
  public String uploadAction() { return UPLOAD_ACTION ; }

  public FileInfo getFileInfo() { return fileInfo_ ; }
  public void setFileInfo(FileInfo info) { this.fileInfo_ = info ; }
  
  public byte[] getFileData() { return this.fileData_ ; }
  public void setFileData(byte[] data) { this.fileData_ = data ; }

  public boolean uploadSuccess() { return success_ ; }
  public void setUploadSuccess(boolean success) { this.success_ = success ; }      
     
  public Node getContentNode() { return contentNode_ ; }
  public void setContentNode(Node node) { this.contentNode_ = node ; }
  
  public String getFileIcon() { return fileIcon_ ; } 
  public void setFileIcon(String fileIcon) {
    this.fileIcon_= fileIcon ;
  }

  public void onChange(UIJCRExplorer explorer, Node node) {
    setRendered(false) ;
    setUploadSuccess(false) ;        
  }

  static public class UploadActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIUploadForm uiForm = (UIUploadForm) event.getSource();
      UIUploadInputStream input = (UIUploadInputStream) uiForm.getUIInput("upload");                        
      String fileName = input.getFileName();
      Information iprovider = findInformationProvider(uiForm) ;
      ResourceBundle res = getApplicationResourceBundle() ;
      if(fileName == null ||fileName.length() == 0) {        
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN,
            res.getString("UIForm.msg.non-upload"))) ;
        return ;
      }      
      MimeTypeResolver mimeTypeResolver = new MimeTypeResolver() ;
      String mimeType = mimeTypeResolver.getMimeType(fileName) ;            
      byte[] data = IOUtil.getStreamContentAsBytes(input.getInputStream()) ;
      String sSize = String.valueOf((((float)(data.length/100))/10));                      
      
      FileInfo fileInfo = new FileInfo() ;
      fileInfo.setFileName(fileName) ;
      fileInfo.setMimeType(mimeType) ;      
      fileInfo.setFileSize(sSize+" Kb") ;           
      
      uiForm.setFileIcon(CMSUtils.getFileIcon(fileName,NT_FILE,true)) ;
      uiForm.setFileInfo(fileInfo) ;  
      uiForm.setFileData(data) ;
      uiForm.setUploadSuccess(true) ; 
      uiForm.getUIStringInput("nodeName").setValue(fileName) ;
      Object[] arg = {fileName } ; 
      iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_INFO,
          res.getString("UIForm.msg.upload-success"),arg)) ;
    }    
  }

  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIUploadForm uiForm = (UIUploadForm) event.getSource();
      Information iprovider = findInformationProvider(uiForm) ;
      ResourceBundle res = getApplicationResourceBundle() ;
      String nodeName = uiForm.getUIStringInput("nodeName").getValue() ;
      if(nodeName == null || nodeName.length() == 0) {        
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN,
            res.getString("UIForm.msg.non-upload"))) ;
        return ;
      }
      uiForm.getFileInfo().setNodeName(nodeName) ;

      UIJCRExplorer uiExplorer = (UIJCRExplorer)uiForm.getAncestorOfType(UIJCRExplorer.class) ;
      Node selectedNode = uiExplorer.getSelectNode() ;           
      String mimeType = uiForm.getFileInfo().getMimeType() ;           
      boolean isExist = selectedNode.hasNode(nodeName) ;
      CmsService cmsService = (CmsService)PortalContainer.getComponent(CmsService.class) ;
      UIContentUploader uploader = (UIContentUploader)uiForm.getSibling(UIContentUploader.class) ;
      Object[] args = {uiForm.getFileInfo().getFileName(),nodeName} ;
      if(isExist) {
        Node node = selectedNode.getNode(nodeName) ;
        if(!node.isNodeType(NT_FILE)) {
          iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN,
              res.getString("UIContentUploader.msg.cannot-upload"))) ;
          return ;
        }         
        try{
          if(!node.isNodeType(MIX_VERSION)) {
            node.addMixin(MIX_VERSION) ;            
            node.save() ;            
            node.checkin() ;
            node.checkout() ;
          }         
          Node contentNode = node.getNode(JCR_CONTENT);
          contentNode.setProperty(JCR_DATA, new ByteArrayInputStream(uiForm.getFileData()));
          contentNode.setProperty(JCR_MIMETYPE, mimeType);
          contentNode.setProperty(JCR_LASTMODIFIED, new GregorianCalendar());          
          node.save() ;       
          node.checkin() ;
          node.checkout() ;          
          uiExplorer.getSession().save() ;                          
          iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_INFO,
              res.getString("UIForm.msg.save-success"),args)) ;
          uiForm.reset();
          uploader.setUploadInfo(uiForm.getFileInfo(),contentNode) ;          
          uiForm.setRenderedSibling(UIContentUploader.class) ;
          return;
        }catch (Exception e) {          
          JCRExceptionManager.process(iprovider, e);
          return ;
        }        
      }                

      Map<String,JcrInputProperty> inputProperties = new HashMap<String,JcrInputProperty>() ;
      JcrInputProperty nodeInput = new JcrInputProperty() ;
      nodeInput.setJcrPath("/node") ;
      nodeInput.setValue(nodeName) ;
      nodeInput.setMixintype("mix:i18n,mix:votable,mix:commentable") ;
      nodeInput.setType(JcrInputProperty.NODE) ;
      inputProperties.put("/node",nodeInput) ;

      JcrInputProperty jcrContent = new JcrInputProperty() ;
      jcrContent.setJcrPath("/node/jcr:content") ;
      jcrContent.setValue("") ;
      jcrContent.setNodetype(NT_RESOURCE) ;
      jcrContent.setType(JcrInputProperty.NODE) ;
      inputProperties.put("/node/jcr:content",jcrContent) ;

      JcrInputProperty jcrData = new JcrInputProperty() ;
      jcrData.setJcrPath("/node/jcr:content/jcr:data") ;          
      jcrData.setValue(uiForm.getFileData()) ;          
      inputProperties.put("/node/jcr:content/jcr:data",jcrData) ; 

      JcrInputProperty jcrMimeType = new JcrInputProperty() ;
      jcrMimeType.setJcrPath("/node/jcr:content/jcr:mimeType") ;
      jcrMimeType.setValue(mimeType) ;          
      inputProperties.put("/node/jcr:content/jcr:mimeType",jcrMimeType) ;

      JcrInputProperty jcrLastModified = new JcrInputProperty() ;
      jcrLastModified.setJcrPath("/node/jcr:content/jcr:lastModified") ;
      jcrLastModified.setValue(new GregorianCalendar()) ;
      inputProperties.put("/node/jcr:content/jcr:lastModified",jcrLastModified) ;

      JcrInputProperty jcrEncoding = new JcrInputProperty() ;
      jcrEncoding.setJcrPath("/node/jcr:content/jcr:encoding") ;
      jcrEncoding.setValue("UTF-8") ;
      inputProperties.put("/node/jcr:content/jcr:encoding",jcrEncoding) ;                            
      try{
        String path = cmsService.storeNode(NT_FILE,selectedNode,inputProperties,true) ;
        selectedNode.save() ;
        selectedNode.getSession().save() ;
        String newNodePath = path.substring(path.indexOf(":") + 1 ) ;
        Node newNode = (Node)uiExplorer.getSession().getItem(newNodePath) ;
        uiForm.setContentNode(newNode.getNode(JCR_CONTENT)) ;        
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_INFO,
            res.getString("UIForm.msg.save-success"),args)) ;
        uiForm.reset();
        uploader.setUploadInfo(uiForm.getFileInfo(),newNode.getNode(JCR_CONTENT)) ;        
        uiForm.setRenderedSibling(UIContentUploader.class) ;        
      }catch (Exception e) {        
        JCRExceptionManager.process(iprovider, e);
        return ;
      }                
    }            
  }      
}
