/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.watch.component;

import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.Node;

import org.exoplatform.container.SessionContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.jcr.component.ChangeNodeListener;
import org.exoplatform.portlets.content.jcr.component.UIJCRExplorer;
import org.exoplatform.services.cms.watch.WatchDocumentService;

/**
 * Created by The eXo Platform SARL
 * Author : Xuan Hoa Pham
 *          hoapham@exoplatform.com
 *          phamvuxuanhoa@gmail.com
 * Nov 30, 2006  
 */
public class UIWatchForm extends UISimpleForm implements ChangeNodeListener{

  final private static String WATCH_ACTION= "watchDocument".intern() ;
  final private static String BACK_FORM_ACTION = "back".intern() ;
  final static public String UNWATCH_ACTION = "unWatch".intern() ;  
  final static public String NOTIFICATION_BY_EMAIL = "Email".intern() ;
  final static public String NOTIFICATION_BY_RSS = "RSS".intern() ;
  final static public String NOTIFICATION_TYPE = "notificationType".intern() ;   

  protected UISelectBox uiNotifyTypeSelection_ ;  
  private Node document4Watcher_ ;

  public static Parameter[] WATCH_DOCUMENT_PARAMS = { new Parameter(ACTION , WATCH_ACTION)} ;
  public static Parameter[] BACK_FORM_PARAMS = { new Parameter(ACTION, BACK_ACTION)} ;
  public static Parameter[] UNWATCH_PARAMS = {new Parameter(ACTION, UNWATCH_ACTION) } ;

  private WatchDocumentService watchService_ ;
  private String userName_ ;
  private UIJCRExplorer jcrExplorer_ ;

  public UIWatchForm(WatchDocumentService watchService) throws Exception{
    super("UIWatchForm","post",null) ;
    watchService_ = watchService ;
    setRendererType("SimpleFormVelocityRenderer");

    uiNotifyTypeSelection_ = new UISelectBox(NOTIFICATION_TYPE,NOTIFICATION_BY_EMAIL,new ArrayList<SelectItem>()) ;            
    add(uiNotifyTypeSelection_) ;            
    userName_ = SessionContainer.getInstance().getRemoteUser() ;
    addActionListener(WatchDocumentActionListener.class,WATCH_ACTION) ;
    addActionListener(BackActionListener.class,BACK_FORM_ACTION) ;
    addActionListener(UnwatchActionListener.class,UNWATCH_ACTION) ;    
  }

  public void onChange(UIJCRExplorer uiExplorer, Node node) throws Exception {
    jcrExplorer_ = uiExplorer ;
    document4Watcher_ = node ;
  }

  public UIJCRExplorer getJcrExplorer() { return jcrExplorer_ ; }   

  public String getWatchAction() { return WATCH_ACTION ; }
  public String getUnWatchAction() {return UNWATCH_ACTION ; }
  public String getBackAction() {return BACK_FORM_ACTION ; }

  public WatchDocumentService  getWatchService() { return watchService_ ; }
  public Node getDocument4Watch() { return document4Watcher_ ; }
  public String getWatcherName() { return userName_ ; }

  public boolean isWatching() throws Exception{   
    List<SelectItem> options = new ArrayList<SelectItem>() ;    
    int notifyType = watchService_.getNotificationType(document4Watcher_,userName_) ;    
    if(notifyType == WatchDocumentService.FULL_NOTIFICATION) {
      options.add(new SelectItem(NOTIFICATION_BY_EMAIL,NOTIFICATION_BY_EMAIL)) ;
      uiNotifyTypeSelection_.setOptions(options) ;
      return true ;
    }else if(notifyType == WatchDocumentService.NOTIFICATION_BY_EMAIL ) {
      options.add(new SelectItem(NOTIFICATION_BY_EMAIL,NOTIFICATION_BY_EMAIL)) ;
      uiNotifyTypeSelection_.setOptions(options) ;
      return true ;
    }else if(notifyType == WatchDocumentService.NOTIFICATION_BY_RSS) {
      options.add(new SelectItem(NOTIFICATION_BY_RSS,NOTIFICATION_BY_RSS)) ;
      uiNotifyTypeSelection_.setOptions(options) ;
      return true ; 
    }else {
      options.add(new SelectItem(NOTIFICATION_BY_EMAIL,NOTIFICATION_BY_EMAIL)) ;
      uiNotifyTypeSelection_.setOptions(options) ;
      return false ;
    }          
  }

  public int getNotifyType() throws Exception {
    return watchService_.getNotificationType(document4Watcher_,userName_) ;
  }

  public void setWatchableDocument( Node documentNode) throws Exception{
    document4Watcher_ = documentNode ;    
  }    

  public static class WatchDocumentActionListener extends ExoActionListener {

    public void execute(ExoActionEvent event) throws Exception {
      UIWatchForm watchForm = (UIWatchForm)event.getSource() ;      
      String notifyType = 
        watchForm.getUISelectBox(UIWatchForm.NOTIFICATION_TYPE).getValue() ;
      Node documentNode = watchForm.getDocument4Watch() ;
      String watcherName = watchForm.getWatcherName() ;
      ResourceBundle res = getApplicationResourceBundle() ;
      Information iprovider = findInformationProvider(watchForm) ;
      WatchDocumentService watchService = watchForm.getWatchService() ;
      try{
        if(notifyType.equalsIgnoreCase(UIWatchForm.NOTIFICATION_BY_EMAIL)) {
          watchService.watchDocument(documentNode,watcherName,WatchDocumentService.NOTIFICATION_BY_EMAIL) ;
        }else if(notifyType.equals(UIWatchForm.NOTIFICATION_BY_RSS)) {
          watchService.watchDocument(documentNode,watcherName,WatchDocumentService.NOTIFICATION_BY_RSS) ;          
        }
        else throw new Exception("We don't support this type of notification. Please choose another type we offer to you.") ;               
      }catch (Exception e) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR,e.getLocalizedMessage())) ;
      }
      iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_INFO,res.getString("UIWatchForm.msg.watching-successfully"))) ;
      UIJCRExplorer explorer = watchForm.getJcrExplorer() ;
      explorer.broadcastOnChange(explorer.getSelectNode()) ;
      watchForm.setRendered(false) ;
    }

  }

  public static class BackActionListener  extends ExoActionListener {    
    public void execute(ExoActionEvent event) throws Exception {     
      UIWatchForm watchForm = (UIWatchForm)event.getSource() ;             
      UIJCRExplorer explorer = watchForm.getJcrExplorer() ;
      explorer.broadcastOnChange(explorer.getSelectNode()) ;  
      watchForm.setRendered(false) ;
    }   
  }

  public static class UnwatchActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIWatchForm watchForm = (UIWatchForm)event.getSource() ;
      WatchDocumentService watchService = watchForm.getWatchService() ;
      Node documentNode = watchForm.getDocument4Watch() ;
      String watcherName = watchForm.getWatcherName() ;
      ResourceBundle res = getApplicationResourceBundle() ;
      Information iprovider = findInformationProvider(watchForm) ;
      String notifyType = 
        watchForm.getUISelectBox(UIWatchForm.NOTIFICATION_TYPE).getValue() ;

      try {
        if(notifyType.equalsIgnoreCase(UIWatchForm.NOTIFICATION_BY_EMAIL)) {
          watchService.unwatchDocument(documentNode,watcherName,WatchDocumentService.NOTIFICATION_BY_EMAIL) ;
        }else if(notifyType.equals(UIWatchForm.NOTIFICATION_BY_RSS)) {
          watchService.unwatchDocument(documentNode,watcherName,WatchDocumentService.NOTIFICATION_BY_RSS) ;          
        }
        else throw new Exception("We don't support this type of notification. Please choose another type we offer to you.") ;               
      }catch (Exception e) {
        JCRExceptionManager.process(iprovider,e) ;
      }
      iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_INFO,
          res.getString("UIWatchForm.msg.unwatching-successfully"))) ;      
      UIJCRExplorer explorer = watchForm.getJcrExplorer() ;
      explorer.broadcastOnChange(explorer.getSelectNode()) ;  
      watchForm.setRendered(false) ;
    }

  }  
}
