/***************************************************************************
 * Copyright 2001-2007 The eXo Platform SAS         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.cms;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.jcr.Session;
import javax.jcr.Workspace;

import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.core.XASession;
import org.exoplatform.services.jcr.impl.core.XASessionImpl;

/**
 * Created by The eXo Platform SAS
 * Author : Pham Xuan Hoa
 *          hoa.pham@exoplatform.com
 * May 31, 2007  
 */
public class JCRSessionManager {
  private RepositoryService repositoryService_ ;  
  private Map<String, List<Session>> applicationSessions ;

  public JCRSessionManager(RepositoryService service) {    
    repositoryService_ = service ;    
    applicationSessions = new Hashtable<String, List<Session>> () ;
  }

  public Session openSystemSession(String workspaceName, String appSessionId) throws Exception {    
    List<Session>  sessions = applicationSessions.get(appSessionId) ;
    if(sessions == null) {
      sessions = new ArrayList<Session>() ;
      applicationSessions.put(appSessionId, sessions) ;
    }
    Session session = repositoryService_.getRepository().getSystemSession(workspaceName) ;
    sessions.add(session) ;
    return session ;
  }

  public Session openSession(String workspaceName, String appSessionId) throws Exception {    
    List<Session>  sessions = applicationSessions.get(appSessionId) ;
    if(sessions == null) {
      sessions = new ArrayList<Session>() ;
      applicationSessions.put(appSessionId, sessions) ;
    }
    Session session = repositoryService_.getRepository().login(workspaceName) ;
    sessions.add(session) ;
    return session ;
  }    

  public void openSession(Workspace ws) throws Exception {    
    ws.getSession() ;
  }

  public void expireApplicationSession(String sessionId) {
    List<Session>  sessions = applicationSessions.remove(sessionId) ;
    if(sessions != null) {
      for(Session session : sessions) {
        session.logout() ;
      } 
    }        
  }
}
