/**
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.core;

import java.security.AccessControlException;
import java.util.Map;

import javax.jcr.Node;
import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.access.AccessControlList;

/**
 * Created by The eXo Platform SARL        .<br/>
 * The extension for JSR-170 standard Node interface
 * @author Gennady Azarenkov
 * @version $Id: ExtendedNode.java 9819 2006-10-26 17:44:01Z geaz $
 */

public interface ExtendedNode extends Node {

  /**
   * Sets permission
   * @param permissions
   * @throws RepositoryException
   * @throws AccessControlException
   */
  void setPermissions(Map permissions) throws RepositoryException, AccessControlException;
  
  /**
   * @deprecated since 1.1 use setPermissions(Map permissions) instead 
   */
  void setChildrenPermissions(Map permissions) throws RepositoryException, AccessControlException;

  /**
   * @return Access Control List
   */
  AccessControlList getACL();
  
  /**
   * @deprecated since 1.1 use getACL() instead 
   */
  AccessControlList getChildrenACL();

  /**
   * Clears Access Control List
   * @throws RepositoryException
   * @throws AccessControlException
   */
  void clearACL() throws RepositoryException, AccessControlException;

  /**
   * @deprecated since 1.1 use clearACL() instead 
   */
  void clearChildrenACL() throws RepositoryException, AccessControlException;

  /**
   * Removes permissions for perticular identity
   * @param identity
   * @throws RepositoryException
   * @throws AccessControlException
   */
  void removePermission(String identity) throws RepositoryException, AccessControlException;

  /**
   * @deprecated since 1.1 use removePermission(String identity) instead 
   */
  void removeChildrenPermission(String identity) throws RepositoryException, AccessControlException;

  /**
   * Sets permissions for particular identity
   * @param identity
   * @param permission
   * @throws RepositoryException
   * @throws AccessControlException
   */
  void setPermission(String identity, String[] permission) throws RepositoryException, AccessControlException;

  /**
   * @deprecated since 1.1 use setPermission(String identity, String[] permission) instead 
   */
  void setChildrenPermission(String identity, String[] permission) throws RepositoryException, AccessControlException;

  /**
   * Checks if there are permission to perform some actions
   * @param actions
   * @throws AccessControlException if no such permissions found
   * @throws RepositoryException
   */
  void checkPermission(String actions) throws AccessControlException, RepositoryException;

  /**
   * @deprecated since 1.1 use checkPermission(String actions) instead 
   */
  void checkChildrenPermission(String actions) throws AccessControlException, RepositoryException; 

}