/**
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.core;

import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.ItemData;

/**
 * Created by The eXo Platform SARL        .
 * @author Gennady Azarenkov
 * @version $Id: ItemState.java 7271 2006-07-21 11:58:01Z peterit $
 */
public class ItemState {
  
  public static final int ADDED = 1;

  public static final int UPDATED = 2;

  public static final int DELETED = 3;

  public static final int UNCHANGED = 4;
  
  public static final int MIXIN_CHANGED = 8;
  
  /**
   * @deprecated
   */
  public static final int AUTO_ADDED = 5;
  
  /**
   * @deprecated
   */
  public static final int AUTO_DELETED = 6;

  protected final ItemData data;
  
  protected final int state;
  
  protected boolean eventFire;
  /**
   * for Session.move()
   */
  private transient InternalQPath ancestorToSave;


  /**
   * The constructor
   * @param data underlying data
   * @param state
   * @param eventFire - if the state cause some event firing
   * @param ancestorToSave - path of item which should be called in save (usually for session.move())
   */
  public ItemState(ItemData data, int state, boolean eventFire, InternalQPath ancestorToSave) {
    this.data = data;
    this.state = state;
    this.eventFire = eventFire;
    if(ancestorToSave == null)
      this.ancestorToSave = data.getQPath();
    else
      this.ancestorToSave = ancestorToSave;

  }

  /**
   * @deprecated
   */
  public ItemState(ItemData data, int state) {
    this.data = data;
    this.state = state;  
  }


  /**
   * @return data.
   */
  public ItemData getData() {
    return data;
  }
  
  /**
   * @return state.
   */
  public int getState() {
    return state;
  }
  
  public boolean isNode() {
    return data.isNode();
  }
  
  public boolean isAdded() {
    return (state == ADDED);
  }

  public boolean isUpdated() {
    return (state == UPDATED);
  }

  public boolean isDeleted() {
    return (state == DELETED);
  }

  public boolean isUnchanged() {
    return (state == UNCHANGED);
  }
  
  public boolean isMixinChanged() {
    return (state == MIXIN_CHANGED);
  }
  
  public boolean isEventFire() {
    return eventFire;
  }

  public InternalQPath getAncestorToSave() {
    return ancestorToSave;
  }
  
  public boolean equals(Object obj) {
    if(this == obj)
      return true;
    
    if (obj instanceof ItemState) {
      ItemState other = (ItemState) obj;
      return other.getData().equals(data) && 
        other.getState() == state;
    }
    
    return false;

  }
  


// USE THESE STATIC METHODS INSTEAD OF DEPRECATED ONES!!!!
  /**
   * creates ADDED item state
   * shortcut for new ItemState(data, ADDED, true, true, null)
   * @param data
   * @param needValidation
   * @return
   */
  public static ItemState createAddedState(ItemData data) {
    return new ItemState(data, ADDED, true, null); 
  }

  /**
   * creates UPDATED item state
   * shortcut for new ItemState(data, UPDATED, true, true, null)
   * @param data
   * @param needValidation
   * @return
   */
  public static ItemState createUpdatedState(ItemData data) {
    return new ItemState(data, UPDATED, true, null); 
  }

  /**
   * creates DELETED item state
   * shortcut for new ItemState(data, DELETED, true, true, null)
   * @param data
   * @param needValidation
   * @return
   */
  public static ItemState createDeletedState(ItemData data) {
    return new ItemState(data, DELETED, true, null); 
  }

  /**
   * creates UNCHANGED item state
   * shortcut for new ItemState(data, UNCHANGED, false, false, null)
   * @param data
   * @param needValidation
   * @return
   */
  public static ItemState createUnchangedState(ItemData data) {
    return new ItemState(data, UNCHANGED, false, null); 
  }

  public static String getStateName(int state) {
    if (state == ADDED)
      return "ADDED";
    else if (state == UPDATED)
      return "UPDATED";
    else if (state == DELETED)
      return "DELETED";
    else if (state == UNCHANGED)
      return "UNCHANGED";
    else if (state == MIXIN_CHANGED)
      return "MIXIN CHANGED";
//    else if (state == AUTO_DELETED)
//      return "AUTO DELETED";
    else
      return "UNDEFINED";
  }
  
  

  /**
   * @deprecated
   */
  public static ItemState createAdded(ItemData data) {
    return new ItemState(data, ItemChangeState.ADDED); 
  }

  /**
   * @deprecated
   */
  public static ItemState createUpdated(ItemData data) {
    return new ItemState(data, ItemChangeState.UPDATED); 
  }

  /**
   * @deprecated
   */
  public static ItemState createDeleted(ItemData data) {
    return new ItemState(data, ItemChangeState.DELETED); 
  }
  

}
