package org.exoplatform.services.jcr.impl.core.itemvisitors;

import java.util.List;

import javax.jcr.ItemExistsException;
import javax.jcr.ItemNotFoundException;
import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.impl.core.NodeImpl;

/**
 * Created by The eXo Platform SARL .<br>
 * Clone accepted node to parent:
 *   - just copy if not referenceable or referenceable and not found
 *   - if referenceable: add to list for delete or throw and add this state
 *        
 * 
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov
 *         </a>
 * @version $Id: NodeCloneVisitor.java 7488 2006-08-01 15:14:59Z peterit $
 */

public class NodeCloneVisitor extends NodeCopyVisitor {

  private boolean removeExisting;
  protected Session destSession;
  protected List nodesToRemove;

  public NodeCloneVisitor(NodeImpl context, boolean removeExisting, List nodesToRemove) {
    super(context, false, false);
    this.destSession = context.getSession();
    this.removeExisting = removeExisting;
    this.nodesToRemove = nodesToRemove;
  }
  
  protected void entering(Node node, int level) throws RepositoryException {
    
    if (log.isDebugEnabled())
      log.debug("Clone node: " + node.getPath() + ", level: " + level);
    
    //checkExistence(node);
    if(level == 0)
      return;

    //String name = node.getName();
    NodeImpl nodeImpl = (NodeImpl) node;
    NodeImpl clonedNode;
    if (node.isNodeType("mix:referenceable")) {
      String uuid = node.getUUID();
      try {
        Node relNode = destSession.getNodeByUUID(uuid);
        if(!removeExisting) {
          throw new RepositoryException(new ItemExistsException(
              "Item exists uuid = " + uuid + " at "
                  + destSession.getWorkspace().getName()));
        } else {
          this.nodesToRemove.add(relNode);
        }
      } catch (ItemNotFoundException e) { // save UUID (i.e. clone)
      }

      clonedNode = curParent().copyChildNodeInmemory(
          nodeImpl.getActualItemData().getQName(),
          (NodeData)nodeImpl.getActualItemData(), 
          node.isNodeType("mix:referenceable"),
          false, false, false, false);
      
      parents.push(clonedNode);
      if (log.isDebugEnabled())
        log.debug("Clone node: [" + node.getSession().getWorkspace().getName() + "] " + node.getPath() 
            + " --> [" + destSession.getWorkspace().getName() + "] " + clonedNode.getPath());
    } else {  // is not referenceable - just copy
      super.entering(node, level);
    }
  }
}
  
  
