/**
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.dataflow.persistent;

import java.util.List;

import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.access.AccessControlList;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.ItemData;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.storage.WorkspaceDataContainer;

/**
 * Created by The eXo Platform SARL        .
 * Data Manager supported ACL Inheritance
 * @author Gennady Azarenkov
 * @version $Id: $
 */

public class ACLInheritanceSupportedWorkspaceDataManager extends CacheableWorkspaceDataManager {

  /**
   * @param dataContainer
   * @param cache
   */
  public ACLInheritanceSupportedWorkspaceDataManager(WorkspaceDataContainer dataContainer, WorkspaceStorageCacheImpl cache) {
    super(dataContainer, cache);
  }

  /* 
   * Guaranteed ACL  
   * @see org.exoplatform.services.jcr.impl.dataflow.persistent.CacheableWorkspaceDataManager#getACL(org.exoplatform.services.jcr.datamodel.InternalQPath)
   */
  public AccessControlList getACL(InternalQPath qpath) throws RepositoryException {
    AccessControlList acl = super.getACL(qpath);
    if(acl == null) {
      NodeData data = getNearestACAncestor(qpath);
      if(data == null)
        throw new RepositoryException("FATAL: Node Data not found for ACL:  "+qpath);
      acl = data.getACL(); 
    }
    return acl;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.dataflow.persistent.CacheableWorkspaceDataManager#getChildNodesData(org.exoplatform.services.jcr.datamodel.NodeData)
   */
  public List<NodeData> getChildNodesData(NodeData nodeData) throws RepositoryException {
    List<NodeData> nodes = super.getChildNodesData(nodeData);
    for(NodeData node:nodes) 
      initACL(node);
    return nodes;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.dataflow.persistent.CacheableWorkspaceDataManager#getItemData(org.exoplatform.services.jcr.datamodel.InternalQPath)
   */
  public ItemData getItemData(InternalQPath qpath) throws RepositoryException {
    return initACL(super.getItemData(qpath));
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.dataflow.persistent.CacheableWorkspaceDataManager#getItemData(java.lang.String)
   */
  public ItemData getItemData(String uuid) throws RepositoryException {
    return initACL(super.getItemData(uuid));
  }

  /**
   * finds the nearest access controllable ancestor
   * @param qpath
   * @return access controllable node data or root node data(anyway)
   * @throws RepositoryException
   */
  private NodeData getNearestACAncestor(InternalQPath qpath) throws RepositoryException {
    ItemData item = super.getItemData(qpath);
    if (item == null || !item.isNode())
      return getNearestACAncestor(qpath.makeParentPath());
    else if (((NodeData) item).getACL() != null) 
//    root or AccessControllable node (root always has ACL)
      return (NodeData) item;
    else
      // Not AccessControllable nor Root node
      return getNearestACAncestor(qpath.makeParentPath());
  }
  
  private ItemData initACL(ItemData data) throws RepositoryException {
    if(data != null && data.isNode() && ((NodeData)data).getACL() == null) {
      ((NodeData)data).setACL(super.getACL(data.getQPath().makeParentPath()));
    }
    return data;
  }
}
