/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl.dataflow.session;

import java.util.ArrayList;
import java.util.List;

import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.ItemData;
import org.exoplatform.services.jcr.impl.core.ItemState;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.dataflow.ItemDataChangesLog;
import org.exoplatform.services.jcr.impl.dataflow.MutableItemDataChangesLog;
import org.exoplatform.services.jcr.impl.dataflow.TransientItemData;

/**
 * Created by The eXo Platform SARL        .<br/>
 * 
 * Responsible for managing session changes log.
 * Relying on fact that ItemData inside ItemState SHOULD be TransientItemData  
 *
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: SessionChangesLog.java 7657 2006-08-07 13:23:11Z peterit $
 */
 public final class SessionChangesLog extends MutableItemDataChangesLog {
   
  //private transient Map <InternalQPath, ItemState>lastStates = new HashMap<InternalQPath, ItemState>();
   
  public SessionChangesLog(SessionImpl session) {
    super(session);
  }

  public SessionChangesLog(String sessionId) {
    super(sessionId);
  }
  

  /**
   * Removes the item at the rootPath and all descendants from the log
   * @param changes
   */
  public void remove(InternalQPath rootPath) {
    List <ItemState> tmpList = new ArrayList <ItemState> ();
    tmpList.addAll(items);
    for(int i=0; i<tmpList.size(); i++) {
      TransientItemData item = (TransientItemData)tmpList.get(i).getData();
      InternalQPath qPath = item.getQPath(); 
      if(qPath.equals(rootPath) || qPath.isDescendantOf(rootPath, false)) {
        items.remove(i);
      }
    }
  }

  /**
   * @param rootPath
   * @return item state at the rootPath and its descendants
   */
  public List <ItemState> getChanges(InternalQPath rootPath) {
    List <ItemState> list = new ArrayList <ItemState> ();
    for(int i=0; i<items.size(); i++) {
      TransientItemData item = (TransientItemData)items.get(i).getData();
      if(item.getQPath().equals(rootPath) || 
          item.getQPath().isDescendantOf(rootPath, false))
        list.add(items.get(i));
    }
    return list;
  }

  
  /**
   * creates new changes log with rootPath and its descendants of 
   * this one and removes those entries  
   * @param rootPath
   * @return ItemDataChangesLog
   */
  public ItemDataChangesLog pullOutLog(InternalQPath rootPath) {
    SessionChangesLog cLog = new SessionChangesLog(sessionId);
    cLog.addAll(getChanges(rootPath));
    remove(rootPath);
    return cLog;
  }
  

  /**
   * @param qpath 
   * @param forNodes retrieves nodes' ItemStates is true, or properties' otherwice
   * @return child item states
   */
  public List <ItemState> getLastStates(InternalQPath qpath, boolean forNodes) {
    List <ItemState> children = new ArrayList<ItemState>();
    List <ItemState> changes = getChanges(qpath);
    for (ItemState child : changes) {
      ItemData data = child.getData();
      // try to remove the same item to keep the latest state only
      // TODO optimize this code
      for (int i = 0; i < children.size(); i++) {
        ItemData resData = children.get(i).getData();
        if (data.getQPath().equals(resData.getQPath())) 
          children.remove(i);
      }
      
      // add state to result 
      if (data.isNode() == forNodes && !data.getQPath().equals(qpath))
        children.add(child);
      
    }
    return children;
  }


}
