/**
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.access;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import javax.jcr.RepositoryException;

import org.apache.commons.logging.Log;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.datamodel.ValueData;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SARL .<br/>
 * Access Control List
 * 
 * @author Gennady Azarenkov
 * @version $Id: AccessControlList.java 9660 2006-10-21 16:14:41Z peterit $
 */

public class AccessControlList {

  private Log log = ExoLogger.getLogger("jcr.AccessControlList");
  
  public static final String DELIMITER = " ";
  
  private String owner;
  
  private List <AccessControlEntry> aces; 

  //private Map permissions;

  /**
   * @deprecated use AccessControlList()
   * @param initDefault
   */
  public AccessControlList(boolean initDefault) {
    this();
//    this.owner = SystemIdentity.SYSTEM;
//    this.permissions = new HashMap();
////    if(initDefault)
//    permissions.put(SystemIdentity.ANY, PermissionType.ALL);
  }

  public AccessControlList() {
    this(SystemIdentity.SYSTEM);
  }

  /**
   * default ACL owned by ownerName
   * @param ownerName
   */
  public AccessControlList(String ownerName) {
    this.owner = ownerName;
    //this.permissions = new HashMap();
    this.aces = new ArrayList<AccessControlEntry>();
    for(String str : PermissionType.ALL) {
      aces.add(new AccessControlEntry(SystemIdentity.ANY, str));
    }
    //permissions.put(SystemIdentity.ANY, PermissionType.ALL);
  }

  public AccessControlList(String owner, List <AccessControlEntry> aces) {
    this.owner = owner;
    this.aces = aces; 
  }


  public AccessControlList(PropertyData ownerProp, PropertyData permissionsProp)
      throws RepositoryException {

    try {
      if(ownerProp != null) // if property already initialized in storage
        this.owner = new String(ownerProp.getValues().get(0).getAsByteArray());
      
      if (permissionsProp != null) {
        //permissions = new HashMap();
        this.aces = new ArrayList<AccessControlEntry>();
        List<ValueData> permValues = permissionsProp.getValues();
  
        for (int i = 0; i < permValues.size(); i++) {
          String p = new String(permValues.get(i).getAsByteArray());
          StringTokenizer parser = new StringTokenizer(p, DELIMITER);
          aces.add(new AccessControlEntry(parser.nextToken(), parser.nextToken()));
//          addPermission(parser.nextToken(), parser.nextToken());
        }
      }
    } catch(IOException e) {
      log.error("Error of AccessControlList create: " + e.getMessage(), e);
    }
  }
  
  public void addPermissions(String identity, String[] perm) {
    for(String p: perm) {
      aces.add(new AccessControlEntry(identity, p));
    }
  }
  
  public void removePermissions(String identity) {
    List <AccessControlEntry> aces4Del = new ArrayList <AccessControlEntry>(); 
    for(AccessControlEntry a: aces) {
      if(a.getIdentity().equals(identity))
        aces4Del.add(a);
    }
    for(AccessControlEntry del : aces4Del) 
      aces.remove(del);
  }

  /**
   * @return Returns the owner.
   */
  public String getOwner() {
    return owner;
  }
  
  public List <AccessControlEntry> getPermissionEntries() {
    return aces;
  }

  
  public boolean equals(Object obj) {
    if(obj == this)
      return true;
    if(obj instanceof AccessControlList) {
      AccessControlList another = (AccessControlList) obj;
      return dump().equals(another.dump());
    }
    return false;
  }
  
  public String dump() {
    String res = "OWNER: "+owner+"\n";
    for(AccessControlEntry a: aces) {
      res+=a.getAsString()+"\n";
    }
    return res;
  }
}