/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.dataflow;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

import org.exoplatform.services.jcr.datamodel.IllegalPathException;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.ItemData;
import org.exoplatform.services.jcr.datamodel.MutableItemData;
import org.exoplatform.services.jcr.impl.Constants;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: TransientItemData.java 5307 2006-04-27 13:55:57Z peterit $
 */
public abstract class TransientItemData implements MutableItemData, Externalizable {

  protected InternalQPath QPath;

  protected String        UUID;

  protected String        parentUUID;

  protected int           persistedVersion;

  /**
   * @param path QPath
   * @param uuid id
   * @param version persisted version
   * @param parentUUID parentId
   */
  TransientItemData(InternalQPath path, String uuid, int version, String parentUUID) {
    this.parentUUID = parentUUID;
    this.QPath = path;
    this.UUID = uuid;
    this.persistedVersion = version;
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.exoplatform.services.jcr.datamodel.ItemData#getQPath()
   */
  public InternalQPath getQPath() {
    return QPath;
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.exoplatform.services.jcr.datamodel.ItemData#getUUID()
   */
  public String getUUID() {
    return UUID;
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.exoplatform.services.jcr.datamodel.ItemData#getPersistedVersion()
   */
  public int getPersistedVersion() {
    return persistedVersion;
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.exoplatform.services.jcr.datamodel.ItemData#getParentUUID()
   */
  public String getParentUUID() {
    return parentUUID;
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.exoplatform.services.jcr.datamodel.MutableItemData#increasePersistedVersion()
   */
  public void increasePersistedVersion() {
    this.persistedVersion++;
  }

  /*
   * (non-Javadoc)
   * 
   * @see java.lang.Object#equals(java.lang.Object)
   */
  @Override
  public boolean equals(Object obj) {
    if (obj instanceof ItemData) {
      return getUUID().equals(((ItemData) obj).getUUID());
    } else {
      return false;
    }
  }

  /**
   * @return Qname - shortcut for getQPath().getName();
   */
  public InternalQName getQName() {
    return QPath.getName();
  }

// Need for serializable
// ----------- [ BEGIN ] -----------
  TransientItemData() {
  }

  public void writeExternal(ObjectOutput out) throws IOException {
//    System.out.println("-->TransientItemData--> writeExternal(ObjectOutput out)");

    out.writeInt(QPath.getAsString().getBytes().length);
    out.write(QPath.getAsString().getBytes());

    out.writeInt(UUID.getBytes().length);
    out.write(UUID.getBytes());

    out.writeInt(parentUUID.getBytes().length);
    out.write(parentUUID.getBytes());

    out.writeInt(persistedVersion);
  }

  public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
//    System.out.println("-->TransientItemData--> readExternal(ObjectInput in)");
    
    byte[] buf;

    try {
      buf = new byte[in.readInt()];
      in.read(buf);
      String sQPath = new String(buf, Constants.DEFAULT_ENCODING);
      QPath = InternalQPath.parse(sQPath);
    } catch (IllegalPathException e) {
      e.printStackTrace();
    }

    buf = new byte[in.readInt()/*UUIDGenerator.UUID_LENGTH*/];
    in.read(buf);
    UUID = new String(buf , Constants.DEFAULT_ENCODING);

    buf = new byte[in.readInt()];
    in.read(buf);
    parentUUID = new String(buf, Constants.DEFAULT_ENCODING);

    persistedVersion = in.readInt();
  }
  // ----------- [ END ] -----------

}