/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl.storage.jdbc;

import java.io.File;
import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import javax.jcr.PropertyType;
import javax.sql.DataSource;

import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.datamodel.ValueData;
import org.exoplatform.services.jcr.impl.util.io.FileCleaner;
import org.exoplatform.services.jcr.storage.value.ValueStoragePluginProvider;
import org.exoplatform.services.jcr.util.io.BLOBUtil;

/**
 * Created by The eXo Platform SARL
 * 27.04.2006
 * 
 * @author <a href="mailto:peter.nedonosko@exoplatform.com.ua">Peter Nedonosko</a>
 * @version $Id: SingleDbJDBCConnection.java 9852 2006-10-28 11:34:29Z peterit $
 */
public class SingleDbJDBCConnection extends JDBCStorageConnection {

  
  SingleDbJDBCConnection(DataSource dataSource,
      String containerName, boolean multiDb,
      ValueStoragePluginProvider valueStorageProvider,
      int maxBufferSize, File swapDirectory, FileCleaner swapCleaner) throws SQLException {
  
    super(dataSource, containerName, false, valueStorageProvider, 
        maxBufferSize, swapDirectory, swapCleaner);
  }
  
  protected String getInternalId(final String uuid) {
    return containerName + uuid;
  }

  protected String getUuid(final String internalId) {
    
//    System.out.println("--------------->>>>> "+internalId+" "+containerName);
    if(internalId == null) // possible for root parent
      return null;
    else
      return internalId.substring(containerName.length());
  }   

  
  /**
   * Prepared queries at start time
   * @throws SQLException
   */
  @Override
  protected final void prepareQueries() throws SQLException {
    JCR_FK_NODE_PARENT = "JCR_FK_SNODE_PARENT";
    JCR_FK_NODE_ITEM = "JCR_FK_SNODE_ITEM";
    JCR_FK_PROPERTY_NODE = "JCR_FK_SPROPERTY_N";
    JCR_FK_PROPERTY_ITEM = "JCR_FK_SPROPERTY_I";
    JCR_FK_VALUE_PROPERTY = "JCR_FK_SVALUE_PROP";
    JCR_PK_ITEM = "JCR_SITEM_PKEY";
    
    FIND_ITEM_BY_ID = "select I.*, N.ID as NID, N.ORDER_NUM as NORDER_NUM, N.PARENT_ID as NPARENT_ID, P.ID as PID, P.TYPE as PTYPE, P.PARENT_ID as PPARENT_ID, P.MULTIVALUED as PMULTIVALUED" 
      + " from JCR_SITEM I LEFT JOIN JCR_SNODE N ON I.ID=N.ID LEFT JOIN JCR_SPROPERTY P ON I.ID=P.ID"
      + " where I.ID=? and I.CONTAINER_NAME=?";

    FIND_ITEM_BY_PATH = "select I.*, N.ID as NID, N.ORDER_NUM as NORDER_NUM, N.PARENT_ID as NPARENT_ID, P.ID as PID, P.TYPE as PTYPE, P.PARENT_ID as PPARENT_ID, P.MULTIVALUED as PMULTIVALUED" 
      + " from JCR_SITEM I LEFT JOIN JCR_SNODE N ON I.ID=N.ID LEFT JOIN JCR_SPROPERTY P ON I.ID=P.ID"
      + " where I.PATH=? and I.CONTAINER_NAME=? order by I.VERSION DESC";
    
    FIND_CHILDITEM_BY_PATH = "select I.*, N.ID as NID, N.ORDER_NUM as NORDER_NUM, N.PARENT_ID as NPARENT_ID, P.ID as PID, P.TYPE as PTYPE, P.PARENT_ID as PPARENT_ID, P.MULTIVALUED as PMULTIVALUED" 
      + " from JCR_SITEM I left join JCR_SNODE N on (I.ID=N.ID and N.PARENT_ID=?) left join JCR_SPROPERTY P on (I.ID=P.ID and P.PARENT_ID=?)"
      + " where I.PATH=? and I.CONTAINER_NAME=? order by I.VERSION DESC";
    
    FIND_CHILD_PROPERTY_BY_PATH = "select I.*, P.ID as PID, P.TYPE as PTYPE, P.PARENT_ID as PPARENT_ID, P.MULTIVALUED as PMULTIVALUED" 
      + " from JCR_SITEM I, JCR_SPROPERTY P"
      + " where I.CONTAINER_NAME=? and I.PATH=? and I.ID=P.ID and P.PARENT_ID=? order by I.VERSION DESC";

    FIND_DESCENDANT_NODES_LIKE_PATH = "select I.*, N.ID as NID" 
      + " from JCR_SITEM I, JCR_SNODE N"
      + " where I.CONTAINER_NAME=? and I.ID=N.ID and N.PARENT_ID=? and I.PATH like ?"
      + " order by I.PATH";
    
    FIND_DESCENDANT_PROPERTIES_LIKE_PATH = "select I.*, P.ID as PID" 
      + " from JCR_SITEM I, JCR_SPROPERTY P"
      + " where I.CONTAINER_NAME=? and I.ID=P.ID and P.PARENT_ID=? and I.PATH like ?"
      + " order by I.PATH";    
    
    FIND_REFERENCES = "select R.NODE_ID as NID, R.PROPERTY_ID as PID, I.PATH, I.VERSION, P.TYPE as PTYPE, P.PARENT_ID as PPARENT_ID, P.MULTIVALUED as PMULTIVALUED" +
    " from JCR_SREF R, JCR_SITEM I, JCR_SPROPERTY P" +
    " where R.PROPERTY_ID=I.ID and R.PROPERTY_ID=P.ID and I.ID=P.ID and R.NODE_ID=? and I.CONTAINER_NAME=?";  

    FIND_REFERENCEABLE = "select R.NODE_ID as NID, R.PROPERTY_ID as PID, I.PATH, I.VERSION, P.TYPE as PTYPE, P.PARENT_ID as PPARENT_ID, P.MULTIVALUED as PMULTIVALUED" +
    " from JCR_SREF R, JCR_SITEM I, JCR_SPROPERTY P" +
    " where R.PROPERTY_ID=I.ID and R.PROPERTY_ID=P.ID and I.ID=P.ID and R.PROPERTY_ID=? and I.CONTAINER_NAME=?";
    
    FIND_VALUES_BY_PROPERTYID = "select * from JCR_SVALUE where PROPERTY_ID=? order by ORDER_NUM";
    FIND_VALUE_BY_PROPERTYID_OREDERNUMB = "select DATA from JCR_SVALUE where PROPERTY_ID=? and ORDER_NUM=?";

    FIND_NODES_BY_PARENTID = "select I.*, N.ID as NID, N.ORDER_NUM as NORDER_NUM, N.PARENT_ID as NPARENT_ID" 
      + " from JCR_SITEM I, JCR_SNODE N"
      + " where I.ID=N.ID and N.PARENT_ID=?" 
      //+ " order by I.ID";
      // [PN] 26.12.06
      //+ " order by I.ID"; 
      + " order by NORDER_NUM"; 
    
    FIND_NODES_IDS_BY_PARENTID = "select I.ID" 
      + " from JCR_SITEM I, JCR_SNODE N"
      + " where I.ID=N.ID and N.PARENT_ID=?"
      + " group by I.ID"
      + " order by I.ID";
    
    FIND_PROPERTIES_BY_PARENTID = "select I.*, P.ID as PID, P.TYPE as PTYPE, P.PARENT_ID as PPARENT_ID, P.MULTIVALUED as PMULTIVALUED" 
      + " from JCR_SITEM I, JCR_SPROPERTY P"
      + " where I.ID=P.ID and P.PARENT_ID=?" 
      + " order by I.ID";
    
    FIND_NODESCOUNT_BY_PARENTID = "select count(*) from JCR_SNODE where PARENT_ID=?";
    FIND_PROPERTIESCOUNT_BY_PARENTID = "select count(*) from JCR_SPROPERTY where PARENT_ID=?";
    
    INSERT_ITEM = "insert into JCR_SITEM(ID, PATH, VERSION, CONTAINER_NAME) VALUES(?,?,?,?)";
    INSERT_NODE = "insert into JCR_SNODE(ID, ORDER_NUM, PARENT_ID) VALUES(?,?,?)";
    INSERT_PROPERTY = "insert into JCR_SPROPERTY(ID, TYPE, MULTIVALUED, PARENT_ID) VALUES(?,?,?,?)";
    INSERT_VALUE = "insert into JCR_SVALUE(DATA, ORDER_NUM, PROPERTY_ID) VALUES(?,?,?)";
    INSERT_REF = "insert into JCR_SREF(NODE_ID, PROPERTY_ID, ORDER_NUM) VALUES(?,?,?)";

    UPDATE_ITEM = "update JCR_SITEM set VERSION=? where ID=? and CONTAINER_NAME=?";
    UPDATE_ITEM_PATH = "update JCR_SITEM set PATH=?, VERSION=? where ID=? and CONTAINER_NAME=?";
    UPDATE_NODE = "update JCR_SNODE set ORDER_NUM=? where ID=?";
    UPDATE_PROPERTY = "update JCR_SPROPERTY set TYPE=? where ID=?";
    
    DELETE_ITEM = "delete from JCR_SITEM where ID=?";
    //DELETE_ITEM = "delete from JCR_SITEM where ID=? and CONTAINER_NAME=?";
    DELETE_NODE = "delete from JCR_SNODE where ID=?";
    DELETE_PROPERTY = "delete from JCR_SPROPERTY where ID=?";
    DELETE_VALUE = "delete from JCR_SVALUE where PROPERTY_ID=?";
    //DELETE_REF = "delete from JCR_SREF where NODE_ID=? and PROPERTY_ID=? and ORDER_NUM=?";
    DELETE_REF = "delete from JCR_SREF where PROPERTY_ID=?";
  }

  @Override
  protected void addNodeRecord(NodeData data) throws SQLException {
    
    String uuid = getInternalId(data.getUUID());
    
    insertItem.setString(1, uuid);
    insertItem.setString(2, data.getQPath().getAsString());
    insertItem.setInt(3, data.getPersistedVersion());
    insertItem.setString(4, containerName);
    insertItem.executeUpdate();

    insertNode.setString(1, uuid);
    insertNode.setInt(2, data.getOrderNumber());
    insertNode.setString(3, data.getParentUUID() != null ? getInternalId(data.getParentUUID()) : null);
    insertNode.executeUpdate();    
  }

  @Override
  protected void addPropertyRecord(PropertyData data) throws SQLException {
    
    String uuid = getInternalId(data.getUUID());
    
    insertItem.setString(1, uuid);
    insertItem.setString(2, data.getQPath().getAsString());
    insertItem.setInt(3, data.getPersistedVersion());
    insertItem.setString(4, containerName);
    insertItem.executeUpdate();

    insertProperty.setString(1, uuid);
    insertProperty.setInt(2, data.getType());
    insertProperty.setBoolean(3, data.isMultiValued());
    insertProperty.setString(4, getInternalId(data.getParentUUID()));
    insertProperty.executeUpdate();
  }

  /**
   * For REFERENCE properties only
   */
  @Override
  protected void addReference(PropertyData data) throws SQLException, IOException {
    List<ValueData> values = data.getValues();
    for (int i=0; i<values.size(); i++) {
      ValueData vdata = values.get(i);
      String refNodeUuid = new String(BLOBUtil.readValue(vdata));
      insertReference.setString(1, getInternalId(refNodeUuid));
      insertReference.setString(2, getInternalId(data.getUUID()));
      insertReference.setInt(3, i);
      insertReference.executeUpdate();
    }
  }  
  
  /**
   * For REFERENCE properties only
   */
  @Override
  protected void deleteReference(String propertyCid) throws SQLException {
//    List<ValueData> values = data.getValues();
//    for (int i=0; i<values.size(); i++) {
//      ValueData vdata = values.get(i); 
//      String refNodeUuid = new String(
//          vdata.isByteArray() ? vdata.getAsByteArray() : BLOBUtil.readStream(vdata.getAsStream())
//        );
//      deleteRef.setString(1, refNodeUuid);
//      deleteRef.setString(2, data.getUUID());
//      deleteRef.setInt(3, i);
//      deleteRef.executeUpdate();
//    }
    deleteReference.setString(1, propertyCid);
    deleteReference.executeUpdate();    
  }

  @Override
  protected int deleteItemByUUID(String cid) throws SQLException {
    deleteItem.setString(1, cid);
    //deleteItem.setString(2, containerName);
    return deleteItem.executeUpdate();
  }

  @Override
  protected int deleteNodeByUUID(String cid) throws SQLException {
    deleteNode.setString(1, cid);
    return deleteNode.executeUpdate();
  }

  @Override
  protected int deletePropertyByUUID(String cid) throws SQLException {
    deleteProperty.setString(1, cid);
    return deleteProperty.executeUpdate();
  }

  @Override
  protected ResultSet findChildNodesByParentUUID(String parentCid) throws SQLException {
    findNodesByParentId.setString(1, parentCid);
    return findNodesByParentId.executeQuery();
  }
  
  @Override
  protected ResultSet findChildNodesIdsByParentUUID(String parentCid) throws SQLException {
    findNodesIdsByParentId.setString(1, parentCid);
    return findNodesIdsByParentId.executeQuery();
  }

  @Override
  protected ResultSet findChildPropertiesByParentUUID(String parentCid) throws SQLException {
    findPropertiesByParentId.setString(1, parentCid);
    return findPropertiesByParentId.executeQuery();
  }

  @Override
  protected ResultSet findItemByPath(String path) throws SQLException {
    findItemByPath.setString(1, path);
    findItemByPath.setString(2, containerName);
    return findItemByPath.executeQuery();
  }

  @Override
  protected ResultSet findItemByPath(String parentCid, String path) throws SQLException {

    findChildItemByPath.setString(1, parentCid);
    findChildItemByPath.setString(2, parentCid);
    findChildItemByPath.setString(3, path);
    findChildItemByPath.setString(4, containerName);
    return findChildItemByPath.executeQuery();
  }
  
  @Override
  protected ResultSet findPropertyByPath(String parentId, String path) throws SQLException {
    findChildPropertyByPath.setString(1, containerName);
    findChildPropertyByPath.setString(2, path);
    findChildPropertyByPath.setString(3, parentId);
    return findChildPropertyByPath.executeQuery();
  }  

  @Override
  protected ResultSet findItemByUUID(String cid) throws SQLException {
    findItemById.setString(1, cid);
    findItemById.setString(2, containerName);
    return findItemById.executeQuery();
  }

  protected ResultSet findDescendantNodes(String parentId, String parentPath) throws SQLException {
    findDescendantNodes.setString(1, containerName);
    findDescendantNodes.setString(2, parentId);
    findDescendantNodes.setString(3, parentPath + "%");
    return findDescendantNodes.executeQuery();
  }
  
  protected ResultSet findDescendantProperties(String parentId, String parentPath) throws SQLException {
    findDescendantProperties.setString(1, containerName);    
    findDescendantProperties.setString(2, parentId);
    findDescendantProperties.setString(3, parentPath + "%");
    return findDescendantProperties.executeQuery();
  }
  
  @Override
  protected ResultSet findReferences(String cid) throws SQLException {
    findReferences.setString(1, cid);
    findReferences.setString(2, containerName);
    return findReferences.executeQuery();
  }
  
  @Override
  protected ResultSet findReferenceable(String propertyCid) throws SQLException {
    findReferenceable.setString(1, propertyCid);
    findReferenceable.setString(2, containerName);
    return findReferenceable.executeQuery();
  }

  @Override
  protected ResultSet getChildNodesCountByParentUUID(String parentCid) throws SQLException {
    findNodesCountByParentId.setString(1, parentCid);
    return findNodesCountByParentId.executeQuery();
  }

  @Override
  protected ResultSet getChildPropertiesCountByParentUUID(String parentCid) throws SQLException {
    findPropertiesCountByParentId.setString(1, parentCid);    
    return findPropertiesCountByParentId.executeQuery();
  }

  @Override
  protected int updateItemVersionByUUID(int versionValue, String cid) throws SQLException {
    updateItem.setInt(1, versionValue);
    updateItem.setString(2, cid);
    updateItem.setString(3, containerName);
    return updateItem.executeUpdate();
  }
  
  @Override
  protected int updateItemPathByUUID(String qpath, int version, String cid) throws SQLException {
    updateItemPath.setString(1, qpath);
    updateItemPath.setInt(2, version);
    updateItemPath.setString(3, cid);
    updateItemPath.setString(4, containerName);
    return updateItemPath.executeUpdate();
  }    
  
  @Override
  protected int updateNodeOrderNumbByUUID(int orderNumb, String cid) throws SQLException {
    updateNode.setInt(1, orderNumb);
    updateNode.setString(2, cid);
    return updateNode.executeUpdate();
  }
  
  @Override
  protected int updatePropertyTypeByUUID(int type, String cid) throws SQLException {
    updateProperty.setInt(1, type);
    updateProperty.setString(2, cid);
    return updateProperty.executeUpdate();
  }
}
