/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.api.writing;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.nodetype.NodeType;

import org.exoplatform.services.jcr.JcrAPIBaseTest;

/**
 * Created by The eXo Platform SARL
 *
 * 27.12.2006
 *
 * @author <a href="mailto:peter.nedonosko@exoplatform.com.ua">Peter Nedonosko</a>
 * @version $Id: TestOrderBefore.java 11662 2007-01-06 02:46:01Z peterit $
 */
public class TestOrderBefore extends JcrAPIBaseTest {

  protected final String TEST_ROOT = "order_test";
  
  private Node testBase = null;

  
  @Override
  public void setUp() throws Exception {
    super.setUp();
    
    testBase = root.addNode(TEST_ROOT);
    root.save();
  }
  
  private void initSimpleCase() throws Exception {
    testBase.addNode("n1");
    testBase.addNode("n2");
    testBase.addNode("n3");
    testBase.addNode("n4");
    
    root.save();
  }
  
  private void initSNSCase1() throws Exception {
    testBase.addNode("n1");
    testBase.addNode("n2");
    testBase.addNode("n2");
    testBase.addNode("n3");
    
    root.save();
  }
  
  private void initSNSCase2() throws Exception {
    testBase.addNode("n1");
    testBase.addNode("n1");
    Node n3 = testBase.addNode("n1");
    n3.addMixin("mix:referenceable");
    testBase.addNode("n1");
    
    root.save();
  }
  
  private void initSNSCase3() throws Exception {
    testBase.addNode("n1");
    testBase.addNode("n1");
    Node n3 = testBase.addNode("n1");
    n3.addMixin("mix:referenceable");
    testBase.addNode("n1");
    testBase.addNode("n2");
    
    root.save();
  }
  
  private void initSNSCase4() throws Exception {
    testBase.addNode("n1");
    testBase.addNode("n1");
    Node n21 = testBase.addNode("n2");
    n21.addMixin("mix:referenceable");
    testBase.addNode("n2");
    Node n13 = testBase.addNode("n1");
    n13.addMixin("mix:referenceable");
    testBase.addNode("n3");
    
    root.save();
  }

  @Override
  protected void tearDown() throws Exception {
    try {
      testBase.remove();
      root.save();
    } catch(Throwable e) {
      log.error("TEAR DOWN ERROR. " + getName() + ". " + e.getMessage(), e);
    }
    
    super.tearDown();
  }
  
  /**
   * nodes and positions must be of same length
   * @param nodes 
   */
  private void checkOrder(Node testRoot, String[] nodes) throws Exception {

    log.info(">>>> CHECK ORDER >>>>");
    
    NodeIterator childs = testRoot.getNodes();
    int orderPos = -1;
    while (childs.hasNext()) {
      orderPos++;
      Node next = childs.nextNode();
      String nodeName = nodes[orderPos];
      if (!next.getPath().endsWith(nodeName)) {
        String failMsg = "Nodes order is invalid. Expected: " + nodeName + ". Found: " + next.getPath() + ". Position: " + orderPos;
        log.info(failMsg);
        fail(failMsg);
      }      
      String mixins = "";
      for (NodeType nt: next.getMixinNodeTypes()) {
        mixins += nt.getName() + " ";
      }
      mixins = mixins.trim();
      log.info(">> " + next.getPath() + ", " + next.getPrimaryNodeType().getName() + " " + mixins);
    }
    log.info("<<<< CHECK ORDER <<<<");
  }
  
  /**
   * nodes and positions must be of same length
   * @param nodes 
   */
  private void checkOrderAnotherSession(String[] nodes) throws Exception {

    Session newSession = repository.login(session.getCredentials(), session.getWorkspace().getName());
    Node testRoot = newSession.getRootNode().getNode(TEST_ROOT);
    checkOrder(testRoot, nodes);
    newSession.logout();
  }
  
  /**
   * nodes and positions must be of same length
   * @param nodes 
   */
  private void checkOrder(String[] nodes) throws Exception {

    checkOrder(testBase, nodes);
  }
  
  // -------------- Tests ---------------
  
  // -------------- Simple use-case: child nodes n1, n2, n3, n4  ---------------
  
  public void testOrderUp() throws Exception {
    initSimpleCase();
    
    testBase.orderBefore("n4", "n3");
    
    String[] order = new String[] {"n1", "n2", "n4", "n3"};
    
    checkOrder(order);
    
    testBase.save();
    
    checkOrderAnotherSession(order);
  }
  
  public void testOrderBegin() throws Exception {
    initSimpleCase();
    
    testBase.orderBefore("n3", "n1");
    
    String[] order = new String[] {"n3", "n1", "n2", "n4"};
    
    checkOrder(order);
    
    testBase.save();
    
    checkOrderAnotherSession(order);
  }
  
  public void testOrderDown() throws Exception {
    initSimpleCase();
    
    testBase.orderBefore("n2", "n4");
    
    String[] order = new String[] {"n1", "n3", "n2", "n4"};
    
    checkOrder(order);
    
    testBase.save();
    
    checkOrderAnotherSession(order);
  } 

  public void testOrderEnd() throws Exception {
    initSimpleCase();
    
    testBase.orderBefore("n2", null);
    
    String[] order = new String[] {"n1", "n3", "n4", "n2"};
    
    checkOrder(order);
    
    testBase.save();
    
    checkOrderAnotherSession(order);
  }

  // ----- Same-name sibling use-case: child nodes n1, n2[1], n2[2], n3  -----
  
  public void testOrderUp_SNS1() throws Exception {
    initSNSCase1();
    
    testBase.orderBefore("n3", "n2");
    
    String[] order = new String[] {"n1", "n3", "n2", "n2[2]"};
    
    checkOrder(order);
    
    testBase.save();
    
    checkOrderAnotherSession(order);
  }
  
  public void testOrderBegin_SNS1() throws Exception {
    initSNSCase1();
    
    testBase.orderBefore("n2[2]", "n1");
    
    String[] order = new String[] {"n2", "n1", "n2[2]", "n3"};
    
    checkOrder(order);
    
    testBase.save();
    
    checkOrderAnotherSession(order);
  }
  
  public void testOrderDown_SNS1() throws Exception {
    initSNSCase1();
    
    testBase.orderBefore("n1", "n2[2]");
    
    String[] order = new String[] {"n2", "n1", "n2[2]", "n3"};
    
    checkOrder(order);
    
    testBase.save();
    
    checkOrderAnotherSession(order);
  } 
  
  /**
   * A childs order map is unchanged after the order
   */
  public void testOrderDown1_SNS1() throws Exception {
    initSNSCase1();
    
    testBase.orderBefore("n2", "n3");
    
    String[] order = new String[] {"n1", "n2", "n2[2]", "n3"};
    
    checkOrder(order);
    
    testBase.save();
    
    checkOrderAnotherSession(order);
  }

  public void testOrderEnd_SNS1() throws Exception {
    initSNSCase1();
    
    testBase.orderBefore("n2[2]", null);
    
    String[] order = new String[] {"n1", "n2", "n3", "n2[2]"};
    
    checkOrder(order);
    
    testBase.save();
    
    checkOrderAnotherSession(order);
  }

  // ----- Same-name sibling use-case: child nodes n1, n1[2], n1[3], n1[4]. n1[3] - mix:referenceable  -----
  
  public void testOrderBegin_SNS2() throws Exception {
    initSNSCase2();
    
    String n3uuid = testBase.getNode("n1[3]").getUUID(); 
    
    testBase.orderBefore("n1[3]", "n1");
    
    String[] order = new String[] {"n1", "n1[2]", "n1[3]", "n1[4]"}; // n1[3] -> n1 
    
    checkOrder(order);
    
    try {
      String n1uuid = testBase.getNode("n1").getUUID();
      assertEquals("A UUIDs must be equals after order node to another position", n3uuid, n1uuid);
    } catch (RepositoryException e) {
      fail("A node is not mix:referenceable after order to another position, " + testBase.getNode("n1").getPath() + ". " + e);
    }
    
    testBase.save();
    
    checkOrderAnotherSession(order);
    
    try {
      String n1uuid = testBase.getNode("n1").getUUID();
      assertEquals("A UUIDs must be equals after order node to another position", n3uuid, n1uuid);
    } catch (RepositoryException e) {
      fail("A node is not mix:referenceable after order to another position, " + testBase.getNode("n1").getPath() + ". " + e);
    }
  }
  
  public void testOrderUp_SNS2() throws Exception {
    initSNSCase2();
    
    String n3uuid = testBase.getNode("n1[3]").getUUID(); 
    
    testBase.orderBefore("n1[4]", "n1[2]");
    
    String[] order = new String[] {"n1", "n1[2]", "n1[3]", "n1[4]"}; // n1[3] -> n1[4]  
    
    checkOrder(order);
    
    try {
      String n4uuid = testBase.getNode("n1[4]").getUUID(); 
      assertEquals("A UUIDs must be equals after order node to another position", n3uuid, n4uuid);
    } catch (RepositoryException e) {
      fail("A node is not mix:referenceable after order to another position, " + testBase.getNode("n1[4]").getPath() + ". " + e);
    }
    
    testBase.save();
    
    checkOrderAnotherSession(order);
    
    try {
      String n4uuid = testBase.getNode("n1[4]").getUUID();
      assertEquals("A UUIDs must be equals after order node to another position", n3uuid, n4uuid);
    } catch (RepositoryException e) {
      fail("A node is not mix:referenceable after order to another position, " + testBase.getNode("n1[4]").getPath() + ". " + e);
    }
  }  
  
  public void testOrderDown_SNS2() throws Exception {
    initSNSCase2();
    
    String n3uuid = testBase.getNode("n1[3]").getUUID();
    
    testBase.orderBefore("n1", "n1[3]");
    
    String[] order = new String[] {"n1", "n1[2]", "n1[3]", "n1[4]"}; // n1[3] is unchanged in location
    
    checkOrder(order);
    
    try {
      String n3uuid_same = testBase.getNode("n1[3]").getUUID();
      assertEquals("A UUIDs must be unchanged after order an other node", n3uuid, n3uuid_same);
    } catch (RepositoryException e) {
      fail("A node is not mix:referenceable after order of an other node, " + testBase.getNode("n1[3]").getPath() + ". " + e);
    }
    
    testBase.save();
    
    checkOrderAnotherSession(order);
    
    try {
      String n3uuid_same = testBase.getNode("n1[3]").getUUID();
      assertEquals("A UUIDs must be equals after order node to another position", n3uuid, n3uuid_same);
    } catch (RepositoryException e) {
      fail("A node is not mix:referenceable after order to another position, " + testBase.getNode("n1[3]").getPath() + ". " + e);
    }
  } 
  
  /**
   * A childs order map is unchanged after the order
   */
  public void testOrderDown1_SNS2() throws Exception {
    initSNSCase2();
    
    String n3uuid = testBase.getNode("n1[3]").getUUID();
    
    testBase.orderBefore("n1[2]", "n1[4]");
    
    String[] order = new String[] {"n1", "n1[2]", "n1[3]", "n1[4]"}; // n1[3] -> n1[2]
    
    checkOrder(order);
    
    try {
      String n2uuid = testBase.getNode("n1[2]").getUUID();
      assertEquals("A UUIDs must be equals after order node to another position", n3uuid, n2uuid);
    } catch (RepositoryException e) {
      fail("A node is not mix:referenceable after order to another position, " + testBase.getNode("n1[2]").getPath() + ". " + e);
    }
    
    testBase.save();
    
    checkOrderAnotherSession(order);
    
    try {
      String n2uuid = testBase.getNode("n1[2]").getUUID();
      assertEquals("A UUIDs must be equals after order node to another position", n3uuid, n2uuid);
    } catch (RepositoryException e) {
      fail("A node is not mix:referenceable after order to another position, " + testBase.getNode("n1[2]").getPath() + ". " + e);
    }
  }

  public void testOrderEnd_SNS2() throws Exception {
    initSNSCase2();
    
    String n3uuid = testBase.getNode("n1[3]").getUUID();
    
    testBase.orderBefore("n1[2]", null);
    
    String[] order = new String[] {"n1", "n1[2]", "n1[3]", "n1[4]"}; // n1[3] -> n1[2]
    
    checkOrder(order);
    
    try {
      String n2uuid = testBase.getNode("n1[2]").getUUID();
      assertEquals("A UUIDs must be equals after order node to another position", n3uuid, n2uuid);
    } catch (RepositoryException e) {
      fail("A node is not mix:referenceable after order to another position, " + testBase.getNode("n1[2]").getPath() + ". " + e);
    }
    
    testBase.save();
    
    checkOrderAnotherSession(order);
    
    try {
      String n2uuid = testBase.getNode("n1[2]").getUUID();
      assertEquals("A UUIDs must be equals after order node to another position", n3uuid, n2uuid);
    } catch (RepositoryException e) {
      fail("A node is not mix:referenceable after order to another position, " + testBase.getNode("n1[2]").getPath() + ". " + e);
    }
  }
  
// ----- Same-name sibling use-case: child nodes n1, n1[2], n1[3], n1[4], n2. n1[3] - mix:referenceable  -----
  
  public void testOrderBegin_SNS3() throws Exception {
    initSNSCase3();
    
    String n3uuid = testBase.getNode("n1[3]").getUUID(); 
    
    testBase.orderBefore("n2", "n1");
    
    String[] order = new String[] {"n2", "n1", "n1[2]", "n1[3]", "n1[4]"}; // n1[3] unchanged 
    
    checkOrder(order);
    
    try {
      String n3uuid_same = testBase.getNode("n1[3]").getUUID();
      assertEquals("A UUIDs must be unchanged after order an other node", n3uuid, n3uuid_same);
    } catch (RepositoryException e) {
      fail("A node is not mix:referenceable after order of an other node, " + testBase.getNode("n1[3]").getPath() + ". " + e);
    }
    
    testBase.save();
    
    checkOrderAnotherSession(order);
    
    try {
      String n3uuid_same = testBase.getNode("n1[3]").getUUID();
      assertEquals("A UUIDs must be unchanged after order an other node", n3uuid, n3uuid_same);
    } catch (RepositoryException e) {
      fail("A node is not mix:referenceable after order of an other node, " + testBase.getNode("n1[3]").getPath() + ". " + e);
    }
  }
  
  public void testOrderUp_SNS3() throws Exception {
    initSNSCase3();
    
    String n3uuid = testBase.getNode("n1[3]").getUUID(); 
    
    testBase.orderBefore("n2", "n1[3]");
    
    String[] order = new String[] {"n1", "n1[2]", "n2", "n1[3]", "n1[4]"}; // n1[3] unchanged  
    
    checkOrder(order);
    
    try {
      String n3uuid_same = testBase.getNode("n1[3]").getUUID();
      assertEquals("A UUIDs must be unchanged after order an other node", n3uuid, n3uuid_same);
    } catch (RepositoryException e) {
      fail("A node is not mix:referenceable after order of an other node, " + testBase.getNode("n1[3]").getPath() + ". " + e);
    }
    
    testBase.save();
    
    checkOrderAnotherSession(order);
    
    try {
      String n3uuid_same = testBase.getNode("n1[3]").getUUID();
      assertEquals("A UUIDs must be unchanged after order an other node", n3uuid, n3uuid_same);
    } catch (RepositoryException e) {
      fail("A node is not mix:referenceable after order of an other node, " + testBase.getNode("n1[3]").getPath() + ". " + e);
    }
  }  
  
  public void testOrderDown_SNS3() throws Exception {
    initSNSCase3();
    
    String n3uuid = testBase.getNode("n1[3]").getUUID();
    
    testBase.orderBefore("n1", "n2");
    
    String[] order = new String[] {"n1", "n1[2]", "n1[3]", "n1[4]", "n2"}; // n1[3] -> n1[2]
    
    checkOrder(order);
    
    try {
      String n2uuid = testBase.getNode("n1[2]").getUUID();
      assertEquals("A UUIDs must be equals after order node to another position", n3uuid, n2uuid);
    } catch (RepositoryException e) {
      fail("A node is not mix:referenceable after order to another position, " + testBase.getNode("n1[2]").getPath() + ". " + e);
    }
    
    testBase.save();
    
    checkOrderAnotherSession(order);
    
    try {
      String n2uuid = testBase.getNode("n1[2]").getUUID();
      assertEquals("A UUIDs must be equals after order node to another position", n3uuid, n2uuid);
    } catch (RepositoryException e) {
      fail("A node is not mix:referenceable after order to another position, " + testBase.getNode("n1[2]").getPath() + ". " + e);
    }
  } 
  
  /**
   * A childs order map is unchanged after the order
   */
  public void testOrderDown1_SNS3() throws Exception {
    initSNSCase3();
    
    String n3uuid = testBase.getNode("n1[3]").getUUID();
    
    testBase.orderBefore("n1[2]", "n1[4]");
    
    String[] order = new String[] {"n1", "n1[2]", "n1[3]", "n1[4]", "n2"}; // n1[3] -> n1[2]
    
    checkOrder(order);
    
    try {
      String n2uuid = testBase.getNode("n1[2]").getUUID();
      assertEquals("A UUIDs must be equals after order node to another position", n3uuid, n2uuid);
    } catch (RepositoryException e) {
      fail("A node is not mix:referenceable after order to another position, " + testBase.getNode("n1[2]").getPath() + ". " + e);
    }
    
    testBase.save();
    
    checkOrderAnotherSession(order);
    
    try {
      String n2uuid = testBase.getNode("n1[2]").getUUID();
      assertEquals("A UUIDs must be equals after order node to another position", n3uuid, n2uuid);
    } catch (RepositoryException e) {
      fail("A node is not mix:referenceable after order to another position, " + testBase.getNode("n1[2]").getPath() + ". " + e);
    }
  }

  public void testOrderEnd_SNS3() throws Exception {
    initSNSCase3();
    
    String n3uuid = testBase.getNode("n1[3]").getUUID();
    
    testBase.orderBefore("n1[3]", null);
    
    String[] order = new String[] {"n1", "n1[2]", "n1[3]", "n2", "n1[4]"}; // n1[3] -> n1[4]
    
    checkOrder(order);
    
    try {
      String n4uuid = testBase.getNode("n1[4]").getUUID();
      assertEquals("A UUIDs must be equals after order node to another position", n3uuid, n4uuid);
    } catch (RepositoryException e) {
      fail("A node is not mix:referenceable after order to another position, " + testBase.getNode("n1[4]").getPath() + ". " + e);
    }
    
    testBase.save();
    
    checkOrderAnotherSession(order);
    
    try {
      String n4uuid = testBase.getNode("n1[4]").getUUID();
      assertEquals("A UUIDs must be equals after order node to another position", n3uuid, n4uuid);
    } catch (RepositoryException e) {
      fail("A node is not mix:referenceable after order to another position, " + testBase.getNode("n1[4]").getPath() + ". " + e);
    }
  }  
}
