/**
* Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
* Please look at license.txt in info directory for more license detail.   *
*/

package org.exoplatform.services.webdav;

import java.util.HashMap;
import java.util.Iterator;

import javax.jcr.Credentials;
import javax.jcr.LoginException;
import javax.jcr.Repository;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.SimpleCredentials;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.logging.Log;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SARL
 * Author : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 * @version $Id: WebDavSessionProvider.java 13162 2007-03-06 01:07:49Z brice $
 */

public class WebDavSessionProvider {
  
  private static Log log = ExoLogger.getLogger("jcr.WebDavSessionProvider");
  
  private WebDavService davService = null;
  private Repository repository = null;
  
  private HashMap<String, Session> openedSessions = new HashMap<String, Session>();
  
  public WebDavSessionProvider(WebDavService davService, Repository repository) {
    this.davService = davService;
    this.repository = repository;
  }  
    
  public Session getSession(String authHeader, String workspace) throws RepositoryException {
    if (openedSessions.containsKey(workspace)) {
      return openedSessions.get(workspace);
    }
    
    Credentials credentials = null;    
    String decodedAuth = "";
    
    if (authHeader != null) {
      try {
        String []basic = authHeader.split(" ");
        if (basic.length >= 2 && basic[0].equalsIgnoreCase(HttpServletRequest.BASIC_AUTH)) {
          decodedAuth = new String(Base64.decodeBase64(basic[1].getBytes()));
        }        
      } catch (Exception exc) {
        throw new LoginException();
      }
    } else {
      decodedAuth = davService.getConfig().getDefIdentity();
      if (decodedAuth == null) {
        throw new LoginException();
      }
    }
    
    String []authParams = decodedAuth.split(":", 2);
    credentials = new SimpleCredentials(authParams[0], authParams[1].toCharArray());    
    Session session = repository.login(credentials, workspace);
    openedSessions.put(workspace, session);
    return session;
  }
  
  public void logOutAllSessions() throws RepositoryException {
    Iterator<String> workspaceIter = openedSessions.keySet().iterator();
    while (workspaceIter.hasNext()) {
      String curWorkspaceName = workspaceIter.next();      
      Session session = openedSessions.get(curWorkspaceName);
      session.logout();
    }
  }
  
//  private void test() {
//    try {
//      
//      String lockToken = "";
//      
//      {
//        log.info(">>> login");
//        Credentials credentials = new SimpleCredentials("admin", "admin".toCharArray());
//        Session session = repository.login(credentials, "production");
//        
//        Node root1 = session.getRootNode();
//        Node fileNode = root1.addNode("test file 1", DavConst.NodeTypes.NT_FILE);
//        Node contentNode = fileNode.addNode(DavConst.NodeTypes.JCR_CONTENT, DavConst.NodeTypes.NT_RESOURCE);
//        
//        contentNode.setProperty(DavConst.NodeTypes.JCR_MIMETYPE, "text/xml");
//        contentNode.setProperty(DavConst.NodeTypes.JCR_LASTMODIFIED, Calendar.getInstance());
//        contentNode.setProperty(DavConst.NodeTypes.JCR_DATA, new ByteArrayInputStream("TEST FILE CONTENT".getBytes()));
//        
//        session.save();
//        
//        log.info(">>> SAved");
//        
//        fileNode.addMixin(DavConst.NodeTypes.MIX_LOCKABLE);
//        session.save();
//        
//        log.info(">>> SAved2");
//        
//        Lock lock = fileNode.lock(true, false);
//        lockToken = lock.getLockToken();
//        
//        log.info(">>> locked");
//        
//        session.logout();
//        log.info(">>> logout");
//      }
//
//      log.info("LOCKTOKEN: [" + lockToken + "]");
//      
//      {
//        log.info(">>> login");
//        Credentials credentials = new SimpleCredentials("admin", "admin".toCharArray());
//        Session session = repository.login(credentials, "production");
//        
//        session.addLockToken(lockToken);
//        
//        log.info(">>> added lock tokens");
//        
//        Node root1 = session.getRootNode();
//        Node fileNode = root1.getNode("test file 1");
//        Node contentNode = fileNode.getNode(DavConst.NodeTypes.JCR_CONTENT);
//        
//        log.info(">>> test node: " + contentNode);
//        
//        contentNode.setProperty(DavConst.NodeTypes.JCR_MIMETYPE, "text/plain");
//        //contentNode.setProperty(DavConst.NodeTypes.JCR_LASTMODIFIED, Calendar.getInstance());
//        //contentNode.setProperty(DavConst.NodeTypes.JCR_DATA, new ByteArrayInputStream("TEST FILE CONTENT".getBytes()));
//        
//        session.save();
//        
//        log.info("maybe OK");
//      }
//      
//      
//    } catch (Exception exc) {
//      log.info("Unhandled exception. " + exc.getMessage(), exc);
//    }
//  }

}
