/**
 * Copyright (c) 2001-2006 eXo Platform SARL, http://www.exoplatform.com/
 *
 * All rights reserved. Please look at license.txt in info directory for
 * more license detail.
 *
 * -------------------------------------------------------------------------
 * $Id: FileLoggerFactory.java 10464 2006-11-21 18:55:33Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.exoplatform.applet.logging.file;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.exoplatform.applet.logging.Level;
import org.exoplatform.applet.logging.Logger;
import org.exoplatform.applet.logging.LoggingSupport;
import org.exoplatform.applet.plugin.AppletPlugin;

/**
 * @version $Rev: 10464 $ $Date: 2006-11-22 01:55:33 +0700 (Wed, 22 Nov 2006) $
 * @author <a href="mailto:rafael.marins@exoplatform.com">Rafael Marins</a>
 */
public class FileLoggerFactory implements LoggingSupport, AppletPlugin {

	private static FileLoggerFactory singletonInstance;

	private Map loggers;

	private Level level;

    public FileLoggerFactory() {
	    super();
	    this.loggers = new HashMap();
	    this.level = Level.OFF;
    }

    public static FileLoggerFactory getInstance() {
    	if (singletonInstance == null)
    		singletonInstance = new FileLoggerFactory();
    	return singletonInstance;
    }

    /* (non-Javadoc)
     * @see org.exoplatform.applet.plugin.AppletPlugin#getName()
     */
    public String getName() {
        return "LoggingPlugin";
    }

    /* (non-Javadoc)
     * @see org.exoplatform.applet.plugin.AppletPlugin#getDescription()
     */
    public String getDescription() {
        return "Exo Applet file logging plugin";
    }

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.logging.LoggingSupport#getLogger(java.lang.String)
	 */
	public Logger getLogger(String key) {
		return getLogger(key, null);
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.logging.LoggingSupport#getLogger(java.lang.String, java.lang.String)
	 */
	public Logger getLogger(String key, String bundleName) {

		Logger l = (Logger) loggers.get(key);

		if (l == null) {
			l = createLogger(key, bundleName);
			loggers.put(key, l);
		}

		return l;
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.logging.LoggingSupport#getLoggers()
	 */
	public Logger[] getLoggers() {
	    Collection col = loggers.values();
	    return (Logger[]) col.toArray(new Logger[col.size()]);
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.logging.LoggingSupport#configureLevel(org.exoplatform.applet.logging.Level)
	 */
	public void configureLevel(final Level level) {

        final Iterator it = loggers.values().iterator();

        Thread t = new Thread(new Runnable() {
			public void run() {
				while (it.hasNext()) {
					Logger l = (Logger) it.next();
					l.setLevel(level);
				}
			}
		});

		t.start();
		
	    this.level = level;
	}

	private Logger createLogger(String key, String bundleName) {

		Logger l = new FileLogger(key, bundleName);
		l.setLevel(level);

		return l;
	}
}
