/**
 * Copyright (c) 2001-2006 eXo Platform SARL, http://www.exoplatform.com/
 *
 * All rights reserved. Please look at license.txt in info directory for
 * more license detail.
 *
 * -------------------------------------------------------------------------
 * $Id: PluginManager.java 8955 2006-09-18 13:46:06Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.exoplatform.applet.plugin;

import java.applet.Applet;


/**
 * @version $Rev: 8955 $ $Date: 2006-09-18 20:46:06 +0700 (Mon, 18 Sep 2006) $
 * @author <a href="mailto:rafael.marins@exoplatform.com">Rafael Marins</a>
 */
public class PluginManager {

	private class LifeCycleCommand implements Runnable {

		public static final int LC_INIT = 1;
		public static final int LC_START = 2;
		public static final int LC_STOP = 3;
		public static final int LC_SHUTDOWN = 4;

		private static final String LC_COMMAND_THREAD_NAME = "PluginManager.LifeCycleCommandThread";

		private Applet applet;
		private AppletLifecyclePlugin plugin;
		private int command;

        public LifeCycleCommand(Applet applet, AppletLifecyclePlugin plugin, int command) {
	        super();
	        this.applet = applet;
	        this.plugin = plugin;
	        this.command = command;
        }

        public void run() {

        	switch (command) {
        	case LC_INIT:
        		plugin.doInit(applet);
        		break;
        	case LC_START:
        		plugin.doStart();
        		break;
        	case LC_STOP:
        		plugin.doStop();
        		break;
        	case LC_SHUTDOWN:
        		plugin.doShutdown();
        		break;
        	}
	        
        }

        public Thread createThread() {
        	return new Thread(this, LC_COMMAND_THREAD_NAME);
        }
		
	}

	private Applet target;
	private AppletPlugin[] plugins;

	/**
     * @param target
	 * @param plugins 
     */
    public PluginManager(Applet target, AppletPlugin[] plugins) {
	    super();
	    this.target = target;
	    this.plugins = plugins;
    }

    public void initLifeCyclePlugins() {
    	fireLifeCyclePlugins(LifeCycleCommand.LC_INIT);
    }

    public void startLifeCyclePlugins() {
    	fireLifeCyclePlugins(LifeCycleCommand.LC_START);
    }

    public void stopLifeCyclePlugins() {
    	fireLifeCyclePlugins(LifeCycleCommand.LC_STOP);
    }

    public void shutdownLifeCyclePlugins() {
    	fireLifeCyclePlugins(LifeCycleCommand.LC_SHUTDOWN);
    }

    private void fireLifeCyclePlugins(int command) {

    	for (int i=0; i<plugins.length; i++) {

    		AppletPlugin p = plugins[i];

    		if (p instanceof AppletLifecyclePlugin) {

    			LifeCycleCommand cmd = new LifeCycleCommand(target,
    					(AppletLifecyclePlugin) p, command);

    			Thread t = cmd.createThread();
    			t.start();
    		}
    	}
    }

}
