/**
 * Copyright (c) 2001-2006 eXo Platform SARL, http://www.exoplatform.com/
 *
 * All rights reserved. Please look at license.txt in info directory for
 * more license detail.
 *
 * -------------------------------------------------------------------------
 * $Id: JVMRestrictedAction.java 9134 2006-09-26 23:03:22Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.exoplatform.applet.security;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.security.Provider;
import java.security.Security;

/**
 * @version $Rev: 9134 $ $Date: 2006-09-27 06:03:22 +0700 (Wed, 27 Sep 2006) $
 * @author <a href="mailto:rafael.marins@exoplatform.com">Rafael Marins</a>
 */
public class JVMRestrictedAction extends AbstractRestrictedAction {

	private static final String USER_HOME_PROPERTY_KEY = "user.home";

    JVMRestrictedAction() {
	    super();
    }

    /* (non-Javadoc)
     * @see org.exoplatform.applet.security.TrustedAccess#addSecurityProvider(java.security.Provider)
     */
    public int addSecurityProvider(final Provider prov) {
    	Integer result = (Integer)
    	AccessController.doPrivileged(new PrivilegedAction() {
    		public Object run() {
    			return Integer.valueOf(Security.addProvider(prov));
    		}
    	});
        return result.intValue();
    }

//	/* (non-Javadoc)
//	 * @see org.exoplatform.applet.security.RestrictedAction#addSSLProvider()
//	 */
//	public void addSSLProvider() throws IOException {
//        try {
//            AccessController.doPrivileged(new PrivilegedExceptionAction() {
//                public Object run() throws PrivilegedActionException {
//                    Security.addProvider(new com.sun.net.ssl.internal.ssl.Provider());
//                    return null;
//                }
//            });
//        } catch (PrivilegedActionException e) {
//        }
//	}
//
//	/* (non-Javadoc)
//	 * @see org.exoplatform.applet.security.RestrictedAction#connect(java.net.URLConnection)
//	 */
//	public void connect(final URLConnection con) throws IOException {
//        try {
//            AccessController.doPrivileged(new PrivilegedExceptionAction() {
//                public Object run() throws PrivilegedActionException {
//                    try {
//                        con.connect();
//                    } catch (IOException e) {
//                        throw new PrivilegedActionException(e);
//                    }
//                    return null;
//                }
//            });
//        } catch (PrivilegedActionException e) {
//            Throwable cause = e.getCause();
//            if (cause instanceof IOException){
//                throw (IOException) e.getCause();
//            } else {
//                throw new IOException("Exception from JVMRestrictedAction.connect(): "+e+" "+e.getMessage()+" cause:"+cause);
//            }
//        }
//	}
//
//	/* (non-Javadoc)
//	 * @see org.exoplatform.applet.security.RestrictedAction#createModalDialog(java.awt.Frame, java.lang.String)
//	 */
//	public Dialog createModalDialog(final Frame parent, final String title) {
//        return (Dialog) AccessController.doPrivileged(new PrivilegedAction() {
//            public Object run() {
//                return new Dialog(parent, title, true);
//            }
//        });
//	}
//
//	/* (non-Javadoc)
//	 * @see org.exoplatform.applet.security.RestrictedAction#createThread(java.lang.Runnable)
//	 */
//	public Thread createThread(final Runnable runnable) {
//        return (Thread) AccessController.doPrivileged(new PrivilegedAction() {
//            public Object run() {
//                return new Thread(runnable);
//            }
//        });
//	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.security.RestrictedAction#delete(java.io.File)
	 */
	public boolean delete(final File f) {
        try {
            Boolean res = (Boolean) AccessController.doPrivileged(new PrivilegedAction() {
                public Object run() {
                    if (f.delete()) {
                        return Boolean.TRUE;
                    } else {
                        return Boolean.FALSE;
                    }
                }
            });
            return res.booleanValue();
        } catch (Throwable t) {
            return false;
        }
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.security.RestrictedAction#exists(java.io.File)
	 */
	public boolean exists(final File f) {
        try {
            Boolean res = (Boolean) AccessController.doPrivileged(new PrivilegedAction() {
                public Object run() {
                    if (f.exists()) {
                        return Boolean.TRUE;
                    } else {
                        return Boolean.FALSE;
                    }
                }
            });
            return res.booleanValue();
        } catch (Throwable t) {
            return false;
        }
	}

//	/* (non-Javadoc)
//	 * @see org.exoplatform.applet.security.RestrictedAction#getFileDialog(java.awt.Frame, java.lang.String, int)
//	 */
//	public FileDialog getFileDialog(final Frame parent, final String title, final int mode) {
//        return (FileDialog) AccessController.doPrivileged(new PrivilegedAction() {
//            public Object run() {
//                return new FileDialog(parent, title, mode);
//            }
//        });
//	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.security.RestrictedAction#getFileInputStream(java.lang.String)
	 */
	public FileInputStream getFileInputStream(final String fileName) {
        return (FileInputStream) AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                try {
                    return new FileInputStream(fileName);
                } catch (FileNotFoundException e) {
                    return null;
                }
            }
        });
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.security.RestrictedAction#getFileInputStream(java.io.File)
	 */
	public FileInputStream getFileInputStream(final File file) {
        return (FileInputStream) AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                try {
                    return new FileInputStream(file);
                } catch (FileNotFoundException e) {
                    return null;
                }
            }
        });
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.security.RestrictedAction#getFileOutputStream(java.lang.String)
	 */
	public FileOutputStream getFileOutputStream(final String fileName) {
        return (FileOutputStream) AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                try {
                    return new FileOutputStream(fileName);
                } catch (FileNotFoundException e) {
                    return null;
                }
            }
        });
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.security.RestrictedAction#getFileOutputStream(java.io.File)
	 */
	public FileOutputStream getFileOutputStream(final File file) {
        return (FileOutputStream) AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                try {
                    return new FileOutputStream(file);
                } catch (FileNotFoundException e) {
                    return null;
                }
            }
        });
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.security.RestrictedAction#getFileOutputStream(java.lang.String, boolean)
	 */
	public FileOutputStream getFileOutputStream(final String fileName, final boolean append) {
        return (FileOutputStream) AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                try {
                    return new FileOutputStream(fileName, append);
                } catch (FileNotFoundException e) {
                    return null;
                }
            }
        });
	}

//	/* (non-Javadoc)
//	 * @see org.exoplatform.applet.security.RestrictedAction#getInputStream(java.net.URLConnection)
//	 */
//	public InputStream getInputStream(final URLConnection con) {
//        return (InputStream) AccessController.doPrivileged(new PrivilegedAction() {
//            public Object run() {
//                try {
//                    return con.getInputStream();
//                } catch (IOException e) {
//                    return null;
//                }
//            }
//        });
//	}
//
//	/* (non-Javadoc)
//	 * @see org.exoplatform.applet.security.RestrictedAction#getOutputStream(java.net.URLConnection)
//	 */
//	public OutputStream getOutputStream(final URLConnection con) {
//        return (OutputStream) AccessController.doPrivileged(new PrivilegedAction() {
//            public Object run() {
//                try {
//                    return con.getOutputStream();
//                } catch (IOException e) {
//                    return null;
//                }
//            }
//        });
//	}
//
//	/* (non-Javadoc)
//	 * @see org.exoplatform.applet.security.RestrictedAction#getPrintJob(java.awt.Frame, java.lang.String, java.util.Properties)
//	 */
//	public PrintJob getPrintJob(final Frame frame, final String title, final Properties prefs) {
//        return (PrintJob) AccessController.doPrivileged(new PrivilegedAction() {
//            public Object run() {
//                return Toolkit.getDefaultToolkit().getPrintJob(frame, title, prefs);
//            }
//        });
//	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.security.RestrictedAction#getProperty(java.lang.String)
	 */
	public String getProperty(final String key) {
		return (String) AccessController.doPrivileged(new PrivilegedAction() {
			public Object run() {
			    return System.getProperty(key);
			}
		});
	}

//	/* (non-Javadoc)
//	 * @see org.exoplatform.applet.security.RestrictedAction#getResourceAsStream(java.lang.Class, java.lang.String)
//	 */
//	public InputStream getResourceAsStream(final Class clazz, final String name) {
//        return (InputStream) AccessController.doPrivileged(new PrivilegedAction() {
//            public Object run() {
//                return clazz.getResourceAsStream(name);
//            }
//        });
//	}
//
//	/* (non-Javadoc)
//	 * @see org.exoplatform.applet.security.RestrictedAction#getResponseCode(java.net.HttpURLConnection)
//	 */
//	public int getResponseCode(final HttpURLConnection con) throws IOException {
//        Integer result = null;
//        try {
//            result = (Integer) AccessController.doPrivileged(new PrivilegedExceptionAction() {
//                public Object run() throws PrivilegedActionException {
//                    try {
//                        return new Integer(con.getResponseCode());
//                    } catch (IOException e) {
//                        throw new PrivilegedActionException(e);
//                    }
//                }
//            });
//        } catch (PrivilegedActionException e) {
//            Throwable cause = e.getCause();
//            if (cause instanceof IOException){
//                throw (IOException) e.getCause();
//            } else {
//                throw new IOException("Exception from SunRestrictedAction.getResponseCode(): "+e+" "+e.getMessage()+" cause:"+cause);
//            }
//        }
//        return result.intValue();
//	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.security.RestrictedAction#getUserHome()
	 */
	public String getUserHome() {
		return getProperty(USER_HOME_PROPERTY_KEY);
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.security.RestrictedAction#isDirectory(java.io.File)
	 */
	public boolean isDirectory(final File f) {
        try {
            Boolean res = (Boolean) AccessController.doPrivileged(new PrivilegedAction() {
                public Object run() {
                    if (f.isDirectory()) {
                        return Boolean.TRUE;
                    } else {
                        return Boolean.FALSE;
                    }
                }
            });
            return res.booleanValue();
        } catch (Throwable t) {
            return false;
        }
	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.security.RestrictedAction#list(java.io.File, java.io.FilenameFilter)
	 */
	public String[] list(final File dir, final FilenameFilter filter) {
        return (String[]) AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                return dir.list(filter);
            }
        });
	}

//	/* (non-Javadoc)
//	 * @see org.exoplatform.applet.security.RestrictedAction#load(java.lang.String)
//	 */
//	public boolean load(final String lib) {
//        AccessController.doPrivileged(new PrivilegedAction() {
//            public Object run() {
//                System.load(lib);
//                return null;
//            }
//        });
//        return true;
//	}
//
//	/* (non-Javadoc)
//	 * @see org.exoplatform.applet.security.RestrictedAction#loadLibrary(java.lang.String)
//	 */
//	public boolean loadLibrary(final String lib) {
//        AccessController.doPrivileged(new PrivilegedAction() {
//            public Object run() {
//                System.loadLibrary(lib);
//                return null;
//            }
//        });
//        return true;
//	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.security.RestrictedAction#mkdirs(java.io.File)
	 */
	public boolean mkdirs(final File f) {
        try {
            Boolean res = (Boolean) AccessController.doPrivileged(new PrivilegedAction() {
                public Object run() {
                    if (f.mkdirs()) {
                        return Boolean.TRUE;
                    } else {
                        return Boolean.FALSE;
                    }
                }
            });
            return res.booleanValue();
        } catch (Throwable t) {
            return false;
        }
	}

//	/* (non-Javadoc)
//	 * @see org.exoplatform.applet.security.RestrictedAction#setDefaultSSLSocketFactory(java.lang.Object)
//	 */
//	public void setDefaultSSLSocketFactory(final Object sslSocketFactory) {
//        AccessController.doPrivileged(new PrivilegedAction() {
//            public Object run() {
//                HttpsURLConnection.setDefaultSSLSocketFactory((SSLSocketFactory) sslSocketFactory);
//                return null;
//            }
//        });
//	}

	/* (non-Javadoc)
	 * @see org.exoplatform.applet.security.RestrictedAction#setProperty(java.lang.String, java.lang.String)
	 */
	public void setProperty(final String name, final String value) {
        AccessController.doPrivileged(new PrivilegedAction() {
            public Object run() {
                System.setProperty(name, value);
                return null;
            }
        });
	}

//	/* (non-Javadoc)
//	 * @see org.exoplatform.applet.security.RestrictedAction#showFileDialog(java.awt.FileDialog)
//	 */
//	public void showFileDialog(final FileDialog dialog) {
//        AccessController.doPrivileged(new PrivilegedAction() {
//            public Object run() {
//                dialog.show();
//                return null;
//            }
//        });
//	}

}
