/**
 * Copyright (c) 2001-2006 eXo Platform SARL, http://www.exoplatform.com/
 *
 * All rights reserved. Please look at license.txt in info directory for
 * more license detail.
 *
 * -------------------------------------------------------------------------
 * $Id: DistributionConfiguration.java 10468 2006-11-22 00:20:32Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.exoplatform.services.appletconfig.impl;

import java.io.IOException;
import java.io.Serializable;

import org.exoplatform.applet.deploy.LauncherConfiguration;
import org.exoplatform.applet.util.Base64Serializer;
import org.exoplatform.container.component.BaseComponentPlugin;
import org.exoplatform.container.xml.InitParams;

/**
 * 
 * @version $Rev: 10468 $ $Date: 2006-11-22 07:20:32 +0700 (Wed, 22 Nov 2006) $
 * @author <a href="mailto:rafael.marins@exoplatform.com">Rafael Marins</a>
 */
public class DistributionConfiguration extends BaseComponentPlugin implements Serializable {
    
    private static final long serialVersionUID = 1L;

    private ParameterConfiguration parameterConfig;

    private LauncherConfiguration launcherConfig;

    private String b64LauncherConfig;

    private String appletConfigName;

    public DistributionConfiguration(InitParams params) {
        super();
        readConfiguration(params);
    }

    /**
     * @param params
     */
    private void readConfiguration(InitParams params) {
        appletConfigName = params.getValueParam("applet-config-name")
                .getValue();
        parameterConfig = (ParameterConfiguration) params.getObjectParam(
                "applet-parameters").getObject();
        launcherConfig = (LauncherConfiguration) params.getObjectParam(
                "launcher-configuration").getObject();
    }

    /**
     * @return the appletConfigName
     */
    public String getAppletConfigName() {
        return appletConfigName;
    }

    /**
     * @return the launcherConfig
     */
    public LauncherConfiguration getLauncherConfig() {
        return launcherConfig;
    }

    /**
     * @return the parameterConfig
     */
    public ParameterConfiguration getParameterConfig() {
        return parameterConfig;
    }

    public String getB64LauncherConfig() {
        if (b64LauncherConfig == null) {
            try {
                byte[] encoded = Base64Serializer.serialize(launcherConfig);
                b64LauncherConfig = new String(encoded);
            } catch (IOException e) {
                // do nothing
            }
        }

        return b64LauncherConfig;
    }

}
