/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.indexing.test;

import java.util.ArrayList;
import java.util.List;

import org.apache.lucene.document.Document;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.services.indexing.FileIndexerPlugin;
import org.exoplatform.services.indexing.IndexingService;
import org.exoplatform.services.indexing.MultipleFieldSearchInput;
import org.exoplatform.services.indexing.Searcher;
import org.exoplatform.services.indexing.SingleFieldSearchInput;
import org.exoplatform.services.indexing.impl.FileIndexerPluginImpl;
import org.exoplatform.test.BasicTestCase;

/**
 * Thu, May 15, 2003 @   
 * @author: Tuan Nguyen
 * @version: $Id: TestIndexingService.java 5799 2006-05-28 17:55:42Z geaz $
 * @since: 0.0
 * @email: tuan08@yahoo.com
 */
public class TestIndexingService  extends BasicTestCase {
  static private IndexingService service_ ;
  public TestIndexingService(String name) {
    super(name);
  }

  public void setUp() throws Exception {
	if(service_ == null) {
      //PortalContainer pcontainer = PortalContainer.getInstance() ;
      service_ = (IndexingService) PortalContainer.getInstance().getComponentInstanceOfType(IndexingService.class) ;
    }
  }
 
  public void testIndexingService() throws Exception {
    //assertTrue("expect no indexer plugin is found",service_.getIndexerPlugins().size() ==0) ;
    FileIndexerPluginImpl plugin = 
      (FileIndexerPluginImpl)service_.getIndexerPlugin(FileIndexerPlugin.IDENTIFIER) ;    
    String[] acceptExt = { "java" , "xml"} ;    
    plugin.indexDirectory("src","guest", acceptExt , true) ;    
    // wait to make sure that indexer thread finish its job 
    Thread.sleep(2000) ;    
    assertTrue("expect found FileIndexerPlugin",service_.getIndexerPlugin(FileIndexerPlugin.IDENTIFIER)!=null) ;
    assertTrue("expect found this FileIndexerPlugin",service_.getIndexerPlugin(plugin.getPluginIdentifier())!=null) ;
    assertTrue("expect found only one indexer plugin",service_.getIndexerPlugins().size() ==1) ;            
    /*------------Search by single input-----*/    
    Searcher searcher = plugin.getSearcher() ;
    SingleFieldSearchInput docSingleSearchInput = 
      new SingleFieldSearchInput(IndexingService.DOCUMENT_FIELD) ;
    assertTrue("expect found DOCUMENT_FIELD:",
        docSingleSearchInput.getField().equals(IndexingService.DOCUMENT_FIELD)) ;
    docSingleSearchInput.setTerm("Tuan") ;     
    List hits =  searcher.search(docSingleSearchInput).getAll() ;
    assertTrue("expect found documents",hits.size()>0) ;   
    Document foundDoc = null ;
    String objectId = null ;    
    for(int i = 0; i < hits.size(); i ++) {
      foundDoc = (Document)hits.get(i) ;
      objectId =  foundDoc.getField(IndexingService.IDENTIFIER_FIELD).stringValue();
      String document = plugin.getObjectAsText(null, objectId) ;      
      assertTrue("expect term is found in documemt",document.contains(docSingleSearchInput.getTerm())) ;
    }
    /*---------search document by multi search input---------*/
    List fields = new ArrayList() ;        
    fields.add(IndexingService.DOCUMENT_FIELD) ;    
    fields.add(IndexingService.TITLE_FIELD) ;
        
    MultipleFieldSearchInput docMultiSearchInput = new MultipleFieldSearchInput(fields) ;
    assertTrue("expect found DOCUMENT_FIELD:",
        docMultiSearchInput.getField().contains(IndexingService.DOCUMENT_FIELD)) ;
    assertTrue("expect found TITLE_FIELD:",
        docMultiSearchInput.getField().contains(IndexingService.TITLE_FIELD)) ;    
    docMultiSearchInput.setTerm("configuration") ;        
    hits = searcher.search(docMultiSearchInput).getAll() ;  
    assertTrue("expect found documents",hits.size()>0) ;
    foundDoc = null ;
    for(int i = 0; i < hits.size(); i ++) {
      foundDoc = (Document)hits.get(i) ;      
      objectId =  foundDoc.getField(IndexingService.IDENTIFIER_FIELD).stringValue() ;            
      String document = plugin.getObjectAsText(null, objectId) ;
      assertTrue("expect term is found in documemt",document.contains(docMultiSearchInput.getTerm())) ;
    }    
    //---search document by combine single search input and multi search input----
    //--with multi search input, search's tearm is required on all field------
    docSingleSearchInput.isRequired() ;    
    docMultiSearchInput.setTerm("Indexing*") ;
    docMultiSearchInput.setMatchAllFields(true) ;        
    List inputs = new ArrayList() ;
    inputs.add(docSingleSearchInput) ;    
    inputs.add(docMultiSearchInput) ;
    hits = searcher.search(inputs).getAll() ;
    assertTrue("expect found documents",hits.size()>0) ;
    foundDoc = null ;
    for(int i = 0; i < hits.size(); i ++) {
      foundDoc = (Document)hits.get(i) ;
      objectId =  foundDoc.getField(IndexingService.IDENTIFIER_FIELD).stringValue() ;
      String document = plugin.getObjectAsText(null, objectId) ;
      assertTrue("expect found 'Tuan' in document",document.contains(docSingleSearchInput.getTerm())) ;      
      assertTrue("expect found 'Indexing' in document",document.contains("Indexing")) ;
      assertTrue("expect found 'Indexing' in document title",
          foundDoc.getField(IndexingService.TITLE_FIELD).stringValue().contains("Indexing")) ;      
    }
    /*------search module by query------*/
    List modules = new ArrayList() ;
    modules.add("FileIndexerPlugin") ;    
    hits = searcher.search(docSingleSearchInput,modules).getAll() ;
    assertTrue("expect found document in module",hits.size() >0) ;
    /*----------remove index-----*/
    plugin.removeIndex() ;
    Thread.sleep(3500) ;
    searcher = plugin.getSearcher() ;
    hits =  searcher.search(docSingleSearchInput).getAll() ;
    assertEquals("exect no result is found", 0 , hits.size()) ;
    service_.optimizeDatabase() ;    
  }
}