/**
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */

package org.exoplatform.services.templates.groovy;

import java.io.StringWriter;
import java.util.HashMap;
import java.util.Map;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.test.BasicTestCase;
/**
 * Created y the eXo platform team
 * User: Benjamin Mestrallet
 * Date: 16 juin 2004
 */
public class TestGroovyService extends BasicTestCase {
  private GroovyTemplateService service_ ;
  
  public TestGroovyService(String name) {
    super(name);
  }

  public void setUp() throws Exception {
    service_ = 
      (GroovyTemplateService) PortalContainer.getInstance().
       getComponentInstanceOfType(GroovyTemplateService.class) ;
  }

  public void testGroovyService() throws Exception {
    /*----------add template file loader-------*/
    String basedir = System.getProperty("basedir") ;    
    GroovyTemplateFileLoader floader = new GroovyTemplateFileLoader(basedir + "/src/resource") ;
    floader.setName("FileLoaderTemplate") ;
    assertTrue("expect TemplateFileLoader can load",
                floader.canLoad("file","FileLoaderTemplate.gtmpl") && 
                floader.load("FileLoaderTemplate.gtmpl")!=null) ;    
    assertEquals("expect TemplateFileLoader's priority is NOMAL",
                  floader.getPriority(),GroovyTemplateLoader.NORMAL_PRIORITY) ;
    floader.setPriority(GroovyTemplateLoader.HIGH_PRIORITY) ;
    assertEquals("now, expect TemplateFileLoader's priority is updated",
                  floader.getPriority(),GroovyTemplateLoader.HIGH_PRIORITY) ;    
    service_.addTemplateLoader(floader) ;
    /*----------add template classpath loader-------*/
    GroovyTemplateClasspathLoader cploader = new GroovyTemplateClasspathLoader() ;
    cploader.setName("ClasspathLoaderTemplate") ;
    String cpresource ="ClasspathLoaderTemplate.gtmpl" ;
    assertTrue("expect TemplateClasspathLoader can load",
                cploader.canLoad("classpath",cpresource) && 
                cploader.load(cpresource)!=null) ;    
    service_.addTemplateLoader(cploader) ;
    /*----------add template text loader-------*/
    String templateTextLoader = "this is Text Template Loader $helloSpecial" ;
    GroovyTemplateTextLoader txtloader = new GroovyTemplateTextLoader("templateText",templateTextLoader) ;    
    assertTrue("expect TemplateTextLoader cand load",
                txtloader.canLoad("text","templateText") && 
                txtloader.load("templateText")!=null) ;
    service_.addTemplateLoader(txtloader) ;
    /*----------Create binding map for all templates-------*/
    Map<String,String> binding = new HashMap<String,String>() ;
    binding.put("helloClasspathLoader", "hello classpath loader") ;
    binding.put("helloFileLoader", "hello file loader") ;
    binding.put("helloSpecial", "hello special") ;        
    /*----------binding template file-------*/
    StringWriter w = new StringWriter() ;    
    service_.merge(w, "file", "FileLoaderTemplate.gtmpl", binding) ;
    String fresult = w.getBuffer().toString() ;    
    assertTrue(fresult.indexOf("This  is a  file loader template") >= 0) ;
    String finclude = service_.include("FileLoaderTemplate.gtmpl",binding) ;
    assertTrue("expect template file is binded",finclude.indexOf("This  is a  file loader template")>=0) ;            
    /*----------binding template classpath-------*/
    String cpresult = service_.merge("classpath",cpresource,binding) ;
    assertTrue(cpresult.indexOf("This is a classpath loader template") >= 0) ;
    String cpinclude = service_.include(cpresource,binding) ;
    assertTrue("expect tempalte classpath is binded",cpinclude.indexOf("This is a classpath loader template") >= 0) ;    
    String cpinclude2 = service_.include("classpath",cpresource,binding) ;
    assertTrue(cpinclude2.indexOf("This is a classpath loader template") >= 0) ;    
    /*----------binding template text-------*/
    String txtresult = service_.merge("text","templateText",binding) ;        
    assertTrue("expect template text is binded",txtresult.indexOf("this is Text Template Loader")>=0) ;
    String txtinclude = service_.include("templateText",binding) ;
    assertTrue("expect template text is binded",txtinclude.indexOf("this is Text Template Loader")>=0) ;                
  } 
}