/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.faces.core.component;

import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.imageio.ImageIO;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.logging.Log;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.services.document.image.ImageProcessingService;
import org.exoplatform.services.log.ExoLogger;

/**
 * @author: Monica Franceschini
 * @email: monica.franceschini@eng.it
 */

public class UIImageUploadInput extends UIUploadInput {

	private String contentType;

	private List options;

	private static String GIF_FORMAT = "gif";

	private static String TIFF_FORMAT = "tiff";

	private static String JPG_FORMAT = "jpg";
  
  private static Log log_ = ExoLogger.getLogger("jsf.UIImageUploadInput"); 

	public UIImageUploadInput(String name) {
		super(name);
		setRendererType("ImageUploadInputRenderer");
	}

	public UIImageUploadInput(String name, List optionList) {
		super(name);
		options = optionList;
		setRendererType("ImageUploadInputRenderer");
	}

	public void decodeFromMultipartFields(FacesContext context,
			UIComponent component, List items) {
    
    PortalContainer container = PortalContainer.getInstance();
    ImageProcessingService imageProcessingService = (ImageProcessingService) container.getComponentInstanceOfType(ImageProcessingService.class);
    
		Iterator iter = items.iterator();
		while (iter.hasNext()) {
			FileItem item = (FileItem) iter.next();
			if (!item.isFormField()) {
				String name = item.getFieldName();
				setContentType(item.getContentType());

				if (name.equals(getName())) {
					// Some browsers such as IE specify full paths that we have
					// to parse.
					// The java.io.File class cannot be leveraged to do so as
					// the client may use different path separators than the server.
					String[] fileNameItems = item.getName().split("\\\\|/");
					String fileName = fileNameItems[fileNameItems.length - 1];
					// content
					String formatName = fileName.substring(fileName.lastIndexOf(".") + 1);
					if ((fileName != null && !fileName.equals(""))) {
						try {
							setFileName(fileName);
							// manage gif/tiff formats
							if (formatName.equalsIgnoreCase(GIF_FORMAT)
									|| formatName.equalsIgnoreCase(TIFF_FORMAT)) {
								formatName = JPG_FORMAT;
							}
							ByteArrayInputStream bais = new ByteArrayInputStream(item.get());
							BufferedImage img = ImageIO.read(bais);
							bais.close();

							BufferedImage imageBuf = null;
							// First parameter = transform algorithm. Permitted values =
							// 'crop', 'bound', 'scale'
							String transformOption = (String) options.get(0);
							// NOTE: more accurated controls on parameters should be done...
							if (transformOption != null && !"".equals(transformOption)
									&& transformOption.equalsIgnoreCase("crop")) {
								// Cropped image
								int chosenWidth = Integer.parseInt((String) options.get(1));
								int chosenHeight = Integer.parseInt((String) options.get(2));
                imageBuf = imageProcessingService.createCroppedImage(img, chosenWidth, chosenHeight);

              } else if (transformOption != null && !"".equals(transformOption)
									&& transformOption.equalsIgnoreCase("bound")) {
								// Bound image
								int chosenWidth = Integer.parseInt((String) options.get(1));
								int chosenHeight = Integer.parseInt((String) options.get(2));
								String bgColor = "";
								if (options.size() == 4) {
									// Optional parameter: background color
									bgColor = (String) options.get(3);
								}
                imageBuf = imageProcessingService.createBoundImage(img, chosenWidth, chosenHeight, bgColor);

							} else if (transformOption != null && !"".equals(transformOption)
									&& transformOption.equalsIgnoreCase("scale")) {
								// Scaled image
                imageBuf = imageProcessingService.createScaledImage(img, Double.parseDouble((String) options.get(1)));

							} else {
								// TODO: Default algorithm
							}

							ByteArrayOutputStream out = new ByteArrayOutputStream();
							try {
								ImageIO.write(imageBuf, formatName, out);
							} catch (Exception e) {
								e.printStackTrace();
							}

							setContent(out.toByteArray());
							out.flush();
							out.close();
						} catch (IOException e) {
							e.printStackTrace();
						}
					} else {
						// byte[] buf = item.get();
						// setContent(buf);
					}
				}
			} else {
				String checkBoxFieldName = "remove-" + getName();
				if (item.getFieldName().equals(checkBoxFieldName)) {
					byte[] buf = item.get();
					setContent(buf);
				}
			}
		}
	}

	public String getContentType() {
		return contentType;
	}

	public void setContentType(String contentType) {
		this.contentType = contentType;
	}
	
}
