/**
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */
package org.exoplatform.faces.user.component;

import javax.faces.context.ExternalContext;
import javax.portlet.ActionResponse;

import org.exoplatform.container.SessionContainer;
import org.exoplatform.faces.core.component.UIExoCommand ;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.session.RequestInfo;
import org.exoplatform.services.portletcontainer.pci.Output;
import org.exoplatform.services.security.SecurityService;
/**
 * Created y the eXo platform team
 * User: Benjamin Mestrallet
 * Date: 1 mai 2004
 */
public class UILogout extends UIExoCommand {
  public static final String LOGOUT_ACTION = "logout";
  public static final Parameter[] LOGOUT_PARAMS = {new Parameter(ACTION, LOGOUT_ACTION)} ; 

  private String fullName_;
  private String logoutPath_;
  private boolean adminAccess_ ;
  private SecurityService securityService_;

  public UILogout(SecurityService service) throws Exception {
    setId("UILogout");
    setRendererType("VelocityRenderer") ;
    securityService_ = service ;
    RequestInfo rinfo = (RequestInfo)SessionContainer.getComponent(RequestInfo.class) ;
    String accessibility = rinfo.getAccessibility() ;
    adminAccess_ = RequestInfo.ADMIN_ACCESS.equals(accessibility) ;
    if(rinfo.getPortalOwner().equals(rinfo.getRemoteUser()))
      fullName_ = rinfo.getPortalOwner();
    else
      fullName_ = rinfo.getRemoteUser()+"@"+rinfo.getPortalOwner();
    if (adminAccess_) {
      logoutPath_ = rinfo.getOwnerURI() + "?admin:returnAction=return"  ;
    } else {
      logoutPath_ = rinfo.getContextPath() ;
      addActionListener(LogoutActionListener.class , LOGOUT_ACTION) ;
    }
    
  }

  public boolean isAdminAccess() { return adminAccess_ ; }
  
  public Parameter[]  getLogoutParameters()  {  return LOGOUT_PARAMS ; }
  
  public String getFullName() {  return fullName_; }
  
  public String getLogoutPath() { return logoutPath_ ; }
  
  static public class LogoutActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UILogout comp = (UILogout) event.getComponent() ;
      ExternalContext eContext = comp.getFacesContext().getExternalContext();
      ActionResponse response = (ActionResponse) eContext.getResponse();
      response.addProperty(Output.LOGOUT, "true");
      if (comp.securityService_.isStandaloneAuthentication()) {
        response.sendRedirect(comp.logoutPath_);
      } else if (comp.securityService_.isSSOAuthentication()) {
        //TODO admin logout
        response.sendRedirect(comp.logoutPath_ + "/" + comp.securityService_.getSSOAuthentication() + "_logout.jsp");
      }
    }
  }
}
