/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.config.content.test;

import java.util.List;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.portal.config.content.DocumentContent;
import org.exoplatform.portal.config.content.PortalContentService;
import org.exoplatform.test.BasicTestCase;

/**
 * Created by The eXo Platform SARL
 * Author : Pham Xuan Hoa
 *          phamvuxuanhoa@yahoo.com
 * Feb 23, 2006
 */
public class TestPortalContentService extends BasicTestCase{
  
  private PortalContentService service_ ;
  
  public TestPortalContentService(String name) {
    super(name) ;
  }
  
  public void setUp() throws Exception{
    PortalContainer container = PortalContainer.getInstance() ;
    if(service_ == null) {
      service_ = 
        (PortalContentService)container.getComponentInstanceOfType(PortalContentService.class) ;
    }
  } 
  
  public void teaDown() throws Exception {    
  }
  
  public void testPortalContentService()  throws Exception {
    String basedir = System.getProperty("basedir") ;    
    String id = "file://"+basedir+"/pom.xml" ;
    String content = service_.getExternalContent(id,"UTF-8") ;
    assertTrue("expect content is not null",content!=null) ;
    //before prepare data to test
    List<DocumentContent> list = service_.getAllDocuments() ;
    System.out.println("=====================>"+list.size()) ;
//    assertEquals("expect init content is found ",list.size(),4) ;
    prepareData() ;    
    // after prepare data to test
    list = service_.getAllDocuments() ;        
    assertEquals("expect to find 3 document ",list.size(),3) ;
    
    DocumentContent document = service_.getDocumentById("d1") ;
    assertTrue("expect to found document",document !=null) ;
    assertTrue("expect owner is :exo",document.getOwner().equals("exo")) ;    
    assertTrue("expect content is true",document.getContent().equals("exo d1 content")) ;
    //---update document content
    document.setContent("this document is updated") ;       
    
    service_.update(document) ;    
    list = service_.getAllDocuments() ;
    assertEquals("expect numbers of document is still",3,list.size()) ;    
    document = service_.getDocumentById("d1") ;
    assertTrue("expect document owner don't change",document.getOwner().equals("exo")) ;
    assertTrue("expect document content is updated",document.getContent().equals("this document is updated")) ;                  
    list = service_.getDocumentsByOwner("exo") ;
    assertEquals("expect to found 1 document",list.size(),1) ;
    
    list = service_.getDocumentsByOwner("hoa") ;    
    assertTrue("expect to found 2 document ",list.size() == 2) ;
    for(DocumentContent d:list) {
      assertTrue("expect owner of this document is true",d.getOwner().equals("hoa")) ;
    }
    //--------remove 
    document = service_.getDocumentById("d2") ;
    assertTrue("expect to found this document",document.getId().equals("d2")) ;
    assertTrue("expect to found this document",document.getOwner().equals("hoa")) ;
    service_.remove(document) ;
    list = service_.getAllDocuments() ;
    assertTrue("now, expect to found 2 documents",list.size() == 2) ;
    list = service_.getDocumentsByOwner("hoa") ;
    assertTrue("now,there is 1 documents that found",list.size() == 1) ;
    document = service_.getDocumentById("d2") ;
    assertTrue("expect no document to found",document == null) ;
    
    document = service_.getDocumentById("d1") ;
    assertTrue("expect to found this document",document.getId().equals("d1")) ;
    service_.remove(document) ;
    list = service_.getDocumentsByOwner("exo") ;
    assertTrue("expect no document is found",list.size() == 0) ;
    
    document = service_.getDocumentById("d3") ;
    assertTrue("expect to found this document",document.getId().equals("d3")) ;
    service_.remove(document) ;
    document = service_.getDocumentById("d3") ;
    assertTrue("expect no document to found",document == null) ;
    
    list = service_.getAllDocuments() ;
    assertTrue("expect all document is removed",list.size() == 0) ;
  }
  
  private void prepareData() throws Exception {
    createDocumentContent("d1","exo","exo d1 content") ;
    createDocumentContent("d2","hoa","hoa d2 content") ;
    createDocumentContent("d3","hoa","hoa d3 content") ;
  }
  
  private  void createDocumentContent(String id, String owner,String content) throws Exception {
    DocumentContent document = service_.createDocumentContentInstance() ;
    document.setId(id) ;    
    document.setContent(content) ;        
    document.setOwner(owner) ;
    service_.save(document) ;
  }
}

