/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.faces.component;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.core.validator.PermissionValidator;
import org.exoplatform.faces.user.component.UIExoPermissionSelector;
import org.exoplatform.portal.config.model.PortalConfig;
import org.exoplatform.portal.faces.listener.page.ShowCurrentPageActionListener;
import org.exoplatform.portal.skin.SkinConfigService;
import org.exoplatform.portal.skin.model.Decorator;
import org.exoplatform.portal.skin.model.Style;
import org.exoplatform.services.organization.OrganizationService;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIPortalForm.java,v 1.9 2004/09/29 17:44:51 benjmestrallet Exp $
 */
public class UIPortalForm extends UISimpleForm {
  static String VIEW_PERMISSION = "viewPermission", EDIT_PERMISSION = "editPermission" ,
                DEFAULT_LOCALE = "defaultLocale", RENDERER = "renderer", STYLE = "style";
  private List styleOptions_;
  private List rendererOptions_;

  public UIPortalForm(SkinConfigService service, OrganizationService orgService) throws Exception {
    super("portalForm", "post", null) ;
    setRendererType("SimpleFormVelocityRenderer") ;
    setId("UIPortalForm") ;
    setClazz("UIExoForm") ;
    
    rendererOptions_ = new ArrayList();
    styleOptions_ = new ArrayList();
    Iterator i = service.getPortalDecorators().iterator();
    while(i.hasNext()) {
      Decorator decorator = (Decorator) i.next();
      String rendererType = decorator.getRendererType();
      rendererOptions_.add(new SelectItem(rendererType, rendererType));
    }
    
   
    Decorator currentDecorator =  service.getPortalDecorator("default");
    List styles = currentDecorator.getStyles();
    for(int j= 0; j < styles.size(); j++) {
      Style style = (Style)styles.get(j);
      String name = style.getName();
      styleOptions_.add(new SelectItem(name, name));
    }
    
    UISelectBox rendererInput = new UISelectBox(RENDERER, "", rendererOptions_);
    rendererInput.setUpdateOnChangeAction("updateStyles");
    rendererInput.setValue("");    
    add(new UIStringInput(DEFAULT_LOCALE, "")) ;
    UIExoPermissionSelector selectPermission = new UIExoPermissionSelector(VIEW_PERMISSION, null) ;
    add(selectPermission.addValidator(PermissionValidator.class)) ;
    selectPermission = new UIExoPermissionSelector(EDIT_PERMISSION, null) ; 
    add(selectPermission.addValidator(PermissionValidator.class)) ;
    add(rendererInput) ;
    add(new UISelectBox(STYLE, "", styleOptions_ )) ;
    addActionListener(SaveActionListener.class, SAVE_ACTION) ;
    addActionListener(ShowCurrentPageActionListener.class, CANCEL_ACTION) ;
  }
   
  public String getSaveAction() { return SAVE_ACTION ;  }
  public String getCancelAction() { return CANCEL_ACTION ;  }
  
  public void setUIPortal(UIPortal uiPortal) {
  	PortalConfig model = uiPortal.getPortalConfigModel() ;
    getUIStringInput(DEFAULT_LOCALE).setValue(model.getLocale()) ;
    ((UIExoPermissionSelector)getUIInput(VIEW_PERMISSION)).setPermission(model.getViewPermission()) ;
    ((UIExoPermissionSelector)getUIInput(EDIT_PERMISSION)).setPermission(model.getEditPermission()) ;
    getUISelectBox(RENDERER).setValue(model.getRenderer()) ;
    getUISelectBox(STYLE).setValue(uiPortal.getDecorator()) ;
  }
   
  static public class SaveActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
      UIPortalForm uiForm = (UIPortalForm) event.getSource() ;
	    UIPortal uiPortal = (UIPortal) uiForm.getAncestorOfType(UIPortal.class) ;
			PortalConfig config = uiPortal.getEditablePortalConfigModel() ;
      config.setLocale(uiForm.getUIStringInput(DEFAULT_LOCALE).getValue()) ;
      UIExoPermissionSelector permSelector = 
        (UIExoPermissionSelector)uiForm.getUIInput(VIEW_PERMISSION) ;
      config.setViewPermission(permSelector.getPermission()) ;
      permSelector = (UIExoPermissionSelector)uiForm.getUIInput(EDIT_PERMISSION) ;
      
      config.setEditPermission(permSelector.getPermission()) ;
      config.setRenderer(uiForm.getUISelectBox(RENDERER).getValue()) ;
      config.setDecorator(uiForm.getUISelectBox(STYLE).getValue()) ;
      uiPortal.updateChange() ;
      uiPortal.setBodyComponent(uiPortal.getCurrentUIPage()) ;
      uiPortal.setComponentModified(true) ;
		}
	}
}	