/**
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */

package org.exoplatform.portlets.management.registry.component;

import java.util.ResourceBundle;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UITextArea;
import org.exoplatform.faces.core.event.CheckRoleInterceptor;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.core.validator.NameValidator;
import org.exoplatform.portal.Information;
import org.exoplatform.portal.faces.component.model.PortletCategoryData;
import org.exoplatform.services.portletregistery.PortletCategory;
import org.exoplatform.services.portletregistery.PortletRegisteryService;
/**
 * Created y the eXo platform team
 * User: Benjamin Mestrallet
 * Date: 16 juin 2004
 */
public class UICategoryForm extends UISimpleForm {
	private PortletCategoryData categoryData_ ;
  protected UIStringInput name_;
  protected UITextArea description_;

  public UICategoryForm() {
    super("portletCategoryForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer") ;
    name_ = new UIStringInput("name", "").
    addValidator(NameValidator.class);    
    add(name_) ;
    description_ = new UITextArea("description", "");
    add(description_) ;
    
    addFacesListener(new SaveActionListener().setActionToListen(SAVE_ACTION));
    addFacesListener(new CancelActionListener().setActionToListen(CANCEL_ACTION));
  }
  
  public String getSaveAction() { return SAVE_ACTION ; }
  public String getCancelAction() { return CANCEL_ACTION ; }
  
  public void setPortletCategoryData(PortletCategoryData category) {
  	categoryData_ = category ;
  	if(category == null) {
  		description_.setValue("") ;
  		name_.setValue("") ;
  	} else {
  		name_.setValue(category.getPortletCategoryName()) ;
  		description_.setValue(category.getPortletCategory().getDescription()) ;
  	}
  }
  
  public class SaveActionListener extends ExoActionListener {
    public SaveActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
    public void execute(ExoActionEvent event) throws Exception {
    	UICategoryForm uiForm = (UICategoryForm) event.getComponent() ;
      String categoryName = name_.getValue() ;
      PortletRegisteryService service =
        (PortletRegisteryService) PortalContainer.getComponent(PortletRegisteryService.class) ;
      if(categoryData_ == null) {
        PortletCategory category ;
        try{
          category = service.getPortletCategory(categoryName) ;
        }catch(Exception e){
          category = null ;
        }        
        if(category == null){
          category = service.createPortletCategoryInstance();
          category.setPortletCategoryName(categoryName);
          category.setDescription(description_.getValue());
          service.addPortletCategory(category);
        }else{          
          Information iprovider = findInformationProvider(uiForm);
          ResourceBundle res = getApplicationResourceBundle() ;
          iprovider.addMessage(new ExoFacesMessage(res.getString("UICategoryForm.msg.category-alredy-exist"))) ;
          return ;
        }       	
      } else {
      	PortletCategory category = categoryData_.getPortletCategory() ;
      	category.setPortletCategoryName(categoryName);
      	category.setDescription(description_.getValue());
      	service.updatePortletCategory(category);
      }
      UIPortletCategories uiCategories = 
    		(UIPortletCategories) uiForm.getSibling(UIPortletCategories.class) ;
      uiCategories.refresh() ;
      uiForm.setRenderedSibling(UIPortletCategories.class) ;
    }
  }
  
  public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
    	UICategoryForm uiForm = (UICategoryForm) event.getComponent() ;
      uiForm.setRenderedSibling(UIPortletCategories.class) ;
    }
  }
}