/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.resources.component;

import java.util.Collection;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.CheckRoleInterceptor;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.resources.Query;
import org.exoplatform.services.resources.ResourceBundleData;
import org.exoplatform.services.resources.ResourceBundleService;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen 
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIListResources.java,v 1.6 2004/10/16 21:31:13 tuan08 Exp $
 */
public class UIListResources extends UIExoCommand  {
   private static Parameter[] VIEW_RESOURCE =  { new Parameter(ACTION , VIEW_ACTION) };
   private static Parameter[] DELETE_RESOURCE = { new Parameter(ACTION , DELETE_ACTION)} ;
	
  private UIPageListIterator uiPageIterator_ ;
  private ResourceBundleService service_ ;
  private boolean adminRole_ ;
  private Query query_ ;
  private UISearchForm uiSearchForm_ ; 
  public UIListResources(ResourceBundleService service) throws Exception {
    setId("UIListResources");
    setRendererType("VelocityRenderer");
    service_ = service ;
    adminRole_ = hasRole("admin") ;
    uiSearchForm_ = new UISearchForm(adminRole_);
    uiPageIterator_ = new UIPageListIterator(new PageListDataHandler()) ;    
    getChildren().add(uiSearchForm_);
    getChildren().add(uiPageIterator_);
    addActionListener(ViewActionListener.class, VIEW_ACTION) ;
    addActionListener(DeleteActionListener.class, DELETE_ACTION) ;
    query_ = new Query(null, null) ;
    update() ;
  }
  
  public Collection getPageListDataHandler() {
    return uiPageIterator_.getPageListDataHandler().getDatas() ;
  }
  
  public Parameter[] getViewParams() { return VIEW_RESOURCE ; }
  public Parameter[] getDeleteParams() { return DELETE_RESOURCE ; }
  
  public void update(String name , String lang) throws Exception {
    query_.setName(name) ;
    query_.setLanguage(lang) ;
    uiPageIterator_.setPageList(service_.findResourceDescriptions(query_)) ;
  }
  
  public void update() throws Exception {
  	int currentPage = uiPageIterator_.getCurrentPage() ;
  	int availablePage = uiPageIterator_.getAvailablePage() ;  	
    uiPageIterator_.setPageList(service_.findResourceDescriptions(query_)) ;
    if(currentPage<=availablePage) {    	
    	uiPageIterator_.getPageListDataHandler().selectPage(currentPage) ;
    }    
  }
 
  static public class ViewActionListener extends ExoActionListener  {
  	public void execute(ExoActionEvent event) throws Exception {
  		UIListResources uiListResources = (UIListResources) event.getComponent() ;
  		UIResourcesPortlet uiPortlet = (UIResourcesPortlet)uiListResources.getParent() ;
      UIResource uiResource = 
      	(UIResource)uiPortlet.getChildComponentOfType(UIResource.class);
      String resourceId = event.getParameter(OBJECTID) ;
      ResourceBundleData data = uiListResources.service_.getResourceBundleData(resourceId) ; 
      uiResource.setResourceBundleData(data) ;
      uiResource.setMode(UIResource.VIEW_MODE) ;
      uiPortlet.setRenderedComponent(uiResource.getId()) ;
    }
  }
  
  static public class DeleteActionListener extends ExoActionListener  {
    public DeleteActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
  	public void execute(ExoActionEvent event) throws Exception {
  		UIListResources uicomp = (UIListResources) event.getComponent() ;
  		String resourceId = event.getParameter(OBJECTID) ;  
    	uicomp.service_.removeResourceBundleData(resourceId) ;    	
      uicomp.update();
    }
  }
}