/**
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail. 
 */
package org.exoplatform.portlets.monitor.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.GregorianCalendar;
import java.util.List;

import org.exoplatform.commons.utils.PageList;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.log.UserActivitiesLogService;
import org.exoplatform.portal.log.UserLogQuery;
/**
 * Created y the eXo platform team
 * User: Hung Nguyen
 * Date: 05 oct 2005
 */
public class UIStatisticsUserActionLog  extends UIExoCommand  {
  private String userName_ = "";
  private String portalOwner_ = "";
  private long   fromDateTime_ ;
  private long   toDateTime_ ;
  private String limit_ = "0" ;
  private String maxmin_ = "" ;
  private String accessIp_ = "" ;
  private String timeSlotOrder_ = "Desc" ;
  private String durationOrder_ = "" ;
  private UIPageListIterator uiPageIterator_ ;
  private List<SelectItem> chartStyle_ ;
  final static public String VIEW_ACTION = "view" ;
  private static Parameter[] VIEW_PARAMS = {new Parameter(ACTION , VIEW_ACTION) } ;
  
  public UIStatisticsUserActionLog() throws Exception {
    setId("UIStatisticsUserActionLog")  ;
    setRendererType("VelocityRenderer") ;
    PortalContainer pcontainer =  PortalContainer.getInstance() ;
    portalOwner_ = pcontainer.getSessionManager().getLiveSessions().get(0).getOwner();
    GregorianCalendar calendar = new GregorianCalendar() ;
    calendar.set(GregorianCalendar.HOUR_OF_DAY, 0) ;
    calendar.set(GregorianCalendar.MINUTE, 0) ;
    calendar.set(GregorianCalendar.SECOND, 0) ;
    calendar.set(GregorianCalendar.MILLISECOND, 0) ;
    fromDateTime_ = calendar.getTimeInMillis() ;
    calendar.set(GregorianCalendar.HOUR_OF_DAY, 24) ;
    toDateTime_ = calendar.getTimeInMillis() ;    
    chartStyle_ = new ArrayList<SelectItem>() ;
    chartStyle_.add(new SelectItem("Remote User", "userName")) ;
    chartStyle_.add(new SelectItem("Portal Owner", "portalOwner")) ;
    uiPageIterator_ = new UIPageListIterator(new PageListDataHandler()) ;
    getChildren().add(uiPageIterator_);
    getChildren().add(new UIUserLogFilter());       
    addActionListener(ViewChartActionListener.class, VIEW_ACTION) ; 
    refresh() ;
  }
  public Parameter[] getViewParams() { return VIEW_PARAMS ; }
  public long getFromDateTime() { return fromDateTime_ ; }
  public long getToDateTime() { return toDateTime_ ; }
  public String getUserName() { return userName_ ; }
  public String getPortalName() { return portalOwner_ ; }
  public String getAccessIp() { return accessIp_ ; }
  public String getMaxmin() { return maxmin_ ; }
  public String getLimit() { return limit_ ; }
  public List getChartStyle() { return chartStyle_ ; }
  
  private void refresh() throws Exception{
    uiPageIterator_.getPageListDataHandler().setPageList(queryLogData()) ;
  }
  public Collection getChartData() throws Exception {
    UserActivitiesLogService service = 
      (UserActivitiesLogService) PortalContainer.getComponent(UserActivitiesLogService.class) ;
    UserLogQuery q = new UserLogQuery() ;
    q.setFromDateTime(fromDateTime_) ;
    q.setToDateTime(toDateTime_) ;
    q.setLimit(limit_) ;
    q.setMaxmin(maxmin_) ;
    q.setUserName(userName_) ;
    q.setPortalOwner(portalOwner_) ;
    q.setAccessIp(accessIp_) ;
    q.setOrder(timeSlotOrder_) ;
    q.setDurationOrder(durationOrder_) ;
    return service.getUserLogChartDatas(q, chartStyle_) ;     
  }
  
  public List getCurrentPageData() throws Exception {    
    return uiPageIterator_.getPageListDataHandler().getDatas() ;  
  }
  
  private PageList queryLogData() throws Exception {
    UserActivitiesLogService service = 
      (UserActivitiesLogService) PortalContainer.getComponent(UserActivitiesLogService.class) ;
    UserLogQuery q = new UserLogQuery() ;
    q.setFromDateTime(fromDateTime_) ;
    q.setToDateTime(toDateTime_) ;
    q.setLimit(limit_) ;
    q.setMaxmin(maxmin_) ;
    q.setUserName(userName_) ;
    q.setPortalOwner(portalOwner_) ;
    q.setAccessIp(accessIp_) ;
    q.setOrder(timeSlotOrder_) ;
    q.setDurationOrder(durationOrder_) ;
    return service.getUserLogDatas(q) ;    
  }
  
  public void update(long fromTime, long toTime, String limit, String uName, String pOwner, 
                     String maxmin, String accessIp, String timeOrder, String dOrder) throws Exception {
    fromDateTime_ = fromTime ;
    toDateTime_   = toTime ;
    userName_     = uName ;
    portalOwner_   = pOwner ;
    limit_        = limit ;
    maxmin_       = maxmin ;
    accessIp_     = accessIp ; 
    timeSlotOrder_ = timeOrder ; 
    durationOrder_ = dOrder ;
    refresh() ;
  }  
  static public class ViewActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIStatisticsUserActionLog uiComp  = (UIStatisticsUserActionLog) event.getSource() ;
    }
  }
 
  static public class ViewChartActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIStatisticsUserActionLog uiStatistics = (UIStatisticsUserActionLog) event.getComponent() ;
      UIUserActivitiesLogChart uiMultiChart = (UIUserActivitiesLogChart)uiStatistics.getSibling(UIUserActivitiesLogChart.class) ;
      uiStatistics.setRenderedSibling(UIUserActivitiesLogChart.class) ;
      Collection logChartList = uiStatistics.getChartData();
      uiMultiChart.updateUserLogDataSet(logChartList, uiStatistics.getChartStyle(), 
                                        uiStatistics.getMaxmin()) ;
    }
  }

}