/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.user.component;

import java.util.Map;
import java.util.ResourceBundle;

import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.portlet.PortletRequest;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.SessionContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UINode;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.organization.UserProfile;

/**
 * Sat, Jan 010, 2006 @ 11:16 
 * @author: Nguyen Quang Hung
 * @email:   nguyenkequanghung@yahoo.com
 * @version: $Id: UIPersonalInfoForm.java,v 1.14 2006/01/10 13:07:02 nqhung Exp $
 */
public class UIUserProfileAdmin extends UINode {
  final static public String[] PERSONAL_INFO_KEYS =  {
    "user.name.given", "user.name.family", "user.name.nickName", "user.bdate" , 
    "user.gender" , "user.employer" , "user.department",  "user.jobtitle", 
  } ;

 final static public String[] HOME_INFO_KEYS =  {
    "user.home-info.postal.name", "user.home-info.postal.street", 
    "user.home-info.postal.city", "user.home-info.postal.stateprov" ,
    "user.home-info.postal.postalcode", "user.home-info.postal.country",
    "user.home-info.telecom.mobile.number", "user.home-info.telecom.telephone.number", 
    "user.home-info.online.email", "user.home-info.online.uri" 
  } ;

 final static public String[] BUSINESS_INFO_KEYS = {
    "user.business-info.postal.name", "user.business-info.postal.city", 
    "user.business-info.postal.stateprov", "user.business-info.postal.postalcode",
    "user.business-info.postal.country", "user.business-info.telecom.mobile.number",
    "user.business-info.telecom.telephone.number", "user.business-info.online.email",
    "user.business-info.online.uri"
  } ;
 
  final static public String[] OTHER_KEYS = {
    "user.other-info.avatar.url", "user.other-info.signature", 
  } ;
  
  public UIUserProfileAdmin() throws Exception {
    setRendererType("VelocityNodeMenuRenderer") ;
    setTemplatePath("org/exoplatform/faces/core/renderer/html/UISecondTabLevel.vm") ;
    String userName = SessionContainer.getInstance().getOwner() ;
    ExternalContext eContext = FacesContext.getCurrentInstance().getExternalContext() ;
    //cannot edit profile if remote user is not owner
    if(!userName.equals(eContext.getRemoteUser())) return ;
    
    Map requestMap = eContext.getRequestMap() ;
    Map userInfoMap = (Map) requestMap.get(PortletRequest.USER_INFO) ;    
    // Other form
    UIOtherInfoForm uiOtherInfoForm = new UIOtherInfoForm() ;
    for (int i =0 ; i < OTHER_KEYS.length; i++) {
      String key = OTHER_KEYS[i] ;
      uiOtherInfoForm.updateInfo(i, (String)userInfoMap.get(key)) ;
    }    
    uiOtherInfoForm.setRendered(false) ;
    uiOtherInfoForm.addActionListener(SaveOtherInfoActionListener.class, SAVE_ACTION) ;
    getChildren().add(uiOtherInfoForm) ;
    //Account form
    UIAccountForm uiAccForm = new UIAccountForm() ;
    uiAccForm.customizeUpdateAccountForm() ;
    uiAccForm.setEditingUser(userName) ;
    uiAccForm.setRendered(false) ;
    getChildren().add(uiAccForm) ;
    // Business form
    UIBusinessInfoForm uiBusinessInfoForm = new UIBusinessInfoForm() ;
    for (int i =0 ; i < BUSINESS_INFO_KEYS.length; i++) {
      String key = BUSINESS_INFO_KEYS[i] ;
      uiBusinessInfoForm.updateBusinessInfo(i, (String)userInfoMap.get(key)) ;
    }    
    uiBusinessInfoForm.setRendered(false) ;
    uiBusinessInfoForm.addActionListener(SaveBusinessInfoActionListener.class, SAVE_ACTION) ;
    getChildren().add(uiBusinessInfoForm) ;
    // Personal form
    UIPersonalInfoForm uiPersonalForm = new UIPersonalInfoForm() ;
    for (int i =0 ; i < PERSONAL_INFO_KEYS.length; i++) {
      String key = PERSONAL_INFO_KEYS[i] ;
      uiPersonalForm.updatePersonalInfo(i, (String)userInfoMap.get(key)) ;
    }
    for (int i =0 ; i < HOME_INFO_KEYS.length; i++) {
      String key = HOME_INFO_KEYS[i] ;
      uiPersonalForm.updateHomeInfo(i, (String)userInfoMap.get(key)) ;
    }
    uiPersonalForm.setRendered(true) ;
    uiPersonalForm.addActionListener(SavePersonalInfoActionListener.class, SAVE_ACTION) ;
    getChildren().add(uiPersonalForm) ;
  }
  
  static public class SavePersonalInfoActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIPersonalInfoForm uiForm = (UIPersonalInfoForm) event.getSource() ;
      ResourceBundle res = getApplicationResourceBundle() ;
      Information iprovider = findInformationProvider(uiForm);      
      ExternalContext eContext = FacesContext.getCurrentInstance().getExternalContext() ;
      Map requestMap = eContext.getRequestMap() ;
      Map userInfoMap = (Map) requestMap.get(PortletRequest.USER_INFO) ;
      for (int i=0 ; i < UIUserProfileAdmin.PERSONAL_INFO_KEYS.length; i++) {
        String name = UIUserProfileAdmin.PERSONAL_INFO_KEYS[i] ;
        String value = uiForm.getPersonalFieldInput(i) ;        
        if(value == null || value.length() == 0) {
          userInfoMap.remove(name) ;
        } else {
          userInfoMap.put(name, value) ;
        }
      }
      for (int i=0 ; i < UIUserProfileAdmin.HOME_INFO_KEYS.length; i++) {
        String name = UIUserProfileAdmin.HOME_INFO_KEYS[i] ;
        String value = uiForm.getHomeFieldInput(i) ;        
        if(value == null || value.length() == 0) {
          userInfoMap.remove(name) ;
        } else {
          userInfoMap.put(name, value) ;
        }       
      }
      String userName = SessionContainer.getInstance().getOwner() ;
      OrganizationService service = 
        (OrganizationService)PortalContainer.getComponent(OrganizationService.class) ;
      UserProfile profile = service.getUserProfileHandler().findUserProfileByName(userName) ;
      profile.setUserInfoMap(userInfoMap) ;
      service.getUserProfileHandler().saveUserProfile(profile, true) ;
      iprovider.addMessage(new ExoFacesMessage(res.getString("UIPersonalInfoForm.msg.saved"))) ;
    }
  }
  
  static public class SaveBusinessInfoActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIBusinessInfoForm uiForm = (UIBusinessInfoForm) event.getSource() ;
      ResourceBundle res = getApplicationResourceBundle() ;
      Information iprovider = findInformationProvider(uiForm);
      ExternalContext eContext = FacesContext.getCurrentInstance().getExternalContext() ;
      Map requestMap = eContext.getRequestMap() ;
      Map userInfoMap = (Map) requestMap.get(PortletRequest.USER_INFO) ;
      for (int i=0 ; i < UIUserProfileAdmin.BUSINESS_INFO_KEYS.length; i++) {
        String name = UIUserProfileAdmin.BUSINESS_INFO_KEYS[i] ;
        String value = uiForm.getBusinessFieldInput(i) ;        
        if(value == null || value.length() == 0) {
          userInfoMap.remove(name) ;
        } else {
          userInfoMap.put(name, value) ;
        }       
      }
      String userName = SessionContainer.getInstance().getOwner() ;
      OrganizationService service = 
        (OrganizationService)PortalContainer.getComponent(OrganizationService.class) ;
      UserProfile profile = service.getUserProfileHandler().findUserProfileByName(userName) ;
      profile.setUserInfoMap(userInfoMap) ;
      service.getUserProfileHandler().saveUserProfile(profile, true) ;
      iprovider.addMessage(new ExoFacesMessage(res.getString("UIBusinessInfoForm.msg.saved"))) ;      
    }
  }
  
  static public class SaveOtherInfoActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIOtherInfoForm uiForm = (UIOtherInfoForm) event.getSource() ;
      ResourceBundle res = getApplicationResourceBundle() ;
      Information iprovider = findInformationProvider(uiForm);
      ExternalContext eContext = FacesContext.getCurrentInstance().getExternalContext() ;
      Map requestMap = eContext.getRequestMap() ;
      Map userInfoMap = (Map) requestMap.get(PortletRequest.USER_INFO) ;
      for (int i=0 ; i < UIUserProfileAdmin.OTHER_KEYS.length; i++) {
        String name = UIUserProfileAdmin.OTHER_KEYS[i] ;
        String value = uiForm.getFieldInput(i) ;        
        if(value == null || value.length() == 0) {
          userInfoMap.remove(name) ;
        } else {
          userInfoMap.put(name, value) ;
        }        
      }
      String userName = SessionContainer.getInstance().getOwner() ;
      OrganizationService service = 
        (OrganizationService)PortalContainer.getComponent(OrganizationService.class) ;
      UserProfile profile = service.getUserProfileHandler().findUserProfileByName(userName) ;
      profile.setUserInfoMap(userInfoMap) ;
      service.getUserProfileHandler().saveUserProfile(profile, true) ; 
      iprovider.addMessage(new ExoFacesMessage(res.getString("UIOtherInfoForm.msg.saved"))) ;
    }
  }
}
