 /**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.portletcontainer.test.filters;

import java.io.IOException;
import java.util.*;
import javax.portlet.PortletMode;
import javax.portlet.WindowState;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.commons.lang.StringUtils;
import org.exoplatform.container.StandaloneContainer;
import org.exoplatform.container.SessionContainer;
import org.exoplatform.frameworks.portletcontainer.portalframework.*;
import org.exoplatform.services.portletcontainer.pci.ActionInput;
import org.exoplatform.services.portletcontainer.pci.ActionOutput;
import org.exoplatform.services.portletcontainer.pci.EventInput;
import org.exoplatform.services.portletcontainer.pci.EventOutput;
import org.exoplatform.services.portletcontainer.pci.FragmentInput;
import org.exoplatform.services.portletcontainer.pci.FragmentOutput;
import org.exoplatform.services.portletcontainer.pci.RenderInput;
import org.exoplatform.services.portletcontainer.pci.RenderOutput;
import org.exoplatform.services.portletcontainer.pci.ResourceInput;
import org.exoplatform.services.portletcontainer.pci.ResourceOutput;
import org.exoplatform.services.portletcontainer.plugins.pc.Constants;

/**
 * Created by The eXo Platform SARL .
 *
 * @author <a href="mailto:lautarul@gmail.com">Roman Pedchenko</a>
 * @version $Id: PortletFilter.java 8554 2006-09-04 15:28:35Z sunman $
 */

public class PortletFilter implements Filter {

  public ThreadLocal frameworks = new ThreadLocal();
  public PortalFramework framework = null;

  // Session Replication
  HashMap session_info = new HashMap();
  public static final String session_identifier = "SID";
  public static final String portal_identifier = "PID";
  String portal_container_name = "";

  public String bson(String s) {
    if (s == null)
      return "";
    return s.replace("\"", "\\\"").replace("\r", "\\r").replace("\n", "\\n");
  }

  public HttpServletResponse createDummyResponse(HttpServletResponse original) {
    if (original.getClass().getName().equals("weblogic.servlet.internal.ServletResponseImpl")
        || original.getClass().getName().equals("com.ibm.ws.webcontainer.srt.SRTServletResponse")
//        || original.getClass().getName().equals("com.evermind.server.http.EvermindHttpServletResponse")
        ) {
      return new DummyResponse();
    } else
      return original;
  }

  public PortletInfo createPortletInfo(String plt) {
    PortletInfo portletinfo = new PortletInfo();
    portletinfo.portlet = plt;
    String[] pl = StringUtils.split(plt, "/");
    WindowID2 win = framework.getWindowID(plt);
    portletinfo.wid = win.getUniqueID();
    portletinfo.title = framework.getPortletDisplayName(plt);
    if (portletinfo.title == null)
      portletinfo.title = pl[1];
    // Create supported modes for each portlet
    portletinfo.mode = win.getPortletMode().toString();
    portletinfo.modes = "--";
    try {
      Iterator i = framework.getPortletModes(pl[0], pl[1]);
      String pmode = null;
      while (i.hasNext()) {
        PortletMode mode = (PortletMode) i.next();
        if (pmode == null)
          pmode = mode.toString();
        else
          pmode = pmode + "." + mode.toString();
      }
      portletinfo.modes = pmode;
    } catch (Exception e) {}
    // Create supported states for each portlet
    portletinfo.state = win.getWindowState().toString();
    portletinfo.states = "--";
    try {
      Iterator i = framework.getWindowStates(pl[0], pl[1]);
      String pstate = null;
      while (i.hasNext()) {
        WindowState state = (WindowState) i.next();
        if (pstate == null)
          pstate = state.toString();
        else
          pstate = pstate + "." + state.toString();
      }
      portletinfo.states = pstate;
    } catch (Exception e) {}
    return portletinfo;
  }

  public void init(FilterConfig filterConfig) {
  }

  public void doFilter(ServletRequest servletRequest,
      ServletResponse servletResponse, FilterChain filterChain)
      throws IOException, ServletException {

    HttpServletRequest httpRequest = (HttpServletRequest) servletRequest;
    HttpServletResponse httpResponse = (HttpServletResponse) servletResponse;
    HttpSession httpSession = httpRequest.getSession();
    ServletContext ctx = httpRequest.getSession().getServletContext();

    try {
      StandaloneContainer portalContainer = StandaloneContainer.getInstance();
      SessionContainer sessionContainer =
        (SessionContainer) portalContainer.getComponentInstanceOfType(SessionContainer.class);
      if (sessionContainer == null) // creating session container if there isn't any
        sessionContainer = portalContainer.createSessionContainer("sessioncontainer", "user");

      framework = (PortalFramework) frameworks.get();
      if (framework == null) {
        framework = new PortalFramework(portalContainer);
        frameworks.set(framework);
      }
      framework.init(httpSession);

      portal_container_name = framework.getPortalName();

      // --- this dummy http response is intended to avoid premature response commit on some AS-es
      HttpServletResponse dummyHttpResponse = createDummyResponse(httpResponse);

      framework.setParams(httpRequest, "text/html");

      // Session Replication
      int count = 0;
      String count2 = (String) httpSession.getAttribute("count");
      count2 += "+";
      session_info.clear();

      // actions processing

      String fragmentData = "";
      if (framework.getAction() == Constants.fragmentInt) {

        // processing fragment

        FragmentInput fragmentInput = framework.createFragmentInput(ctx);
        try {
          FragmentOutput o = framework.serveFragment(httpRequest, httpResponse, fragmentInput);
          try {
            fragmentData = new String(o.getContent());
          } catch (Exception oe) {}
        } catch (Exception e1) {
          System.out.println(" !!!!!!!!!!!! error invoking serveFragment in " + framework.getTarget() + ": " + e1);
          e1.printStackTrace();
          System.out.println(" !!!!!!!!!!!! trying to continue...");
        }
// nu uite sa schimbi portlet-mode si window-state daca se trebuie
      } else if (framework.getAction() == Constants.resourceInt) {

        // processing resource

        ResourceInput resourceInput = framework.createResourceInput(ctx);
        try {
          ResourceOutput o = framework.serveResource(httpRequest, httpResponse, resourceInput);
          try {
            String pout = new String(o.getContent());
            httpSession.setAttribute("resource", pout);
          } catch (Exception oe) {}
        } catch (Exception e1) {
          System.out.println(" !!!!!!!!!!!! error invoking serveResource in " + framework.getTarget() + ": " + e1);
          e1.printStackTrace();
          System.out.println(" !!!!!!!!!!!! trying to continue...");
        }
        filterChain.doFilter(servletRequest, servletResponse);
      } else if (framework.getAction() == Constants.actionInt) {

        // processing action

        ActionInput actionInput = framework.createActionInput(ctx);
        try {
          ActionOutput o = framework.processAction(httpRequest, dummyHttpResponse, actionInput);
        } catch (Exception e2) {
          System.out.println(" !!!!!!!!!!!! error processing action of portlet " + framework.getTarget() + ": " + e2);
          e2.printStackTrace();
          System.out.println(" !!!!!!!!!!!! trying to continue...");
        }
      }

      // processing events

      framework.dispatchEvents(ctx, httpRequest, dummyHttpResponse);

      // if redirect is requested we must render nothing

      if (framework.getRedirect() != null) {
        httpResponse.sendRedirect(framework.getRedirect());
        filterChain.doFilter(servletRequest, servletResponse);
        return;
      }

      // render phase

      // collecting portlets to render

      Iterator portlets2render = null;

      // if we need render specified portlets only
      if (framework.getAction() == Constants.renderInt) {
        ArrayList plts = null;
        String[] ps = servletRequest.getParameterValues("portletName");
        if (ps != null) {
          plts = new ArrayList();
          for (String s : ps)
            plts.add(s);
          httpSession.setAttribute("portletName", plts);
        } else {
          plts = (ArrayList) httpSession.getAttribute("portletName");
        }
        if (plts != null)
          portlets2render = plts.iterator();
        else {
          Set keys = framework.getPortletNames();
          portlets2render = keys.iterator();
        }
      }

      if (framework.getAction() == Constants.actionInt) {
        Set keys = framework.getPortletNames();
        portlets2render = keys.iterator();
      }

      // if resource url clicked we must render nothing all is done in the servlet
      if (framework.getAction() == Constants.resourceInt) {}
      else

      // if fragment url requested we must render changes portlets only
      if (framework.getAction() == Constants.fragmentInt) {
        StringBuffer fragmentAssembly = new StringBuffer("{\n  \"data\": \"" + bson(fragmentData) + "\",\n  \"portlets\": [\n");

        portlets2render = framework.getChanges().iterator();
        while (portlets2render.hasNext()) {
          count++;
          String portlet = (String) portlets2render.next();
          PortletInfo portletinfo = createPortletInfo(portlet);
          RenderInput renderInput = framework.createRenderInput(ctx, portlet);
          try {
            RenderOutput o = framework.render(httpRequest, dummyHttpResponse, renderInput);
            portletinfo.title = o.getTitle() + " {mode: " + portletinfo.mode + "; state: " +
              portletinfo.state + "}";

            try {
              portletinfo.out = new String(o.getContent());
            } catch (Exception oe) {
              portletinfo.out = "";
              System.out.println(" !!!!!!!!!!!! error getContent portlet " + portlet + ": " + oe);
            }

            fragmentAssembly.append("    {\n");
            fragmentAssembly.append("      \"wid\": \"" + bson(portletinfo.wid) + "\",\n");
            fragmentAssembly.append("      \"title\": \"" + bson(portletinfo.title) + "\",\n");
            fragmentAssembly.append("      \"mode\": \"" + bson(portletinfo.mode) + "\",\n");
            fragmentAssembly.append("      \"state\": \"" + bson(portletinfo.state) + "\",\n");
            fragmentAssembly.append("      \"content\": \"" + bson(portletinfo.out) + "\"\n");
            if (portlets2render.hasNext())
              fragmentAssembly.append("    },\n");
            else
              fragmentAssembly.append("    }\n");

            //Collecting session info
            HashMap  hm = o.getSessionMap();
            session_info.put(StringUtils.split(portlet, "/")[0], hm);

          } catch (Exception e1) {
            System.out.println(" !!!!!!!!!!!! error rendering portlet " + portlet + ": " + e1);
            e1.printStackTrace();
            System.out.println(" !!!!!!!!!!!! trying to continue...");
          }
        }
        httpSession.setAttribute("fragment", fragmentAssembly + "  ]\n}");
      } else if (portlets2render != null) {
        ArrayList portletinfos = new ArrayList();

        while (portlets2render.hasNext()) {
          count++;
          String portlet = (String) portlets2render.next();
          PortletInfo portletinfo = createPortletInfo(portlet);
          portletinfos.add(portletinfo);
          RenderInput renderInput = framework.createRenderInput(ctx, portlet);

          ArrayList myatr0 = (ArrayList) httpSession.getAttribute("myatr");
          boolean pltIncl = (framework.getPortletParams().containsKey("fis") &&
            (framework.getPortletParams().containsKey("n" + count + "n") &&
            framework.getPortletParams().get("n" + count + "n").equals("on")));
          if (!pltIncl && !framework.getPortletParams().containsKey("fis"))
            pltIncl = (myatr0 != null) && !myatr0.get(count - 1).equals("");

          if (!pltIncl) {
            portletinfo.title = portletinfo.title + " {mode: " + portletinfo.mode + "; state: " +
              portletinfo.state + "}";
            continue;
          }

          try {
            RenderOutput o = framework.render(httpRequest, dummyHttpResponse, renderInput);
            portletinfo.title = o.getTitle() + " {mode: " + portletinfo.mode + "; state: " +
              portletinfo.state + "}";

            try {
              portletinfo.out = new String(o.getContent());
            } catch (Exception oe) {
              portletinfo.out = "";
              System.out.println(" !!!!!!!!!!!! error getContent portlet " + portlet + ": " + oe);
            }

            //Collecting session info
            HashMap  hm = o.getSessionMap();
            session_info.put(StringUtils.split(portlet, "/")[0], hm);

          } catch (Exception e1) {
            System.out.println(" !!!!!!!!!!!! error rendering portlet " + portlet + ": " + e1);
            e1.printStackTrace();
            System.out.println(" !!!!!!!!!!!! trying to continue...");
          }

        }
        httpSession.removeAttribute("portletinfos");
        httpSession.setAttribute("portletinfos", portletinfos);
      }

    } catch (Exception e) {
      e.printStackTrace();
      httpSession.setAttribute("portletinfos", null);
      return;
    }

    filterChain.doFilter(servletRequest, servletResponse);

    // Session Replication
    session_info.put(session_identifier, httpSession.getId());
    session_info.put(portal_identifier, portal_container_name);
    Set set = session_info.keySet();
    try {
      SessionReplicator sr = new SessionReplicator();
//      sr.send(session_info);
    } catch (Exception e){
      //e.printStackTrace();
    }
  }

  public void destroy() {
  }

}
