package org.exoplatform.services.portletcontainer;

import java.util.Collection;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;

import javax.portlet.PortletMode;
import javax.portlet.WindowState;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.exoplatform.services.portletcontainer.pci.Input;
import org.exoplatform.services.portletcontainer.pci.ActionInput;
import org.exoplatform.services.portletcontainer.pci.ActionOutput;
import org.exoplatform.services.portletcontainer.pci.EventInput;
import org.exoplatform.services.portletcontainer.pci.EventOutput;
import org.exoplatform.services.portletcontainer.pci.RenderInput;
import org.exoplatform.services.portletcontainer.pci.RenderOutput;
import org.exoplatform.services.portletcontainer.pci.ResourceInput;
import org.exoplatform.services.portletcontainer.pci.ResourceOutput;
import org.exoplatform.services.portletcontainer.pci.FragmentInput;
import org.exoplatform.services.portletcontainer.pci.FragmentOutput;

/**
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/

/**
 * Created by the Exo Development team.
 * Author : Mestrallet Benjamin
 * benjmestrallet@users.sourceforge.net
 * Date: 10 nov. 2003
 * Time: 09:40:23
 */
public interface PortletContainerService {  

  public void setPortletContainerName(String containerName);

  public void setMajorVersion(int majorVersion);

  public void setMinorVersion(int minorVersion);

  public void setProperties(Map properties);

  public void setSupportedPortletModesWithDescriptions(Collection portletModes);

  public void setSupportedWindowStatesWithDescriptions(Collection windowStates);

  public Collection getSupportedPortletModes();

  public Collection getSupportedWindowStates();

  public Collection getSupportedPortletModesWithDescriptions();

  public Collection getSupportedWindowStatesWithDescriptions();

  public Collection getPortletModes(String portletAppName, String portletName, String markup);

  public boolean isModeSuported(String portletAppName, String portletName,
                                String markup, PortletMode mode);

  public Collection getWindowStates(String portletAppName, String portletName, String markup);

  public boolean isStateSupported(String portletAppName, String portletName,
                                String markup, WindowState state);

  public Map getAllPortletMetaData();

  public ResourceBundle getBundle(HttpServletRequest request,
                                  HttpServletResponse response,
                                  String portletAppName,
                                  String portletName,
                                  Locale locale)
      throws PortletContainerException;

  public void setPortletPreference(Input input, Map preferences) throws PortletContainerException;

  public Map getPortletPreference(Input input);

  public ActionOutput processAction(HttpServletRequest request,
                                    HttpServletResponse response,
                                    ActionInput input) 
      throws PortletContainerException;

  public RenderOutput render(HttpServletRequest request,
                             HttpServletResponse response,
                             RenderInput input)
      throws PortletContainerException;

  public ResourceOutput serveResource(HttpServletRequest request,
                             HttpServletResponse response,
                             ResourceInput input)
      throws PortletContainerException;
  
  public FragmentOutput serveFragment(HttpServletRequest request,
                             HttpServletResponse response,
                             FragmentInput input)
      throws PortletContainerException;

  public EventOutput processEvent(HttpServletRequest request,
                             HttpServletResponse response,
                             EventInput input)
      throws PortletContainerException;

  public void sendAttrs(HttpServletRequest request, HttpServletResponse response,
      Map attrs, String portletApplicationName) throws PortletContainerException;

}
