/**
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/
package org.exoplatform.services.portletcontainer.impl.servlet;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;

import org.apache.commons.logging.Log;
import org.exoplatform.container.ExoContainer;
import org.exoplatform.container.ExoContainerContext;
//import org.exoplatform.services.log.LogService;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.portletcontainer.PortletApplicationRegister;
import org.exoplatform.services.portletcontainer.pci.model.PortletApp;
import org.exoplatform.services.portletcontainer.pci.model.XMLParser;
import org.exoplatform.services.xml.resolving.SimpleResolvingService;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

//for validation
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;
import org.xml.sax.SAXException;


/**
 * Created by the Exo Development team.
 * Author : Mestrallet Benjamin
 * benjmestrallet@users.sourceforge.net
 * Date: 10 nov. 2003
 * Time: 12:58:52
 */
public class PortletApplicationListener implements ServletContextListener {
  public void contextInitialized(ServletContextEvent servletContextEvent) {
    ExoContainer manager = ExoContainerContext.getTopContainer();
//    LogService lservice = (LogService)manager.getComponentInstanceOfType(LogService.class) ;
    Log log = ExoLogger.getLogger("org.exoplatform.services.portletcontainer");
    ServletContext servletContext = servletContextEvent.getServletContext();
    log.info("DEPLOY PORTLET APPLICATION: " + servletContext.getServletContextName());
    log.debug("Real path : "+ servletContext.getRealPath(""));   
    InputStream is = null;
    String oldParser = System.getProperty("javax.xml.parsers.DocumentBuilderFactory") ;
    try {
      is = servletContext.getResourceAsStream("/WEB-INF/portlet.xml");
      
      if (is == null) {
        log.info("PORTLET CONFIGURATION IS NOT FOUND, IGNORE THE PACKAGE");
        return;
      }

      String spec = null;
      
      /*
      //new validation
      StringBuffer out = new StringBuffer();
      byte[] b = new byte[4096];
      for (int n; (n = is.read(b)) != -1;) {
          out.append(new String(b, 0, n));
      }
      String str = out.toString();
      int i1 = str.indexOf("portlet-app");
      int i2 = str.indexOf(">",i1);
      String s = str.substring(i1,i2);
      int res1 = s.indexOf("portlet-app_1_0.xsd");
      if (res1 > 0 ) spec = "1";
      int res2 = s.indexOf("portlet-app_2_0.xsd");
      if (res2 > 0 ) spec = "2";
      */
      
      String specErr1 = null;
      String specErr2 = null;

      if (spec == null) {
        //old validation
        ClassLoader cl = Thread.currentThread().getContextClassLoader();
        StreamSource source = null;      
        //long time = java.lang.System.currentTimeMillis();
        try {
//        validation 1-st portlet spec
          source = new StreamSource(servletContext.getResourceAsStream("/WEB-INF/portlet.xml"));
          SchemaFactory factory = SchemaFactory.newInstance(javax.xml.XMLConstants.W3C_XML_SCHEMA_NS_URI);
          StreamSource schemaFile = new StreamSource(cl.getResourceAsStream("portlet-app_1_0.xsd"));
          Schema schema = factory.newSchema(schemaFile);
          Validator validator = schema.newValidator();
          validator.validate(source);
          spec = "1";
        } catch (SAXException e) {
          specErr1 = e.getMessage();
        }
        //System.out.println(java.lang.System.currentTimeMillis()-time + " - time 1-st portlet spec validation");
        //time = java.lang.System.currentTimeMillis();
        try {
//        validation 2-nd portlet spec
          source = new StreamSource(servletContext.getResourceAsStream("/WEB-INF/portlet.xml"));
          SchemaFactory factory = SchemaFactory.newInstance(javax.xml.XMLConstants.W3C_XML_SCHEMA_NS_URI);
          StreamSource schemaFile = new StreamSource(cl.getResourceAsStream("portlet-app_2_0.xsd"));
          Schema schema = factory.newSchema(schemaFile);
          Validator validator = schema.newValidator();
          validator.validate(source);
          spec = "2";
        } catch (SAXException e) {
          specErr2 = e.getMessage();
        }      
        //System.out.println(java.lang.System.currentTimeMillis()-time + " - time 2-nd portlet spec validation");
      }

      is = servletContext.getResourceAsStream("/WEB-INF/portlet.xml");
      
      if ( spec != null ) {
        log.info("portlet.xml valid portlet spec " + spec);
        PortletApp portletApp = XMLParser.parse(is,spec.equalsIgnoreCase("2")?Boolean.TRUE:Boolean.FALSE);
        is = servletContext.getResourceAsStream("/WEB-INF/web.xml");
        Collection<String> roles = new ArrayList<String>();
        
        XPath xpath = XPathFactory.newInstance().newXPath();
        XPathExpression roleNameExp = xpath.compile("/web-app/security-role/role-name") ;
        DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
        SimpleResolvingService serviceXML = 
            (SimpleResolvingService) manager.getComponentInstanceOfType(SimpleResolvingService.class);
        builder.setEntityResolver(serviceXML.getEntityResolver());
        Document document = builder.parse(is);
        NodeList nodes = (NodeList) roleNameExp.evaluate(document , XPathConstants.NODESET);
        for (int i = 0; i <  nodes.getLength(); i++) {
          Node element = nodes.item(i);
          roles.add(element.getFirstChild().getNodeValue());        
        }
        log.info("  -- read: " + portletApp.getPortlet().size() + " portlets");
        PortletApplicationRegister service = 
          (PortletApplicationRegister) manager.getComponentInstanceOfType(PortletApplicationRegister.class);
        service.registerPortletApplication(servletContext, portletApp, roles);
      } else {
        log.error("The portlet.xml doesn't valid portlet spec 1 and 2");
        if (specErr1 != null )
          System.out.println("[ERROR] validation 1-st portlet spec: " + specErr1);
        if (specErr2 != null )
          System.out.println("[ERROR] validation 2-nd portlet spec: " + specErr2);
      }
    } catch (Exception e) {
      log.error("Cannot deploy " + servletContext.getServletContextName(), e);
    } finally {
      if(oldParser != null) {
        System.setProperty("javax.xml.parsers.DocumentBuilderFactory", oldParser) ;
      }
    }
  }

  public void contextDestroyed(ServletContextEvent servletContextEvent) {
    ServletContext servletContext = servletContextEvent.getServletContext();
    ExoContainer manager = ExoContainerContext.getTopContainer();
//    LogService lservice = (LogService)manager.getComponentInstanceOfType(LogService.class) ;
    Log log = ExoLogger.getLogger("org.exoplatform.services.portletcontainer");
    log.info("UNDEPLOY PORTLET APPLICATION: " + servletContext.getServletContextName());    
    try {
      PortletApplicationRegister service = 
        (PortletApplicationRegister) manager.getComponentInstanceOfType(PortletApplicationRegister.class);
      service.removePortletApplication(servletContext);
    } catch (Exception e) {
      log.error("UNDEPLOY PORTLET APPLICATION: " + e);
    }
  }
}