/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.test.web;

import java.util.*;

import org.exoplatform.test.web.unit.WebUnit;
import org.exoplatform.test.web.validator.WellFormedXhtmlValidator;
import com.meterware.httpunit.*;
/**
 * May 21, 2004
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: ExoWebClient.java,v 1.1 2004/10/11 23:36:03 tuan08 Exp $
 **/
public class ExoWebClient implements Runnable {
  static WellFormedXhtmlValidator wellFormedXhtmlValidator_ = new WellFormedXhtmlValidator() ;
  
  private String url_ ;
  private TestSuites suites_ ;
  public long interval_  = 0 ;
  
  public boolean validateWellFormedXhtml_ ;
  public boolean validateWebUnit_ ;
  private boolean printSummary_ ;
  private TimeListener timeListener_ ;
  private boolean error_ ;
  private boolean storeResult_ = false ;
  public boolean verbose_ = true;

  private SessionContext sessionContext_ ;
  private Stack  sessionStack = new Stack() ;
  
  public ExoWebClient(String name, String url) {
    url_ = url ;
    validateWebUnit_ = true ; 
    validateWellFormedXhtml_ = false ;
    printSummary_ = true ;
    newSessionContext(name) ;
    reset() ;
  }
 
  public SessionContext newSessionContext(String name) {
    sessionContext_ = new SessionContext() ;
    sessionContext_.webclient = new WebConversation();
    timeListener_ = new TimeListener() ;
    sessionContext_.webclient.addClientListener(timeListener_) ;
    sessionContext_.roles.put("guest", "guest") ;
    setName(name) ; 
    return sessionContext_ ;
  }
  
  public void reset() {
    sessionContext_.response = null ;
    error_ = false ;
    sessionStack.empty() ;
  }
  
  public String getName() { return   sessionContext_.name ; }
  public void   setName(String name) { 
    sessionContext_.name = name ;
    sessionContext_.attributes.put("client.name", name );
  }
  
  public String getHomePageURL() { return url_ ; }
  public void   setHomePageURL(String s) { url_ = s ; }
 
  public long  getInterval() { return interval_ ; }
  public void  setInterval(long interval) { interval_ = interval ; }
  
  public boolean isStoreResult() { return storeResult_ ; }
  public void    setStoreResult(boolean b) { storeResult_ = b ; }
  
  public Object getAttribute(String key) { return sessionContext_.attributes.get(key) ; }
  public void   setAttribute(String key , Object obj) { sessionContext_.attributes.put(key, obj) ;} 
  
  public boolean isVerbose()  { return verbose_ ; }
  public void  setVerbose(boolean b) { verbose_ = b ;}
  
  public Map getRoles() { return sessionContext_.roles ; }

  public Stack getSessionContextStack() { return sessionStack ; }
  
  public TestSuites  getTestSuites()  { return suites_ ; }
  public void setSuites(TestSuites  suites) { suites_ = suites ; }
  
  public WebClient getWebClient() { return sessionContext_.webclient ; }
  public void setWebClient(WebClient wc, boolean newWebClient) { 
  	sessionContext_.webclient = wc ; 
    if(newWebClient) {
      wc.addClientListener(timeListener_) ;
    }
  }

  public void setValidateWellFormedXhtml(boolean b) {  validateWellFormedXhtml_ = b ; }
  
  public void setValidateWebUnit(boolean b) {  validateWebUnit_ = b ; }
  
  public boolean getPrintSummary() { return printSummary_ ; }
  public void    setPrintSummary(boolean b) { printSummary_ = b ; }
  
  public boolean hasError() { return error_ ; }
  
  public WebResponse getResponse() { return sessionContext_.response ; }
  
  public String getResponseText() throws Exception {
    if(sessionContext_.response == null)  return "NO DATA AVAILABLE" ;
    return sessionContext_.response.getText() ; 
  }
  
  public SessionContext getSessionContext() { return sessionContext_ ; }
  public void setSessionContext(SessionContext context) { sessionContext_ = context ;}

  public void executeUnit(WebUnit webUnit) throws Exception {        
    if(verbose_) print("\n===============>> START UNIT: " +  webUnit.getName());    
    WebTable table  = null ;
    if(webUnit.getBlockId() != null) {
        table = sessionContext_.response.getFirstMatchingTable(WebTable.MATCH_ID, webUnit.getBlockId()) ;
    } 
    //check conidtions
    if(!webUnit.checkConditions(sessionContext_.response, table,  this)) return   ;
    sessionContext_.response = webUnit.execute(sessionContext_.response, table,this) ;
    webUnit.setBlockId(sessionContext_.response.getText()) ;
    int contentLength = sessionContext_.response.getText().length()  ;
    boolean malformed = false ;
    if(validateWellFormedXhtml_) {
      malformed = !wellFormedXhtmlValidator_.validate(sessionContext_.response, this) ;
    }
    boolean hasError = !webUnit.validate(sessionContext_.response, this) ;
    webUnit.log(timeListener_.getExecutionTime(), contentLength, hasError, malformed) ;
    if(hasError) {
      error_ = true ;
      suites_.getCurrentWebUnitSuite().setStatus(WebUnitSuite.ERROR_STATUS) ;
      webUnit.setStatus(WebUnit.STATUS_BROKEN) ;
    }
    if(storeResult_) webUnit.setLastResponseText(sessionContext_.response.getText()) ;       
    if(verbose_) print("<<=============== END UNIT: " +  webUnit.getName());
    return  ;
  }
  
  public void run()  { run(suites_) ; }
  
  //modified this
  public void runTo(TestSuites suites)  {
    WebUnit webUnit = null ;
    try {  
      suites_  = suites ;
      while(suites.nextUnit()) {
        webUnit = suites.getCurrentWebUnit() ;        
        executeUnit(webUnit) ;        
        if(webUnit.isStop()) return ;
      }
    } catch (InterruptedException ex) {
      Thread.currentThread().interrupt() ;
    } catch (Exception ex) {
      System.err.println("===================================================================================") ;
      if(webUnit != null) System.err.println(webUnit) ;
      ex.printStackTrace() ;
      System.err.println("===================================================================================") ;
    }
  }  
  
  public void run(TestSuites suites)  {
    WebUnit webUnit = null ;
    try {  
      suites_  = suites ;
      while(suites.nextUnit()) {
        webUnit = suites.getCurrentWebUnit() ;        
        executeUnit(webUnit) ;        
        if(interval_  > 0 ) Thread.sleep(interval_) ;
        if(Thread.interrupted()) return ;
      }
    } catch (InterruptedException ex) {
      Thread.currentThread().interrupt() ;
    } catch (Exception ex) {
      System.err.println("====================================================================================") ;
      if(webUnit != null) System.err.println(webUnit) ;
      ex.printStackTrace() ;
      System.err.println("====================================================================================") ;
    }
  }
  
  public ExoWebClient clone(String name) {
    ExoWebClient newClient = new ExoWebClient(name, url_) ;
    if(suites_ != null)  newClient.setSuites( suites_.softClone()) ;
    newClient.setInterval(interval_) ;
    newClient.setValidateWebUnit(validateWebUnit_) ;
    newClient.setValidateWellFormedXhtml(validateWellFormedXhtml_) ;
    newClient.setVerbose(verbose_) ;
    return newClient ;
  }

  private void print(String mesg) { System.out.println(mesg) ; }
  
  static public class SessionContext { 
    public WebResponse response ;
    WebClient webclient ;
    Map<String, Object> attributes;
    Map<String,String> roles ;
    String name ;
    
    public SessionContext() {
      attributes = new HashMap<String, Object>() ;
      roles = new HashMap<String, String>() ;
    }
  }
  
  class TimeListener implements WebClientListener {
    private long endTime_ ;
    private long startTime_ ;
    
    public long getExecutionTime() { return endTime_ - startTime_ ;}
    
    public void requestSent(com.meterware.httpunit.WebClient src, WebRequest req) {
      startTime_ = System.currentTimeMillis() ;
    }

    public void responseReceived(com.meterware.httpunit.WebClient src, WebResponse resp) {
      endTime_ = System.currentTimeMillis() ;
    }
  }
}
