/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.test.web;

import groovy.lang.GroovyClassLoader;
import groovy.lang.Writable;
import groovy.text.SimpleTemplateEngine;
import groovy.text.Template;

import java.io.InputStream;
import java.io.Writer;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
/**
 * Created by The eXo Platform SARL
 * Author : Tuan Nguyen
 *          tuan08@users.sourceforge.net
 * Dec 21, 2005
 */
abstract public class GroovyResourceManager {
  private ThreadLocal tlocal_ = new ThreadLocal() {
    protected synchronized Object initialValue() {
      return new  HashMap<String, Object> ();
    }
  };
  
  private SimpleTemplateEngine engine_ ;
  private Map<String, Template> templates_ ;
  private GroovyClassLoader classloader_ ;
  private boolean cacheTemplate_  = false ;
  
  public GroovyResourceManager(GroovyClassLoader classloader) {
    classloader_  = classloader ;
    engine_ = new SimpleTemplateEngine() ;
    templates_ = new HashMap<String, Template>() ;
  }
  
  public void setCacheTemplate(boolean b) { cacheTemplate_  = b ; }
  
  public void clearTemplateBinding() {
    Map binding = (Map)tlocal_.get() ;
    binding.clear() ;
  }
  
  public String include(String name) throws Exception {
    Map binding = (Map) tlocal_.get() ;
    Map localBinding = new HashMap(binding) ;
    Template template = getTemplate(name) ;
    String s = template.make(localBinding).toString() ;
    return s ;
  }
  
  public void merge(Writer writer, String tname, Map  binding) throws Exception {
    Template template = getTemplate(tname) ;
    Writable writable = template.make(binding) ;
    writable.writeTo(writer) ;
  }
  
  abstract protected InputStream getResourceAsStream(String name) throws Exception ;
  abstract protected URL getResource(String name) throws Exception ;
  abstract protected void addGroovyPath(GroovyClassLoader cl) throws Exception ;
  
  private Template  getTemplate(String id) throws Exception {
    Template template = templates_.get(id) ;
    if(template == null) {
      InputStream is = getResourceAsStream(id) ;
      byte[]  buf = new byte[is.available()] ;
      is.read(buf) ;
      String text =  new String(buf) ;
      template = engine_.createTemplate(text) ;
      if(cacheTemplate_) templates_.put(id, template) ;
    }
    return template ;
  }
  
  public Object getGroovyObject(String name) throws Exception {
    return  getGroovyObject(name, false);
  }
  
  public Object getGroovyObject(String name, boolean reload) throws Exception {
    System.out.println("load object " + name + " reload " + reload);
    GroovyClassLoader cl = classloader_ ;
    if(reload) {
      cl = new GroovyClassLoader(cl.getParent()) ;
      addGroovyPath(cl) ;
    }
    try {
      Class clazz = cl.loadClass(name) ;
      return clazz.newInstance() ;
    } catch(Exception ex) {
      
    }
    name = name.replace(".", "/") ;
    name =  name + ".groovy" ;
    URL  classURL = getResource(name) ;
    if(classURL == null) {
      classURL = cl.getResource(name) ;
    }
    Class clazz = cl.parseClass(classURL.openStream()) ;
    return clazz.newInstance() ;
  }
}