/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.admin;

import condition.* ;
import suites.share.*;
import suites.share.NewAccountSuite;
import suites.share.UserLoginSuite;
import suites.share.UserLogoutSuite;

import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
import org.exoplatform.test.web.unit.NewSessionUnit;
/**
 * Created by The eXo Platform SARL
 * Author : Dung Hoang
 *          dunghoang@exoplatform.com, jobdung@yahoo.com
 * Feb 14, 2006
 */
public class ImportExportSuite extends WebUnitSuite {
  static String DESC = "Go to Import/Export page and test all the basic actions" ;
  static String NAME = "ImportExportSuite" ;
  String COMMENT=null;
  
  public ImportExportSuite() {    
    super(NAME, DESC) ;
    init("/Home/Import") ;
  }  
  public ImportExportSuite(String path) {
    super(NAME, DESC) ;
    init(path) ;
  }  
  public void  init(path) {
    //Go to Import/Export
    goToPage(path) ;
    //Export Data
    //-----Export Organization Data   
    addWebUnit(
      new ClickURLLinkUnit("ExportOrganizationData", "Click on Export button").
      setURL("&uicomponent=UIExportData&op=exportData&objectId=organization")    
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("SelectServiceDataTab", "Click on Service Data tab").
      setURL("&uicomponent=UIBackupPortlet&op=selectNode&objectId=UIBackupServiceDataNode").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIBackupServiceData&op=refreshData"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("RefreshServiceData", "Click on refresh button").
      setURL("&uicomponent=UIBackupServiceData&op=refreshData").
      addValidator(new ExpectTextValidator("organization.zip"))
    ) ;
    //-----Export All Data
    addWebUnit(
      new ClickURLLinkUnit("SelectExportDataTab", "Click on Export Data tab").
      setURL("&uicomponent=UIBackupPortlet&op=selectNode&objectId=UIExportData").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIExportData&op=exportAllData"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ExportAllData", "Click on Export All button").
      setURL("&uicomponent=UIExportData&op=exportAllData")     
    ) ;
    //Check Data
    //-----Check User Data
    addWebUnit(
      new ClickURLLinkUnit("SelectUserDataTab", "Click on Service Data tab").
      setURL("&uicomponent=UIBackupPortlet&op=selectNode&objectId=UIBackupUserDataNode").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIBackupUserData&op=refreshData"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("RefreshUserData", "Click on refresh button").
      setURL("&uicomponent=UIBackupUserData&op=refreshData").
      addValidator(new ExpectTextValidator("admin.zip")).
      addValidator(new ExpectTextValidator("demo.zip")).
      addValidator(new ExpectTextValidator("exo.zip")).
      addValidator(new ExpectTextValidator("exotest.zip")).
      addValidator(new ExpectTextValidator("portal.zip"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ViewPortalData", "Click on view button").
      setURL("&uicomponent=UIBackupUserData&op=viewData&objectId=exoportal.zip").
      addValidator(new ExpectTextValidator("Data Info")).
      addValidator(new ExpectTextValidator("Name")).
      addValidator(new ExpectTextValidator("Creation Date")).
      addValidator(new ExpectTextValidator("Modification Date"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("Back", "Click on cancel button").
      setURL("&uicomponent=UIData&op=PhaseId[1].cancel")     
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("DeletePortalData", "Click on delete button").
      setURL("uicomponent=UIBackupUserData&op=deleteData&objectId=exoportal.zip").
      addValidator(new ExpectNoTextValidator("exoportal.zip"))
    ) ;    
    //-----Check Sevice Data
    addWebUnit(
      new ClickURLLinkUnit("SelectServiceDataTab", "Click on Service Data tab").
      setURL("&uicomponent=UIBackupPortlet&op=selectNode&objectId=UIBackupServiceDataNode").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIBackupServiceData&op=refreshData"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("RefreshData", "Click on refresh button").
      setURL("&uicomponent=UIBackupServiceData&op=refreshData").
      addValidator(new ExpectTextValidator("organization.zip"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ViewServiceData", "Click on view button").
      setURL("&uicomponent=UIBackupServiceData&op=viewData&objectId=organization.zip").
      addValidator(new ExpectTextValidator("Data Info")).
      addValidator(new ExpectTextValidator("Name")).
      addValidator(new ExpectTextValidator("Creation Date")).
      addValidator(new ExpectTextValidator("Modification Date"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("Back", "Click on cancel button").
      setURL("&uicomponent=UIData&op=PhaseId[1].cancel")     
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("DeleteServiceData", "Click on delete button").
      setURL("&uicomponent=UIBackupServiceData&op=deleteData&objectId=organization.zip").
      addValidator(new ExpectNoTextValidator("organization.zip"))
    ) ;
    //Upload Data
    addWebUnit(
      new ClickURLLinkUnit("ClickUploadDataTab", "Select on Upload Data tab").
      setURL("&uicomponent=UIBackupPortlet&op=selectNode&objectId=UIUploadData").
      addValidator(new ExpectFormIdValidator("uploadData"))      
    ) ;
  }
  public java.util.List getRequiredSuites() { 
  	String[] roles = ["user", "admin"] ;
   	return [new UserLoginSuite(CommonUnit.NAME_EXO_ADMIN, "exo", roles)] ;
  }
}