/**************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved. *
 * Please look at license.txt in info directory for more license detail.  *
 **************************************************************************/
package suites.categorybrowser;

import condition.* ;
import suites.portal.*;
import suites.share.*;
import suites.share.NewAccountSuite;
import suites.share.UserLoginSuite;
import suites.share.UserLogoutSuite;
import suites.share.AddPortletSuite;
import suites.share.CommonUnit;
import suites.ecm.user.UserAddFolderSuite;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;

/**
 * Created by The eXo Platform SARL
 * Author : HoaPham
 *          phamvuxuanhoa@yahoo.com
 * Feb 8, 2006
 */

static public class CategoryBrowserPortletSuite extends WebUnitSuite{
  static String DESC = "Test Category Portlet Browser" ;
  static String NAME = "CategoryBrowserPortletSuite" ;
  
  public CategoryBrowserPortletSuite(){       
    super(NAME, DESC) ;
    String[] roles = ["user"] ;
    addWebSuite(new AdminPrepareDataSuite());
    addWebSuite(new NewAccountSuite());    
    addWebSuite(new UserLoginSuite("#{client.name}","#{client.name}",roles));
    addWebSuite(new UserPrepareDataSuite());
    
    //Add CategoryBrowser portlet in to homepage    
    addWebUnit(CommonUnit.GO_HOME_UNIT) ;    
    addWebUnit(CommonUnit.CLICK_EDIT_PAGE_MODE) ;
    addWebUnit(
        new ClickURLLinkUnit("AddPortlet(onPage)", "tell web client click'addPortlet' on page").
        setURL("componentId=page&portal:action=addPortlet")        
    ) ;
    addWebSuite(new AddPortletSuite("cms-content","CategoryBrowser")) ;
    addWebUnit(CommonUnit.CLICK_SAVE_PAGE) ;
    addWebUnit(CommonUnit.CLICK_VIEW_MODE) ;        
    //expect some document in 'news' category
    addWebUnit(
        new ClickURLLinkUnit("Click 'news' category", "click'news' on CategoryBrowser Portlet").
        setURL("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/news").
        //sibling node of this node
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/news")).
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/sports")).
        //children node of this node
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/news/world")).
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/news/econormy")).
        //documents that reference this category
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewDocument&objectId=/cms/home/users/#{client.name}/site/en/articleName3")).
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UICategoryList&op=viewDocument&objectId=/cms/home/users/#{client.name}/test-folder/fileName1")).
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UICategoryList&op=viewDocument&objectId=/cms/home/users/#{client.name}/test-folder/fileName2")).
        //documents that is a child document of this categories
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UICategoryList&op=viewDocument&objectId=/cms/categories/cms/news/aChildFile"))
    ) ;
    //view a reference document
    addWebUnit(
        new ClickURLLinkUnit("ViewReferenceDocument", "click on document to view this  document").
        setURL("&uicomponent=UICategoryList&op=viewDocument&objectId=/cms/home/users/#{client.name}/site/en/articleName3").
        addValidator(new ExpectTextValidator("articleName3")).
        addValidator(new ExpectTextValidator("articleTitle3")).
        addValidator(new ExpectTextValidator("articleSummary3")).
        addValidator(new ExpectTextValidator("articleContent3")).
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDocumentContent&op=closeDocument"))
    );  
    
    //close this document
    addWebUnit(
        new ClickURLLinkUnit("CloseDocument", "click on close button to clode this  document").
        setURL("&uicomponent=UIDocumentContent&op=closeDocument").
        //sibling node of this node
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/news")).
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/sports")).
        //children node of this node
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/news/world")).
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/news/econormy")).
        //documents that reference this category
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewDocument&objectId=/cms/home/users/#{client.name}/site/en/articleName3")).
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UICategoryList&op=viewDocument&objectId=/cms/home/users/#{client.name}/test-folder/fileName1")).
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UICategoryList&op=viewDocument&objectId=/cms/home/users/#{client.name}/test-folder/fileName2")).
        //documents that is a child document of this categories
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UICategoryList&op=viewDocument&objectId=/cms/categories/cms/news/aChildFile"))
       ) ;
    //view a child document
    addWebUnit(
        new ClickURLLinkUnit("ViewChildrenDocument", "click on document to view this  document").
        setURL("uicomponent=UICategoryList&op=viewDocument&objectId=/cms/categories/cms/news/aChildFile").
        addValidator(new ExpectTextValidator("aChildFile")).                
        addValidator(new ExpectTextValidator("This is the child document of news")).
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIDocumentContent&op=closeDocument"))
    );  
    
    //close this document
    addWebUnit(
        new ClickURLLinkUnit("CloseDocument", "click on close button to clode this  document").
        setURL("&uicomponent=UIDocumentContent&op=closeDocument").
        //sibling node of this node
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/news")).
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/sports")).
        //children node of this node
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/news/world")).
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/news/econormy")).
        // documents that reference this category
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewDocument&objectId=/cms/home/users/#{client.name}/site/en/articleName3")).
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UICategoryList&op=viewDocument&objectId=/cms/home/users/#{client.name}/test-folder/fileName1")).
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UICategoryList&op=viewDocument&objectId=/cms/home/users/#{client.name}/test-folder/fileName2")).
        //documents that is a child document of this categories
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UICategoryList&op=viewDocument&objectId=/cms/categories/cms/news/aChildFile"))
       ) ;
    
    //--------------choose portlet preference---
    addWebUnit(
        new ClickURLLinkUnit("EditPortletReference", "click on edit portlet").
        setURL("/#{client.name}?portal:componentId=CategoryBrowser&portal:portletMode=edit").
        addValidator(new ExpectFormIdValidator("UIBrowserConfig"))
    );  
    
    addWebUnit(        
        new SubmitFormUnit("SelectTypeRefereceDocument", "select document type is reference document").
        setFormId("UIBrowserConfig").   
        setField("workspace","production").
        setField("path","/cms/categories/cms").
        setCheckBox("reference",true).
        setCheckBox("child",false).
        setField("op","save").                  
        addValidator(new ExpectTextValidator("Save successful"))
    ) ;    
    // view change---
    addWebUnit(
        new ClickURLLinkUnit("ViewPortletMode", "click on view portlet").
        setURL("/#{client.name}?portal:componentId=CategoryBrowser&portal:portletMode=view").        
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/news")).
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/sports")).
        addValidator(new ExpectTextValidator("No article in this category"))
    );
    
    addWebUnit(
        new ClickURLLinkUnit("Click'news'category", "click'news' on CategoryBrowser Portlet").
        setURL("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/news").
        //sibling node of this node
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/news")).
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/sports")).
        //children node of this node
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/news/world")).
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/news/econormy")).
        //documents that reference this category
//        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewDocument&objectId=/cms/home/users/#{client.name}/site/en/articleName3")).
//        addValidator(new ExpectLinkWithURLValidator("uicomponent=UICategoryList&op=viewDocument&objectId=/cms/home/users/#{client.name}/test-folder/fileName1")).
//        addValidator(new ExpectLinkWithURLValidator("uicomponent=UICategoryList&op=viewDocument&objectId=/cms/home/users/#{client.name}/test-folder/fileName2")).
        //No documents that is a child document of this categories 
        addValidator(new ExpectNoTextValidator("aChildFile"))
    ) ;    
    //  --------------Change portlet preference---
    addWebUnit(
        new ClickURLLinkUnit("EditPortletReference", "click on edit portlet").
        setURL("/#{client.name}?portal:componentId=CategoryBrowser&portal:portletMode=edit").
        addValidator(new ExpectFormIdValidator("UIBrowserConfig"))
    );  
    
    addWebUnit(        
        new SubmitFormUnit("SelectChildrenDocument", "select document type is chilren document").
        setFormId("UIBrowserConfig").   
        setField("workspace","production").
        setField("path","/cms/categories/cms").
        setCheckBox("reference",false).
        setCheckBox("child",true).
        setField("op","save").                  
        addValidator(new ExpectTextValidator("Save successful"))
    ) ;    
    // view change---
    addWebUnit(
        new ClickURLLinkUnit("ViewPortletMode", "click on view portlet").
        setURL("/#{client.name}?portal:componentId=CategoryBrowser&portal:portletMode=view").        
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/news")).
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/sports")).
        addValidator(new ExpectTextValidator("No article in this category"))
    );
    
    addWebUnit(
        new ClickURLLinkUnit("Click 'news' category", "click'news' on CategoryBrowser Portlet").
        setURL("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/news").
        //sibling node of this node
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/news")).
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/sports")).
        //children node of this node
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/news/world")).
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/news/econormy")).
        //No documents that reference this category
//        addValidator(new ExpectNoTextValidator("articleName3")).
//        addValidator(new ExpectNoTextValidator("fileName1")).
//        addValidator(new ExpectNoTextValidator("fileName2")).                
        //documents that is a child document of this categories
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UICategoryList&op=viewDocument&objectId=/cms/categories/cms/news/aChildFile"))
    ) ;
    
    // change to 'sports' category
    addWebUnit(
        new ClickURLLinkUnit("Choose'sports' category", "click'sports' on CategoryBrowser Portlet").
        setURL("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/sports").
        //sibling node of this node
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/news")).
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/sports")).        
        //children node of this node
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/sports/football")).
        //docment with type is nt:file that reference this category
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewDocument&objectId=/cms/home/users/#{client.name}/test-folder/fileName3"))        
    ) ;
    // choose a child's category of 'sports' category
    addWebUnit(
        new ClickURLLinkUnit("ChooseSubCategory(of'sports') ", "Click on subcategory on this category").
        setURL("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/sports/football").
        //sibling node of this node
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/sports/football")).
        //children node of this node
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/sports/football/uefa")).
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UICategoryList&op=viewCategory&objectId=/cms/categories/cms/sports/football/champions-league")).
        // no document with type 'exo:aticle' is references to this  category        
        addValidator(new ExpectTextValidator("No article in this category"))
    ) ;
    
    //------Delete test-folder
    goToPage("/Home/ecm") ;
    //in this portlet, test-folder node of this user is choosing
    //---back to user node
    addWebUnit(
        new ClickURLLinkUnit("BackToUserNode", "back to '#{client.name}' node").
        setURL("uicomponent=UIJCRExplorer&op=changeNode&objectId=/cms/home/users/#{client.name}")      
    ) ;    
    //-----Delete test-folder that  does exist
    addWebUnit(
        new ClickURLLinkUnit("DeleteExistedFolder", "Delete if test-folder is existed").
        setURL("&uicomponent=UIContentManager&op=removeNode&objectId=test-folder").
        addCondition(new HaveLinkWithURLCondition("&uicomponent=UIContentManager&op=removeNode&objectId=test-folder")).
        addValidator(new ExpectTextValidator("The node /cms/home/users/#{client.name}/test-folder has been removed."))
    ) ;  
    // Save session
    addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
    ) ;     
    //---check data in CategoryPortlet to make sure this change to aspect to this portlet
    addWebUnit(CommonUnit.GO_HOME_UNIT) ;       
    //go to 'news' category to check all document that refence to this category is removed 
    // user logout    
    addWebSuite(new UserLogoutSuite());
  }  
  public java.util.List getRequiredSuites(){    
    String[] roles = ["user", "admin"] ;
      return [new UserLoginSuite(CommonUnit.NAME_EXO_ADMIN, "exo", roles)] ;  }
  
}