/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.user;

import condition.* ;
import suites.share.* ;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
import suites.runner.ECMUserSuites;

/**
 * This suite depends on the AddFolderSuite one as the document is added in /test-folder
 *     1) Go to the test-folder
 *     2) Test selection the document type
 *     3) Add a new nt:file document
 *     4) Save session
 *     5) Return to workspace selection
 *
 * Created by The eXo Platform SARL
 * Author : Dung Hoang
 *          dunghoang@exoplatform.com, jobdung@yahoo.com
 * Dec 27, 2005
 */
public class UserAddDocumentSuite extends WebUnitSuite {
  static String DESC = "Go to Ecm Portlet User page and add a new document to test-folder" ;
  static String NAME = "UserAddDocumentSuite" ;
  String COMMENT  = "You must run UserAddFolderSuite before run this suite";
    
  public UserAddDocumentSuite() {    
    super(NAME, DESC) ;
    
    addWebSuite(new UserAddFolderSuite());        
    
    //Select Actions tab
    addWebUnit(
      new ClickURLLinkUnit("SelectActiontab", "Select Actions tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Actions").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=addDocument"))      
    ) ;
    
    //Click on Add document
    addWebUnit(
      new ClickURLLinkUnit("AddDocument", "click on Add Document button").
      setURL("uicomponent=UIToolbar&op=addDocument").    
      addValidator(new ExpectFormIdValidator("selectContentTypeForm")).
      addValidator(new ExpectFormIdValidator("contentForm"))
    ) ;
    
    //-----Test the document type selection
    addWebUnit(
      new SubmitFormUnit("SelectArticle", "select document type is Article").
      setFormId("selectContentTypeForm").   
      setField("contentType","exo:article").
      setField("op","changeType").
      addValidator(new ExpectTextValidator("Summary"))
    ) ;
    
    //-----Add new Nt:file document
    addWebUnit(
      new SubmitFormUnit("SelectFile", "select document type is File").
      setFormId("selectContentTypeForm").   
      setField("contentType","nt:file").
      setField("op","changeType").
      addValidator(new ExpectNoTextValidator("Summary"))      
    ) ;
    
    //----------Save New Document
    addWebUnit(
      new SubmitFormUnit("SaveNewDocument", "click on save button").
      setFormId("contentForm").   
      setField("contentName","test-doc").
      //setField("summary","content").      
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIContentManager&op=editDocument&objectId=/home/users/#{client.name}/test-folder/test-doc"))                                                   
    ) ;  
    if (UserECMFileExplorerSuite.isEnableJCR) {
      //Click on preferences icon
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;
      //Save session
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }         
    //Return to workspace selection
    addWebUnit(
      new ClickURLLinkUnit("SelectDrive", "select Drive button").
      setURL("&uicomponent=MainToolbar&op=selectDrive").                  
      addValidator(new ExpectTextValidator("Drives"))      
    ) ; 
    
  }                 
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user"] ;
    return [new NewAccountSuite(), new UserLoginSuite("#{client.name}", "#{client.name}", roles)] ;
  }  
}