/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.user;

import condition.* ;
import suites.share.* ;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
import suites.runner.ECMUserSuites;

/**
 * This suite does :
 *    1) Import Content Browse portlet
 *    2) Login with demo account, go to home/users/demo to prepare data 
 *        ( create test-folder, then create a nt:file and exo:article)
 *    3) Go back to current user and check some basic actions of path config,query config,script config,document config
 *    4) Login with account demo and delete 2 file test-doc 
 *  
 * Created by The eXo Platform SARL
 * Author : Hoa Nguyen Thi
 *          hoa.nguyen@exoplatform.com, hoa2501@yahoo.com
 * Apr 25, 2006
 */
public class UserContentBrowseSuite extends WebUnitSuite {
  static String DESC = "Add Content Browse portlet and to do somethings" ;
  static String NAME = "UserContentBrowseSuite" ;
  String COMMENT  = "You must run AdminManageTaxonomySuite, AdminViewSuite, AdminDriveSuite and PortletRegistrySuite before run this suite";
  
  public UserContentBrowseSuite() {    
    super(NAME, DESC) ;    
    goToPage("/home");
    
    //Import Content Browse portlet
    addWebUnit(CommonUnit.createClickEditPageMode()) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnAddPortlet", "Click on Add Portlet icon").
        setURL("portal:componentId=news&portal:action=addPortlet").                  
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIPortletSelecttor&op=selecCategory&objectId=cms-content"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOn'cms-content'category", "Expand Cms-Content category").
        setURL("uicomponent=UIPortletSelecttor&op=selecCategory&objectId=cms-content").   
        addCondition(new NoLinkWithURLCondition("uicomponent=UIPortletSelecttor&op=selecPortlet&objectId=cms-content/ContentBrowser")).
        addValidator(new ExpectTextValidator("ContentBrowser"))
    ) ;               
    addWebUnit(
        new ClickURLLinkUnit("ContentBrowser'portlet", "Select the Content Browser portlet").      
        setURL("uicomponent=UIPortletSelecttor&op=selecPortlet&objectId=cms-content/ContentBrowser").
        addValidator(new ExpectLinkWithURLValidator("portal:componentId=news&portal:action=changeTab&objectId=ContentBrowser"))        
    ) ;
    addWebUnit( CommonUnit.createClickSavePage()) ;
    addWebUnit( CommonUnit.createClickViewMode()) ;    
    addWebUnit(
        new ClickURLLinkUnit("ClickContentBrowser", "Click on ContentBrowser tab").
        setURL("portal:componentId=news&portal:action=changeTab&objectId=ContentBrowser").                
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIContentList&op=viewCategory&objectId=/jcr:system/exo:ecm/exo:taxonomies/cms"))
    ) ;      
    
    goToPage("/Home/ecm") ;
    addWebUnit(
      new ClickURLLinkUnit("SelectDrive", "select Workspace button").
      setURL("&uicomponent=MainToolbar&op=selectDrive").
      addCondition(new HaveLinkWithURLCondition("&uicomponent=MainToolbar&op=selectDrive")).
      addValidator(new ExpectTextValidator("Drives"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickOnDrive-test", "click on drive-test button").
      setURL("&uicomponent=UIDriveBrowser&op=selectWorkspace&objectId=drive-test").               
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=selectDrive"))      
    ) ;
    addWebUnit(
      new SubmitFormUnit("SelectCreatedNewView", "select created new view").
      setFormId("UISelectViewForm").            
      setField("views","view-test").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIViewController&op=selectNode&objectId=Admin"))      
    ) ;    
    addWebUnit(
      new ClickURLLinkUnit("SelectActiontab", "Select Actions tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Actions").               
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=addFolder"))      
    ) ;     
    addWebUnit(
      new ClickURLLinkUnit("ClickOnHome", "click on home link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users"))     
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnUsers", "click on Users link").
        setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users").                  
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/#{client.name}"))     
      ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOn #{client.name}", "click on #{client.name} link").
        setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/#{client.name}").                  
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/#{client.name}/site"))     
      ) ;
    //-----Add nt:unstructured folder
    addWebUnit(
      new ClickURLLinkUnit("DeleteExistedFolder", "Delete if test-folder is existed").
      setURL("&uicomponent=UIContentManager&op=removeNode&objectId=/home/users/#{client.name}/test-folder").
      addCondition(new HaveLinkWithTextCondition("test-folder")).
      addValidator(new ExpectTextValidator("The node /home/users/#{client.name}/test-folder has been removed."))
    ) ;            
    addWebUnit(
      new ClickURLLinkUnit("ClickOnAddFolder", "click on Add Folder button").
      setURL("&uicomponent=UIToolbar&op=addFolder").                  
      addValidator(new ExpectFormIdValidator("folderForm"))      
    ) ;        
    addWebUnit(
      new SubmitFormUnit("AddNewFolder", "Select type is nt:unstructured and click on Save button").
      setFormId("folderForm").   
      setField("folderType","nt:unstructured").
      setField("name","test-folder").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/#{client.name}/test-folder"))      
    ) ;
    if (UserECMFileExplorerSuite.isEnableJCR) {
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;      
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }     
    //-----Go to test-folder, add a nt:file and an exo:article
    addWebUnit(
      new ClickURLLinkUnit("Goto test-folder", "click on test-folder").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/#{client.name}/test-folder")      
    ) ;  
    addWebUnit(
      new ClickURLLinkUnit("SelectActiontab", "Select Actions tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Actions").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=addFolder"))      
    ) ;
    //----------Add a nt:file
    for(i in 1..10) {
      addWebUnit(
        new ClickURLLinkUnit("AddFile", "click on Add Document button").
        setURL("uicomponent=UIToolbar&op=addDocument").    
        addValidator(new ExpectFormIdValidator("selectContentTypeForm")).
        addValidator(new ExpectFormIdValidator("contentForm"))
      ) ;
      addWebUnit(
        new SubmitFormUnit("SelectFileType", "select document type is nt:file").
        setFormId("selectContentTypeForm").   
        setField("contentType","nt:file").
        setField("op","changeType").
        addValidator(new ExpectNoTextValidator("Image to upload")).
        addValidator(new ExpectTextValidator("File")).
        addValidator(new ExpectTextValidator("Content:"))      
      ) ;
      addWebUnit(
        new SubmitFormUnit("SaveNewFile", "click on save button").
        setFormId("contentForm").   
        setField("contentName","file"+i).
        //setField("summary","content").      
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=editDocument&objectId=/home/users/#{client.name}/test-folder/file"+i))
      ) ;          
    }
    //----------Add an exo:article
    for(i in 1..10) {
      addWebUnit(
        new ClickURLLinkUnit("AddArticle", "click on Add Document button").
        setURL("uicomponent=UIToolbar&op=addDocument").    
        addValidator(new ExpectFormIdValidator("selectContentTypeForm")).
        addValidator(new ExpectFormIdValidator("contentForm"))
      ) ;    
      addWebUnit(
        new SubmitFormUnit("SelectArticleType", "select document type is exo:article").
        setFormId("selectContentTypeForm").   
        setField("contentType","exo:article").
        setField("op","changeType").
        addValidator(new ExpectTextValidator("Summary")).
        addValidator(new ExpectNoTextValidator("ADMIN DIALOG"))
      ) ;   
      addWebUnit(
        new SubmitFormUnit("SaveNewArticle", "click on save button").
        setFormId("contentForm").   
        setField("contentName","article"+i).
        setField("title","this is article"+i).      
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=editDocument&objectId=/home/users/#{client.name}/test-folder/article"+i))
      ) ;
    }
    if (UserECMFileExplorerSuite.isEnableJCR) {      
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;      
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }     
    
    //Back to current user 
    //  search
    //  edit current config   
    //  add new query config
    //  add new script config
    //  add new document config
    //  add new path config             
    goToPage("/home");
    //-----search
    addWebUnit(
        new ClickURLLinkUnit("ClickOnSearchIcon", "Click on search icon").
        setURL("uicomponent=MainToolbar&op=search").   
        addValidator(new ExpectFormIdValidator("UISearchForm"))
    ) ;
   addWebUnit(
        new SubmitFormUnit("SearchNotExistCategory", "search with not exist category").
        setFormId("UISearchForm").   
        setField("type","Category").
        setField("keyword","notExistCategory").
        setField("op","searchAction").                  
        addValidator(new ExpectTextValidator("No result"))      
    ) ;
    addWebUnit(
        new SubmitFormUnit("SearchChildCategory", "search with exist childCategory").
        setFormId("UISearchForm").   
        setField("type","Category").
        setField("keyword","childCategory").
        setField("op","searchAction").                  
        addValidator(new ExpectTextValidator("childCategory")).      
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIResults&op=changeNode&objectId=/jcr:system/exo:ecm/exo:taxonomies/newCategory/childCategory"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnPathchildCategory", "Click on path of searched childCategory").
        setURL("uicomponent=UIResults&op=changeNode&objectId=/jcr:system/exo:ecm/exo:taxonomies/newCategory/childCategory").                  
        addValidator(new ExpectTextValidator("childCategory"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnSearchIcon", "Click on search icon").
        setURL("uicomponent=MainToolbar&op=search").                  
        addValidator(new ExpectFormIdValidator("UISearchForm"))
    ) ;
    
    //-----edit current config    
    //----------pathconfig
    addWebUnit(
        new ClickURLLinkUnit("ClickOnEditIcon", "Click on edit icon").
        setURL("portal:componentId=ContentBrowser&portal:portletMode=edit").                  
        addValidator(new ExpectFormIdValidator("PathConfig"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("ClickOnEditConfigIcon", "Click on 'Edit config' icon").
        setFormId("PathConfig").   
        setField("op","editConfig").                  
        addValidator(new ExpectFormIdValidator("PathConfig"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("ClickOnSaveIcon", "click on Save icon").
        setFormId("PathConfig").   
        setField("path","/cms").
        setField("op","save").
        addValidator(new ExpectTextValidator("Save successfully"))
    ) ;    
    addWebUnit(
        new ClickURLLinkUnit("ClickOnViewIcon", "Click on View icon").
        setURL("portal:componentId=ContentBrowser&portal:portletMode=view").                  
        addValidator(new ExpectTextValidator("Category"))
    ) ;    
    addWebUnit(
      new ClickURLLinkUnit("ClickOnSearchIcon", "Click on search icon").
      setURL("&uicomponent=MainToolbar&op=search").                  
      addValidator(new ExpectFormIdValidator("UISearchForm"))
    ) ;    
    addWebUnit(
      new SubmitFormUnit("Searchtest-docTitle", "search with exist title test-doc").
      setFormId("UISearchForm").   
      setField("type","Content").
      setField("keyword","article").
      setField("op","searchAction").                  
      addValidator(new ExpectTextValidator("Search Result"))
    ) ;
    COMMENT = "You can't set content by test script, so you can't search content";
    addWebUnit(
      new TODO("can'tSearchContent", COMMENT).
      setComment(COMMENT).
      setStatus(WebUnit.STATUS_INCOMPLETE)
    ) ;   
    //----------test-path
    addWebUnit(
        new ClickURLLinkUnit("ClickOnEditIcon", "Click on edit icon").
        setURL("portal:componentId=ContentBrowser&portal:portletMode=edit").                  
        addValidator(new ExpectFormIdValidator("PathConfig"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("ClickOnEditConfigIcon", "Click on 'Edit config' icon").
        setFormId("PathConfig").   
        setField("op","editConfig").                  
        addValidator(new ExpectFormIdValidator("PathConfig"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("ClickOnSaveIcon", "click on Save icon").
        setFormId("PathConfig").   
        setField("path","/home/users/#{client.name}/test-folder").    
        setField("template","PathList").
        setField("nbPerPage","100").        
        setField("op","save").
        addValidator(new ExpectTextValidator("Save successfully"))
    ) ;    
    addWebUnit(
        new ClickURLLinkUnit("ClickOnViewIcon", "Click on View icon").
        setURL("portal:componentId=ContentBrowser&portal:portletMode=view").                  
        addValidator(new ExpectTextValidator("article"))
    ) ;                
    //-----add new query config
    //----------query config by xpath
    addWebUnit(
        new ClickURLLinkUnit("ClickOnEditIcon", "Click on edit icon").
        setURL("portal:componentId=ContentBrowser&portal:portletMode=edit").                  
        addValidator(new ExpectFormIdValidator("PathConfig"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("ClickOnNewConfig", "Click on 'New Config' icon").
        setFormId("PathConfig").   
        setField("op","newConfig").
        addValidator(new ExpectFormIdValidator("NewConfigForm"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("SubmitQueryConfig", "Submit query config").
        setFormId("NewConfigForm").   
        setField("workspace","production").
        setField("usecase","Using a JCR query").
        setField("op","save").
        addValidator(new ExpectFormIdValidator("QueryConfig"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("SaveXpathQuery", "click on save xpath query").
        setFormId("QueryConfig").   
        setField("queryLanguage","xpath").
        setField("queryStatement","/jcr:root/cms//element(*, exo:article)").
        setField("template","QueryList").
        setField("nbPerPage","100").
        setField("op","save").
        addValidator(new ExpectTextValidator("Save successfully"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnViewIcon", "Click on view icon").
        setURL("portal:componentId=ContentBrowser&portal:portletMode=view").                  
        addValidator(new ExpectTextValidator("article1"))
    ) ;    
    //----------query config by sql
    addWebUnit(
        new ClickURLLinkUnit("ClickOnEditIcon", "Click on edit icon").
        setURL("portal:componentId=ContentBrowser&portal:portletMode=edit").                  
        addValidator(new ExpectFormIdValidator("QueryConfig"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("ClickOnEditConfigIcon", "Click on 'Edit config' icon").
        setFormId("QueryConfig").   
        setField("op","editConfig").                  
        addValidator(new ExpectFormIdValidator("QueryConfig"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("SaveSQLQuery", "click on save sql query").
        setFormId("QueryConfig").   
        setField("queryLanguage","sql").
        setField("queryStatement","select * from exo:article where jcr:path like '/home/users/#{client.name}/%'").
        setField("template","QueryList").
        setField("nbPerPage","100").
        setCheckBox("allowVoting",true).
        setCheckBox("allowComment",true).
        setField("op","save").
        addValidator(new ExpectTextValidator("Save successfully"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnViewIcon", "Click on view icon").
        setURL("portal:componentId=ContentBrowser&portal:portletMode=view").                  
        addValidator(new ExpectTextValidator("article"))
    ) ;        
    //-----add new script config     
    addWebUnit(
        new ClickURLLinkUnit("ClickOnEditIcon", "Click on edit icon").
        setURL("portal:componentId=ContentBrowser&portal:portletMode=edit").                  
        addValidator(new ExpectFormIdValidator("QueryConfig"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("ClickOnNewConfig", "Click on 'New Config' icon").
        setFormId("QueryConfig").   
        setField("op","newConfig").
        addValidator(new ExpectFormIdValidator("NewConfigForm"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("SubmitScriptConfig", "Submit script config").
        setFormId("NewConfigForm").   
        setField("workspace","production").
        setField("usecase","Using a script").
        setField("op","save").
        addValidator(new ExpectFormIdValidator("ScriptConfigForm"))
    ) ;    
    addWebUnit(
        new SubmitFormUnit("ClickOnSaveIcon", "click on save icon").
        setFormId("ScriptConfigForm").   
        setField("scriptList","content-browser/GetDocuments.groovy").
        setField("template","ScriptList").
        //setField("boxTemplate","test-detailView").
        setField("op","save").
        addValidator(new ExpectTextValidator("Save successfully"))
    ) ;    
    addWebUnit(
        new ClickURLLinkUnit("ClickOnViewIcon", "Click on view icon").
        setURL("portal:componentId=ContentBrowser&portal:portletMode=view")        
    ) ;   
    //----------test-script
    addWebUnit(
        new ClickURLLinkUnit("ClickOnEditIcon", "Click on edit icon").
        setURL("portal:componentId=ContentBrowser&portal:portletMode=edit").                  
        addValidator(new ExpectFormIdValidator("ScriptConfigForm"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("ClickOnEditConfigIcon", "Click on 'Edit config' icon").
        setFormId("ScriptConfigForm").   
        setField("op","editConfig").                  
        addValidator(new ExpectFormIdValidator("ScriptConfigForm"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("ClickOnSaveIcon", "click on save icon").
        setFormId("ScriptConfigForm").   
        setField("scriptList","content-browser/GetDocuments.groovy").
        setField("template","ScriptList").
        setField("op","save").
        addValidator(new ExpectTextValidator("Save successfully"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnViewIcon", "Click on view icon").
        setURL("portal:componentId=ContentBrowser&portal:portletMode=view")                  
        //addValidator(new ExpectTextValidator("test-script"))
    ) ; 
        
    //-----add new document config
    addWebUnit(
        new ClickURLLinkUnit("ClickOnEditIcon", "Click on edit icon").
        setURL("portal:componentId=ContentBrowser&portal:portletMode=edit").                  
        addValidator(new ExpectFormIdValidator("ScriptConfigForm"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("ClickOnNewConfig", "Click on 'New Config' icon").
        setFormId("ScriptConfigForm").   
        setField("op","newConfig").
        addValidator(new ExpectFormIdValidator("NewConfigForm"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("SubmitDocumentConfig", "Submit document config").
        setFormId("NewConfigForm").   
        setField("workspace","production").
        setField("usecase","Document").
        setField("op","save").
        addValidator(new ExpectFormIdValidator("DocumentConfig"))
    ) ;   
    addWebUnit(
        new SubmitFormUnit("SaveWithPathEmpty", "Save with path field empty").
        setFormId("DocumentConfig").   
        setField("path"," ").
        setField("documentName"," ").
        setField("op","save").
        addValidator(new ExpectTextValidator("A Repository Exception occured."))
    ) ;
    addWebUnit(
        new SubmitFormUnit("SaveWithDocumentNameEmpty", "Save with document name field empty").
        setFormId("DocumentConfig").   
        setField("path","/home/users/#{client.name}/test-folder").
        setField("documentName","").
        setField("op","save").
        addValidator(new ExpectTextValidator("Name of document must be offered."))
    ) ;
    addWebUnit(
        new SubmitFormUnit("SaveWithDon'tExistedDocument", "Save with documentName field is not existed document").
        setFormId("DocumentConfig").   
        setField("path","/home/users/#{client.name}/test-folder").
        setField("documentName","false-test-doc").
        setField("op","save").
        addValidator(new ExpectTextValidator("Path not found"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("ClickOnSaveIcon", "click on save icon").
        setFormId("DocumentConfig").   
        setField("path","/home/users/#{client.name}/test-folder").
        setField("documentName","article1").
        setField("op","save").
        addValidator(new ExpectTextValidator("Save successfully"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnViewIcon", "Click on view icon").
        setURL("portal:componentId=ContentBrowser&portal:portletMode=view").                  
        addValidator(new ExpectTextValidator("Content Browser"))
    ) ;    
    //----------test-document
    addWebUnit(
        new ClickURLLinkUnit("ClickOnEditIcon", "Click on edit icon").
        setURL("portal:componentId=ContentBrowser&portal:portletMode=edit").                  
        addValidator(new ExpectFormIdValidator("DocumentConfig"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("ClickOnEditConfigIcon", "Click on 'Edit config' icon").
        setFormId("DocumentConfig").   
        setField("op","editConfig").                  
        addValidator(new ExpectFormIdValidator("DocumentConfig"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("ClickOnSaveIcon", "click on save icon").
        setFormId("DocumentConfig").   
        setField("path","/home/users/#{client.name}/test-folder").
        setField("documentName","article1").
        //setField("boxTemplate","test-detailView").
        setField("op","save").
        addValidator(new ExpectTextValidator("Save successfully"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("ClickOnViewIcon", "Click on view icon").
        setURL("portal:componentId=ContentBrowser&portal:portletMode=view").                  
        addValidator(new ExpectTextValidator("this is article1"))
    ) ;   
    
    //-----add new path config
    addWebUnit(
        new ClickURLLinkUnit("ClickOnEditIcon", "Click on edit icon").
        setURL("portal:componentId=ContentBrowser&portal:portletMode=edit").                  
        addValidator(new ExpectFormIdValidator("DocumentConfig"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("ClickOnNewConfig", "Click on 'New Config' icon").
        setFormId("DocumentConfig").        
        setField("op","newConfig").
        addValidator(new ExpectFormIdValidator("NewConfigForm"))
    ) ;    
    addWebUnit(
        new SubmitFormUnit("SubmitConfig", "Submit query config").
        setFormId("NewConfigForm").   
        setField("workspace","production").
        setField("usecase","From Path").
        setField("op","save").
        addValidator(new ExpectFormIdValidator("PathConfig"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("ClickOnSaveIcon", "click on Save icon").
        setFormId("PathConfig").   
        setField("path","/jcr:system/exo:ecm/exo:taxonomies").
        setField("template","PathList").
        setField("op","save").
        addValidator(new ExpectTextValidator("Save successfully"))
    ) ;  
    addWebUnit(
        new ClickURLLinkUnit("ClickOnViewIcon", "Click on view icon").
        setURL("portal:componentId=ContentBrowser&portal:portletMode=view").                  
        addValidator(new ExpectTextValidator("Category"))
    ) ;        
    goToPage("/Home/ecm") ;    
    addWebUnit(
      new ClickURLLinkUnit("SelectDrive", "select Drive button").
      setURL("&uicomponent=MainToolbar&op=selectDrive").
      addCondition(new HaveLinkWithURLCondition("&uicomponent=MainToolbar&op=selectDrive")).
      addValidator(new ExpectTextValidator("Drives"))      
    ) ;
    
  }                 
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user"] ;
    return [new NewAccountSuite(), new UserLoginSuite("#{client.name}", "#{client.name}", roles)] ;
  }  
}
