/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.user;

import condition.* ;
import suites.share.* ;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
import suites.runner.ECMUserSuites;

/**
 * This suite does :
 *    1)Go to /jcr:root/home/users/demo
 *    2)Add new folder and go to this folder
 *    3)Add new action
 *    4)Add new document which has name is approve
 *    5)Add new document which has name is disapprove
 *    6)logout and login with account exoadmin 			
 *    7)Go to workflow 
 *    8)Go to task controler to test actions approve and disapprove
 *    9)Delete created action
 *    10)Delete test-folder
 *    
 * Created by The eXo Platform SARL 
 * Author : Dung Hoang
 *          dunghoang@exoplatform.com, jobdung@yahoo.com
 *          Hoa Nguyen Thi
 *          hoa2501@yahoo.com
 * Dec 27, 2005
 */
public class UserManageActionsSuite extends WebUnitSuite {
  static String DESC = "Go to Ecm Portlet User page, click on Info tab, click on Manage Action button and to do somethings" ;
  static String NAME = "UserManageActionsSuite" ;
  String COMMENT  = 
      "You must run AdminViewSuite and AdminDriveSuite before run this suite " +
      "This test suite login with demo account because: in line 169 and 383 you can't use #{client.name} " +
      "as jcr:root/home/users/#{client.name}/test-folder//element(*, exo:podcast)" ;
  
  public UserManageActionsSuite() {    
    super(NAME, DESC) ;          
    addWebUnit(
      new ContextManagementUnit(ContextManagementUnit.SAVE, "Save Context")
    );
        
    String[] roles = ["user"] ;
    addWebSuite(new UserLoginSuite("demo", "exo", roles)) ; 
    
    goToPage("/Home/ecm") ;   
    addWebUnit(
      new ClickURLLinkUnit("SelectDrive", "select Workspace button").
      setURL("&uicomponent=MainToolbar&op=selectDrive").
      addCondition(new HaveLinkWithURLCondition("&uicomponent=MainToolbar&op=selectDrive")).
      addValidator(new ExpectTextValidator("Drives"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickOnDrive-test", "click on drive-test button").
      setURL("&uicomponent=UIDriveBrowser&op=selectWorkspace&objectId=drive-test").                
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=selectDrive"))                                                   
    ) ;         
    //Select view
    addWebUnit(
      new SubmitFormUnit("SelectCreatedNewView", "select created new view").
      setFormId("UISelectViewForm").            
      setField("views","view-test").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIViewController&op=selectNode&objectId=Admin"))      
    ) ;
    //-----Select Actions tab
    addWebUnit(
      new ClickURLLinkUnit("SelectActiontab", "Select Actions tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Actions").               
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=addFolder"))      
    ) ; 
    //-----Goto /home/users/demo
   addWebUnit(
      new ClickURLLinkUnit("ClickOnHome", "click on home link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users"))     
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnUsers", "click on users link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/demo"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickOnDemo", "click on current user link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/demo")
    ) ;
    //-----Delete test-folder if does exist    
    addWebUnit(
      new ClickURLLinkUnit("DeleteExistedFolder", "Delete if test-folder is existed").
      setURL("&uicomponent=UIContentManager&op=removeNode&objectId=/home/users/demo/test-folder").
      addCondition(new HaveLinkWithTextCondition("test-folder")).
      addValidator(new ExpectTextValidator("The node /home/users/demo/test-folder has been removed."))
    ) ;            
    //-----Add folder directory
    addWebUnit(
      new ClickURLLinkUnit("ClickOnAddFolder", "click on Add Folder button").
      setURL("&uicomponent=UIToolbar&op=addFolder").                  
      addValidator(new ExpectFormIdValidator("folderForm"))      
    ) ;        
    //----------Add nt:unstructured folder
    addWebUnit(
      new SubmitFormUnit("Add Unstructured Folder", "Select type is nt:unstructured and click on Save button").
      setFormId("folderForm").   
      setField("folderType","nt:unstructured").
      setField("name","test-folder").
      setField("op","save").
      addValidator(new ExpectTextValidator("test-folder"))      
    ) ;
    if (UserECMFileExplorerSuite.isEnableJCR) {
      //Click on preferences icon
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;
      //Save session
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }         
    //-----Go to test-folder
    addWebUnit(
      new ClickURLLinkUnit("GotoTest-folder", "click on test-folder").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/demo/test-folder")      
    ) ;        
            
    addWebUnit(
      new ClickURLLinkUnit("SelectAdminTab", "click on Admin tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Admin").             
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=manageActions"))
    ) ;
    
    addWebUnit(
      new ClickURLLinkUnit("ClickOnManageActions", "click on Manage Actions button").
      setURL("&uicomponent=UIToolbar&op=manageActions").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIActionList&op=add"))      
    ) ;
                                  
    //RSS Action
    addWebUnit(
      new ClickURLLinkUnit("AddAction", "click on add button").
      setURL("&uicomponent=UIActionList&op=add").   
      addValidator(new ExpectFormIdValidator("selectActionForm")).
      addValidator(new ExpectFormIdValidator("actionForm"))            
    ) ;
    addWebUnit(
      new SubmitFormUnit("SelectExo:CreateRSSFeedAction", "select action of type is exo:createRSSFeedAction").
      setFormId("selectActionForm").
      setField("actionType","exo:createRSSFeedAction").
      setField("op","change")     
    ) ;    
    addWebUnit(
      new SubmitFormUnit("SaveFeedAction", "select action of type, fill all informations and click save button").
      setFormId("actionForm"). 
      setField("name","rssFeedName").
      setField("lifecycle","add").      
      setField("feed-name","rss-node-name-test").
      setField("store-path","/cms/feed").
      setField("queryPath","/jcr:root/home/users/demo/test-folder//element(*, exo:podcast)").                            
      setField("rss-title","rss-title-test").
      setField("feed-type","rss").
      setField("rss-version","rss_2.0").
      setField("link","rss-link-test").
      setField("description","rss-description-test").                             
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIActionList&op=view&objectId=rssFeedName"))          
    ) ;          
    addWebUnit(
      new ClickURLLinkUnit("SelectActiontab", "Select Actions tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Actions").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=addDocument"))      
    ) ;
    //------Add first document and check
    addWebUnit(
      new ClickURLLinkUnit("AddDocument", "click on Add Document button").
      setURL("uicomponent=UIToolbar&op=addDocument").    
      addValidator(new ExpectFormIdValidator("selectContentTypeForm")).
      addValidator(new ExpectFormIdValidator("contentForm"))
    ) ;    
    addWebUnit(
      new SubmitFormUnit("SaveNewDocument", "click on save button").
      setFormId("contentForm").   
      setField("contentName","doc-test").
      setField("title","title-test").      
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIContentManager&op=editDocument&objectId=/home/users/demo/test-folder/doc-test"))      
    ) ; 
    if (UserECMFileExplorerSuite.isEnableJCR) {
      //Click on preferences icon
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;
      //Save session
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }        
    addWebUnit(
      new ClickURLLinkUnit("ClickHome", "click on home link").
      setURL("&uicomponent=UIJCRExplorer&op=changeNode&objectId=/")
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("RefreshSession", "click on refresh Session button").
      setURL("&uicomponent=MainToolbar&op=refreshSession").              
      addValidator(new ExpectTextValidator("The session has been refreshed successfuly"))      
    ) ;    
    addWebUnit(
      new ClickURLLinkUnit("ClickCMS", "click on cms link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/cms").       
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed"))
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickFeed", "click on feed link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed").                 
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed/rss"))           
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickRSS", "click on rss link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed/rss").           
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed/rss/rss-node-name-test")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=checkin&objectId=/cms/feed/rss/rss-node-name-test",0))      
    ) ;  
    addWebUnit(
      new ClickURLLinkUnit("ClickOnBackToPreviousNode", "Click on Back To Previous Node").
      setURL("&uicomponent=MainToolbar&op=back").          
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed/rss"))                                                   
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnBackToPreviousNode", "Click on Back To Previous Node").
      setURL("&uicomponent=MainToolbar&op=back").          
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed"))
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnBackToPreviousNode", "Click on Back To Previous Node").
      setURL("&uicomponent=MainToolbar&op=back").          
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home"))                                                   
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnBackToPreviousNode", "Click on Back To Previous Node").
      setURL("&uicomponent=MainToolbar&op=back").          
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/demo/test-folder/doc-test"))      
    ) ; 
    //------Add second document and check
    addWebUnit(
      new ClickURLLinkUnit("AddDocument", "click on Add Document button").
      setURL("uicomponent=UIToolbar&op=addDocument").    
      addValidator(new ExpectFormIdValidator("selectContentTypeForm")).
      addValidator(new ExpectFormIdValidator("contentForm"))
    ) ;   
    addWebUnit(
      new SubmitFormUnit("SaveNewDocument", "click on save button").
      setFormId("contentForm").   
      setField("contentName","doc-test1").
      setField("title","title-test1").      
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIContentManager&op=editDocument&objectId=/home/users/demo/test-folder/doc-test1"))      
    ) ;  
    if (UserECMFileExplorerSuite.isEnableJCR) {
      //Click on preferences icon
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;
      //Save session
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }      
    addWebUnit(
      new ClickURLLinkUnit("ClickHome", "click on home link").
      setURL("&uicomponent=UIJCRExplorer&op=changeNode&objectId=/")
    ) ;         
    addWebUnit(
      new ClickURLLinkUnit("RefreshSession", "click on refresh Session button").
      setURL("&uicomponent=MainToolbar&op=refreshSession").              
      addValidator(new ExpectTextValidator("The session has been refreshed successfuly"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickCMS", "click on cms link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/cms").                 
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed"))           
    ) ;   
    addWebUnit(
      new ClickURLLinkUnit("ClickFeed", "click on feed link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed").                 
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed/rss"))           
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickRSS", "click on rss link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed/rss").           
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed/rss/rss-node-name-test")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=checkout&objectId=/cms/feed/rss/rss-node-name-test"))      
    ) ;  
    addWebUnit(
      new ClickURLLinkUnit("ClickOnBackToPreviousNode", "Click on Back To Previous Node").
      setURL("&uicomponent=MainToolbar&op=back").          
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed/rss"))                                                   
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnBackToPreviousNode", "Click on Back To Previous Node").
      setURL("&uicomponent=MainToolbar&op=back").          
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed"))
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnBackToPreviousNode", "Click on Back To Previous Node").
      setURL("&uicomponent=MainToolbar&op=back").          
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home"))                                                   
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnBackToPreviousNode", "Click on Back To Previous Node").
      setURL("&uicomponent=MainToolbar&op=back").          
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIContentManager&op=editDocument&objectId=/home/users/demo/test-folder/doc-test")).
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIContentManager&op=editDocument&objectId=/home/users/demo/test-folder/doc-test1"))      
    ) ;     
    //-----remove RSS Action
    addWebUnit(
      new ClickURLLinkUnit("SelectAdminTab", "click on Admin tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Admin").             
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=manageActions"))      
    ) ;    
    addWebUnit(
      new ClickURLLinkUnit("ClickOnManageActions", "click on Manage Actions button").
      setURL("&uicomponent=UIToolbar&op=manageActions").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIActionList&op=add"))      
    ) ;                              
    addWebUnit(
      new ClickURLLinkUnit("Delete rssFeedName action", "Delete rssFeedName action").
      setURL("&uicomponent=UIActionList&op=delete&objectId=rssFeedName").         
      addValidator(new ExpectNoTextValidator("rssFeedName"))      
    ) ;
    
    //PODCast Action
    addWebUnit(
      new ClickURLLinkUnit("AddAction", "click on add button").
      setURL("&uicomponent=UIActionList&op=add").   
      addValidator(new ExpectFormIdValidator("selectActionForm")).
      addValidator(new ExpectFormIdValidator("actionForm"))            
    ) ;
    addWebUnit(
      new SubmitFormUnit("SelectExo:CreateRSSFeedAction", "select action of type is exo:createRSSFeedAction").
      setFormId("selectActionForm").
      setField("actionType","exo:createRSSFeedAction").
      setField("op","change")     
    ) ;    
    addWebUnit(
      new SubmitFormUnit("SaveFeeddAction", "select action of type, fill all informations and click save button").
      setFormId("actionForm"). 
      setField("name","podcastFeedName").
      setField("lifecycle","add").      
      setField("feed-name","podcast-node-name-test").
      setField("store-path","/cms/feed").
      setField("queryPath","/jcr:root/home/users/demo/test-folder//element(*, exo:podcast)").        
      setField("rss-title","rss-title-test").
      setField("feed-type","podcast").
      setField("rss-version","rss_2.0").
      setField("link","rss-link-test").
      setField("description","rss-description-test").                             
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIActionList&op=view&objectId=podcastFeedName"))          
    ) ;          
    addWebUnit(
      new ClickURLLinkUnit("SelectActiontab", "Select Actions tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Actions").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=addDocument"))      
    ) ;
    //------Add first document and check
    addWebUnit(
      new ClickURLLinkUnit("AddDocument", "click on Add Document button").
      setURL("uicomponent=UIToolbar&op=addDocument").    
      addValidator(new ExpectFormIdValidator("selectContentTypeForm")).
      addValidator(new ExpectFormIdValidator("contentForm"))
    ) ;    
    addWebUnit(
      new SubmitFormUnit("SelectExo:podcast", "select document type is exo:podcast").
      setFormId("selectContentTypeForm").   
      setField("contentType","exo:podcast").
      setField("op","changeType").
      addValidator(new ExpectTextValidator("Podcast"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("SaveNewDocument", "click on save button").
      setFormId("contentForm").   
      setField("contentName","doc-test").
      setField("title","title-test").      
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIContentManager&op=editDocument&objectId=/home/users/demo/test-folder/doc-test"))      
    ) ; 
    if (UserECMFileExplorerSuite.isEnableJCR) {
      //Click on preferences icon
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;
      //Save session
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }      
    addWebUnit(
      new ClickURLLinkUnit("ClickHome", "click on home link").
      setURL("&uicomponent=UIJCRExplorer&op=changeNode&objectId=/")
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("RefreshSession", "click on refresh Session button").
      setURL("&uicomponent=MainToolbar&op=refreshSession").              
      addValidator(new ExpectTextValidator("The session has been refreshed successfuly"))      
    ) ;    
    addWebUnit(
      new ClickURLLinkUnit("ClickCMS", "click on cms link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/cms").       
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed"))
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickFeed", "click on feed link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed").                 
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed/podcast"))           
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickPodcast", "click on podcast link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed/podcast").           
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed/podcast/podcast-node-name-test")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=checkin&objectId=/cms/feed/podcast/podcast-node-name-test",0))      
    ) ;  
    addWebUnit(
      new ClickURLLinkUnit("ClickOnBackToPreviousNode", "Click on Back To Previous Node").
      setURL("&uicomponent=MainToolbar&op=back").          
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed/podcast"))                                                   
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnBackToPreviousNode", "Click on Back To Previous Node").
      setURL("&uicomponent=MainToolbar&op=back").          
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed"))
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnBackToPreviousNode", "Click on Back To Previous Node").
      setURL("&uicomponent=MainToolbar&op=back").          
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home"))                                                   
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnBackToPreviousNode", "Click on Back To Previous Node").
      setURL("&uicomponent=MainToolbar&op=back").          
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/demo/test-folder/doc-test"))      
    ) ; 
    //------Add second document and check
    addWebUnit(
      new ClickURLLinkUnit("AddDocument", "click on Add Document button").
      setURL("uicomponent=UIToolbar&op=addDocument").    
      addValidator(new ExpectFormIdValidator("selectContentTypeForm")).
      addValidator(new ExpectFormIdValidator("contentForm"))
    ) ;   
    addWebUnit(
      new SubmitFormUnit("SelectExo:podcast", "select document type is exo:podcast").
      setFormId("selectContentTypeForm").   
      setField("contentType","exo:podcast").
      setField("op","changeType").
      addValidator(new ExpectTextValidator("Podcast"))
    ) ;
    addWebUnit(
      new SubmitFormUnit("SaveNewDocument", "click on save button").
      setFormId("contentForm").   
      setField("contentName","doc-test1").
      setField("title","title-test1").      
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIContentManager&op=editDocument&objectId=/home/users/demo/test-folder/doc-test1"))      
    ) ;  
    if (UserECMFileExplorerSuite.isEnableJCR) {
      //Click on preferences icon
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;
      //Save session
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }      
    addWebUnit(
      new ClickURLLinkUnit("ClickHome", "click on home link").
      setURL("&uicomponent=UIJCRExplorer&op=changeNode&objectId=/")
    ) ;         
    addWebUnit(
      new ClickURLLinkUnit("RefreshSession", "click on refresh Session button").
      setURL("&uicomponent=MainToolbar&op=refreshSession").              
      addValidator(new ExpectTextValidator("The session has been refreshed successfuly"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickCMS", "click on cms link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/cms").                 
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed"))           
    ) ;   
    addWebUnit(
      new ClickURLLinkUnit("ClickFeed", "click on feed link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed").                 
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed/podcast"))           
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickPodcast", "click on podcast link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed/podcast").           
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed/podcast/podcast-node-name-test")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=checkout&objectId=/cms/feed/podcast/podcast-node-name-test"))      
    ) ;  
    addWebUnit(
      new ClickURLLinkUnit("ClickOnBackToPreviousNode", "Click on Back To Previous Node").
      setURL("&uicomponent=MainToolbar&op=back").          
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed/podcast"))                                                   
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnBackToPreviousNode", "Click on Back To Previous Node").
      setURL("&uicomponent=MainToolbar&op=back").          
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/feed"))
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnBackToPreviousNode", "Click on Back To Previous Node").
      setURL("&uicomponent=MainToolbar&op=back").          
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home"))                                                   
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnBackToPreviousNode", "Click on Back To Previous Node").
      setURL("&uicomponent=MainToolbar&op=back").          
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIContentManager&op=editDocument&objectId=/home/users/demo/test-folder/doc-test")).
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIContentManager&op=editDocument&objectId=/home/users/demo/test-folder/doc-test1"))      
    ) ;     
    //-----remove PODCast Action
    addWebUnit(
      new ClickURLLinkUnit("SelectAdminTab", "click on Admin tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Admin").             
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=manageActions"))      
    ) ;    
    addWebUnit(
      new ClickURLLinkUnit("ClickOnManageActions", "click on Manage Actions button").
      setURL("&uicomponent=UIToolbar&op=manageActions").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIActionList&op=add"))      
    ) ;                              
    addWebUnit(
      new ClickURLLinkUnit("Delete Podcast Feed action", "Delete Podcast Feed action").
      setURL("&uicomponent=UIActionList&op=delete&objectId=podcastFeedName").         
      addValidator(new ExpectNoTextValidator("podcastFeedName"))      
    ) ;       
    
    //Workflow Action
    addWebUnit(
      new ClickURLLinkUnit("AddAction", "click on add button").
      setURL("&uicomponent=UIActionList&op=add").   
      addValidator(new ExpectFormIdValidator("selectActionForm")).
      addValidator(new ExpectFormIdValidator("actionForm"))            
    ) ;  
                                   
    addWebUnit(
      new SubmitFormUnit("SelectWorkflowAction", "select action of type is exo:workflowAction").
      setFormId("selectActionForm").
      setField("actionType","exo:workflowAction").
      setField("op","change").  
      addValidator(new ExpectTextValidator("ContentValidation"))      
    ) ;
    
    //-----add Validator
    addWebUnit(
      new SubmitFormUnit("SearchValidator:", "click on search button").
      setFormId("actionForm").   
      setField("op","selectValidator").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIExoPermissionBrowser&op=selectGroup&objectId=/admin"))      
    ) ;  
          
    addWebUnit(
      new ClickURLLinkUnit("SelectGroup", "select a group").
      setURL("&uicomponent=UIExoPermissionBrowser&op=selectGroup&objectId=/admin").                  
      addValidator(new ExpectFormIdValidator("actionForm"))      
    ) ;
    
    //------Save Action
    addWebUnit(
      new SubmitFormUnit("SaveWorkflowAction", "select action of type, fill all informations and click save button").
      setFormId("actionForm"). 
      setField("name","workflowAction").
      setField("lifecycle","add").      
      setField("description","description-test").
      setField("validator","*:/admin").
      setField("destWorkspace","production").                
      setField("destPath","/cms/publications").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIActionList&op=view&objectId=workflowAction"))
    ) ; 
    if (UserECMFileExplorerSuite.isEnableJCR) {
      //Click on preferences icon
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;
      //Save session
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }     
    //Select Actions tab
    addWebUnit(
      new ClickURLLinkUnit("SelectActiontab", "Select Actions tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Actions").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=addDocument"))      
    ) ;
    
    //Click on Add document
    addWebUnit(
      new ClickURLLinkUnit("AddDocument", "click on Add Document button").
      setURL("uicomponent=UIToolbar&op=addDocument").    
      addValidator(new ExpectFormIdValidator("selectContentTypeForm")).
      addValidator(new ExpectFormIdValidator("contentForm"))
    ) ;
    
    //-----Add new Nt:file document
    addWebUnit(
      new SubmitFormUnit("SelectDocumentType", "select document type is nt:file").
      setFormId("selectContentTypeForm").   
      setField("contentType","nt:file").
      setField("op","changeType").
      addValidator(new ExpectTextValidator("File")).
      addValidator(new ExpectTextValidator("Content:"))
    ) ;
    
    //----------Save New Document
    addWebUnit(
      new SubmitFormUnit("SaveNewDocument", "click on save button").
      setFormId("contentForm").   
      setField("contentName","approve-doc").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIContentManager&op=editDocument&objectId=/home/users/demo/test-folder/approve-doc"))      
    ) ;
    if (UserECMFileExplorerSuite.isEnableJCR) {
      //Click on preferences icon
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;
      //Save session
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }     
    //Click on Add document
    addWebUnit(
      new ClickURLLinkUnit("AddDocument", "click on Add Document button").
      setURL("uicomponent=UIToolbar&op=addDocument").    
      addValidator(new ExpectFormIdValidator("selectContentTypeForm")).
      addValidator(new ExpectFormIdValidator("contentForm"))
    ) ;
    //-----Add new Nt:file document
    addWebUnit(
      new SubmitFormUnit("SelectDocumentType", "select document type is nt:file").
      setFormId("selectContentTypeForm").   
      setField("contentType","nt:file").
      setField("op","changeType").
      addValidator(new ExpectNoTextValidator("Image to upload")).
      addValidator(new ExpectTextValidator("File")).
      addValidator(new ExpectTextValidator("Content:"))
    ) ;
    //----------Save New Document
    addWebUnit(
      new SubmitFormUnit("SaveNewDocument", "click on save button").
      setFormId("contentForm").   
      setField("contentName","disapprove-doc").
      //setField("summary","content-disapprove").      
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIContentManager&op=editDocument&objectId=/home/users/demo/test-folder/disapprove-doc"))
    ) ;
    if (UserECMFileExplorerSuite.isEnableJCR) {
      //Click on preferences icon
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;
      //Save session
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }     
    addWebUnit(
      new ClickURLLinkUnit("ClickOnLogoutButton", "Tell web client click on logout button").
      setURL("&uicomponent=UILogout&op=logout").        
      addValidator(new ExpectFormIdValidator("loginForm"))       
    ) ; 
        
    String[] roleadmin = ["user", "admin"] ;
    addWebSuite(new UserLoginSuite("exoadmin", "exo", roleadmin)) ;   
    //----------Goto Workflow and to do something
    goToPage("/Home/Workflow") ;
    
    addWebUnit(
      new ClickURLLinkUnit("ClickOnTaskController", "click on task controller button").
      setURL("&uicomponent=null&op=selectNode&objectId=UITaskListController").                
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=TemplateRenderer&op=manageState&objectId="))      
    ) ; 
       
    addWebUnit(
      new ClickURLLinkUnit("ClickOnManage", "click on manage link").              
      setURL("&uicomponent=TemplateRenderer&op=manageState&objectId=").        
      addValidator(new ExpectFormIdValidator("taskForm"))            
    ) ;    
    addWebUnit(
      new SubmitFormUnit("ClickApproveIcon", "click on Approve icon to approve this action").
      setFormId("taskForm").   
      setField("op","approve").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=null&op=selectNode&objectId=UITaskListController"))
    ) ; 
    
    addWebUnit(
      new ClickURLLinkUnit("ClickOnManage", "click on manage link").
      setURL("&uicomponent=TemplateRenderer&op=manageState&objectId=").  
      addValidator(new ExpectFormIdValidator("taskForm"))            
    ) ;    
    addWebUnit(
      new SubmitFormUnit("ClickDisApproveIcon", "click on Disapprove icon to disapprove this action").
      setFormId("taskForm").   
      setField("op","disapprove").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=null&op=selectNode&objectId=UITaskListController"))
    ) ;  
    goToPage("/Home/ecm") ;
    addWebUnit(
      new ClickURLLinkUnit("SelectDrive", "select Drive button").
      setURL("&uicomponent=MainToolbar&op=selectDrive").
      addCondition(new HaveLinkWithURLCondition("&uicomponent=MainToolbar&op=selectDrive")).
      addValidator(new ExpectTextValidator("Drives"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickOnDrive-test", "click on drive-test button").
      setURL("&uicomponent=UIDriveBrowser&op=selectWorkspace&objectId=drive-test").               
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=selectDrive"))      
    ) ;  
    addWebUnit(
      new ClickURLLinkUnit("ClickOnCms", "click on home cms").
      setURL("uicomponent=UIContentManager&op=changeNode&objectId=/cms").             
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/publications"))           
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickOnPublications", "click on publications link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/publications").
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UIContentManager&op=removeNode&objectId=/cms/publications/approve-doc"))        
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("DeleteApprove-doc", "click on delete icon on row approve-doc").
      setURL("uicomponent=UIContentManager&op=removeNode&objectId=/cms/publications/approve-doc").   
      addValidator(new ExpectTextValidator("The node /cms/publications/approve-doc has been removed."))      
    ) ;    
    addWebUnit(
      new ClickURLLinkUnit("ClickOnLogoutButton", "Tell web client click on logout button").
      setURL("&uicomponent=UILogout&op=logout").        
      addValidator(new ExpectFormIdValidator("loginForm"))       
    ) ;     
    String[] userRoles = ["user"] ;
    addWebSuite(new UserLoginSuite("demo", "exo", userRoles)) ;
    
    goToPage("/Home/ecm") ;
//    addWebUnit(
//      new ClickURLLinkUnit("SelectDrive", "select Drive button").
//      setURL("&uicomponent=MainToolbar&op=selectDrive").                  
//      addValidator(new ExpectTextValidator("Drives"))      
//    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnDrive-test", "click on drive-test button").
      setURL("&uicomponent=UIDriveBrowser&op=selectWorkspace&objectId=drive-test").               
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=selectDrive"))      
    ) ;  
    //Select view
    addWebUnit(
      new SubmitFormUnit("SelectCreatedNewView", "select created new view").
      setFormId("UISelectViewForm").            
      setField("views","view-test").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIViewController&op=selectNode&objectId=Admin"))      
    ) ;  
    //-----Goto /home/users/demo
    addWebUnit(
      new ClickURLLinkUnit("ClickOnHome", "click on home link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users"))     
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnUsers", "click on users link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/demo"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickOnCurrentUser", "click on current user link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/demo").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/demo/test-folder"))
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickOnTest-folder", "click on test-folder").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/demo/test-folder")      
    ) ;    
    //-----Select Admin tab
    addWebUnit(
      new ClickURLLinkUnit("SelectAdminTab", "click on Admin tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Admin").             
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=manageActions"))      
    ) ;     
    //-----Manage Actions
    addWebUnit(
      new ClickURLLinkUnit("ClickOnManageActions", "click on Manage Actions button").
      setURL("&uicomponent=UIToolbar&op=manageActions").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIActionList&op=delete"))      
    ) ;
    //Delete created Action        
    addWebUnit(
      new ClickURLLinkUnit("DeleteCreatedAction", "delete created action").
      setURL("&uicomponent=UIActionList&op=delete&objectId=workflowAction")
    ) ;                   
    
    /*//Test action ( in this case, it is exo:transformBinaryToTextAction ) when you delete groovy script file
    //-----Create exo:transformBinaryToTextAction action
    addWebUnit(
      new ClickURLLinkUnit("ClickOnAddIcon", "click on Manage Action button").                   
      setURL("&uicomponent=UIActionList&op=add").     
      addValidator(new ExpectFormIdValidator("actionForm"))      
    ) ;
    addWebUnit(
      new SubmitFormUnit("SelectTransformBinarytoTextAction", "select action of type is exo:transformBinaryToTextAction").
      setFormId("selectActionForm").
      setField("actionType","exo:transformBinaryToTextAction").
      setField("op","change").  
      addValidator(new ExpectTextValidator("Transform Binary to Text"))      
    ) ;
    addWebUnit(
      new SubmitFormUnit("SaveForm", "click on save icon").
      setFormId("actionForm").            
      setField("name","transformBinaryToTextAction1").
      setField("lifecycle","read").
      setField("description","description1").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIActionList&op=delete&objectId=transformBinaryToTextAction1"))      
    ) ;  
    addWebUnit(
      new ClickURLLinkUnit("ClickOnDemo", "click on demo link").
      setURL("&uicomponent=UIJCRExplorer&op=changeNode&objectId=/home/users/demo").     
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=custom&actionName=transformBinaryToTextAction1&objectId=test-folder"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ExcecuteAction", "click on transformBinaryToTextAction1 link").
      setURL("&uicomponent=UIContentManager&op=custom&actionName=transformBinaryToTextAction1&objectId=test-folder").     
      addValidator(new ExpectTextValidator("The custom action transformBinaryToTextAction1 has been executed."))      
    ) ;          
    //-----Logout and login with exoadmin account, then delete TransformBinaryChildrenToTextScript.groovy script
    addWebUnit(
      new ClickURLLinkUnit("ClickOnLogoutButton", "Tell web client click on logout button").
      setURL("&uicomponent=UILogout&op=logout").        
      addValidator(new ExpectFormIdValidator("loginForm"))       
    ) ;                 
    addWebSuite(new UserLoginSuite("exoadmin", "exo", roleadmin)) ;   
    goToPage("/Home/ecm") ;    
    addWebUnit(
      new ClickURLLinkUnit("SelectAdminTheECM", "Select Admin The ECM tab").
      setURL("componentId=ECMPage&portal:action=changeTab&objectId=ecm").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIECMAdminSummary&op=manageTemplates"))      
    ) ;      
    addWebUnit(
      new ClickURLLinkUnit("ManageScripts", "Click on Manage Scripts button").
      setURL("&uicomponent=UIECMAdminSummary&op=manageScripts").                  
      addValidator(new ExpectLinkWithURLValidator("uicomponent=UINode&op=selectNode&objectId=CBScriptManager"))      
    ) ;   
    addWebUnit(
      new ClickURLLinkUnit("ClickOnECMScriptManage", "Click On ECM ScriptManage").
      setURL("&uicomponent=UINode&op=selectNode&objectId=ECMScriptManager")                                                             
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("DeleteTransformScript", "Click On Delete icon").
      setURL("&uicomponent=UIECMScriptList&op=delete&objectId=TransformBinaryChildrenToTextScript.groovy").
      addCondition(new HaveLinkWithURLCondition("&uicomponent=UIECMScriptList&op=delete&objectId=TransformBinaryChildrenToTextScript.groovy")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIECMScriptList&op=delete&objectId=TransformBinaryChildrenToTextScript.groovy", 0))                                                       
    ) ; 
    //-----Back to demo user and check what is wrong after you delete TransformBinaryChildrenToTextScript.groovy script    
    addWebUnit(
      new ClickURLLinkUnit("ClickOnLogoutButton", "Tell web client click on logout button").
      setURL("&uicomponent=UILogout&op=logout").        
      addValidator(new ExpectFormIdValidator("loginForm"))       
    ) ;         
    addWebSuite(new UserLoginSuite("demo", "exo", userRoles)) ;    
    goToPage("/Home/ecm") ;
    addWebUnit(
      new ClickURLLinkUnit("SelectDrive", "select Drive button").
      setURL("&uicomponent=MainToolbar&op=selectDrive").                  
      addValidator(new ExpectTextValidator("Drives"))      
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnDrive-test", "click on drive-test button").
      setURL("&uicomponent=UIDriveBrowser&op=selectWorkspace&objectId=drive-test").               
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=selectDrive"))      
    ) ;  
    addWebUnit(
      new SubmitFormUnit("SelectCreatedNewView", "select created new view").
      setFormId("UISelectViewForm").            
      setField("views","view-test").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIViewController&op=selectNode&objectId=Admin"))      
    ) ;  
    addWebUnit(
      new ClickURLLinkUnit("ClickOnHome", "click on home link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users"))     
    ) ; 
    addWebUnit(
      new ClickURLLinkUnit("ClickOnUsers", "click on users link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/demo"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickOnCurrentUser", "click on current user link").
      setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/demo").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/demo/test-folder"))
    ) ;    
    addWebUnit(
      new ClickURLLinkUnit("ExcecuteAction", "click on transformBinaryToTextAction1 link").
      setURL("&uicomponent=UIContentManager&op=custom&actionName=transformBinaryToTextAction1&objectId=test-folder")
    ) ;        
    addWebUnit(
      new ClickURLLinkUnit("DeleteTestFolder", "Delete test folder").
      setURL("uicomponent=UIContentManager&op=removeNode&objectId=/home/users/demo/test-folder").     
      addValidator(new ExpectTextValidator("The node /home/users/demo/test-folder has been removed."))      
    ) ;     */
    //Select Drive
    addWebUnit(
      new ClickURLLinkUnit("SelectDrive", "select Workspace button").
      setURL("&uicomponent=MainToolbar&op=selectDrive").                  
      addValidator(new ExpectTextValidator("Drives"))      
    ) ;       
    //Logout
    addWebUnit(
      new ClickURLLinkUnit("ClickOnLogoutButton", "Tell web client click on logout button").
      setURL("&uicomponent=UILogout&op=logout").        
      addValidator(new ExpectFormIdValidator("loginForm"))       
    ) ;
    //-----Back to current user
    addWebUnit(
      new ContextManagementUnit(ContextManagementUnit.RESTORE, "Restore Context")
    );     
  }                 
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user"] ;
    return [new NewAccountSuite(), new UserLoginSuite("#{client.name}", "#{client.name}", roles)] ;
  }  
}