/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.user;

import condition.* ;
import suites.share.* ;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;


/**
 * This suite does :    
 *   1)Login with account exoadmin, go to cms/publication and add relationDoc node
 *   2)Go back to current user and create /jcr:root/home/users/#{client.name}/test-folder/test-doc    
 *   3)Add test-doc's relation is relationDoc, check infomation in relationDoc
 *   4)Login with account exoadmin, go to cms/publication and rename relationDoc to newRelationDoc
 *   5)Go back current user, go to /jcr:root/home/users/#{client.name}/test-folder/test-doc, 
 *      check infomation in relationDoc          
 *   6)Login with account exoadmin and delete relationDoc
 *   7)Go back current user
 *   
 * Created by The eXo Platform SARL
 * Author :Hoa Nguyen Thi 
 *         hoa2501@yahoo.com
 * Apr 12, 2006
 */
public class UserRelationSuite extends WebUnitSuite {
  static String DESC = "Go to Ecm Portlet User page, click on test-folder and to do somethings" ;
  static String NAME = "UserRelationSuite" ;
  String COMMENT  = "You must run AdminViewSuite and AdminDriveSuite before run this suite";
  
  public UserRelationSuite() {    
    super(NAME, DESC) ; 
    addWebUnit(
      new ContextManagementUnit(ContextManagementUnit.SAVE, "Save Context")
    );
                        //-----Login with admin account    
    String[] roles = ["user", "admin"] ;
    addWebSuite(new UserLoginSuite("exoadmin", "exo", roles)) ; 
    
    goToPage("/Home/ecm") ;
    addWebUnit(
      new ClickURLLinkUnit("SelectDrive", "select Drive button").
      setURL("&uicomponent=MainToolbar&op=selectDrive").
      addCondition(new HaveLinkWithURLCondition("&uicomponent=MainToolbar&op=selectDrive")).
      addValidator(new ExpectTextValidator("Drives"))      
    ) ;
    addWebUnit(
      new ClickURLLinkUnit("ClickOnDrive-test", "click on drive-test button").
      setURL("&uicomponent=UIDriveBrowser&op=selectWorkspace&objectId=drive-test").               
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=selectDrive"))      
    ) ;  
    //Select view
    addWebUnit(
      new SubmitFormUnit("SelectCreatedNewView", "select created new view").
      setFormId("UISelectViewForm").            
      setField("views","view-test").
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIViewController&op=selectNode&objectId=Admin"))      
    ) ;
    
    //Goto publications folder and create relationDoc with nodetype exo:article
	addWebUnit(
        new ClickURLLinkUnit("ClickOnCms", "click on home cms").
        setURL("uicomponent=UIContentManager&op=changeNode&objectId=/cms").             
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/publications"))           
    ) ;    
    addWebUnit(
        new ClickURLLinkUnit("ClickOnPublications", "click on publications link").
        setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/publications").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIViewController&op=selectNode&objectId=Actions"))
    ) ;
    addWebUnit(
        new ClickURLLinkUnit("DeleteRelationDoc", "click on delete icon if relationDoc existed").
        setURL("&uicomponent=UIContentManager&op=removeNode&objectId=/cms/publications/relationDoc").
        addCondition(new HaveLinkWithURLCondition("&uicomponent=UIContentManager&op=removeNode&objectId=/cms/publications/relationDoc")).    		
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=removeNode&objectId=/cms/publications/relationDoc", 0))
    ) ;
    //----------Add new Doc has type is exo:article
    addWebUnit(
        new ClickURLLinkUnit("SelectActiontab", "Select Actions tab").
        setURL("&uicomponent=UIViewController&op=selectNode&objectId=Actions").                  
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=addDocument"))      
    ) ;
    
    addWebUnit(
        new ClickURLLinkUnit("AddNewDocument", "Click on add document icon").
        setURL("uicomponent=UIToolbar&op=addDocument").               
        addValidator(new ExpectFormIdValidator("contentForm"))      
    ) ; 
    
    addWebUnit(
        new SubmitFormUnit("SelectDocumentType", "select document type is exo:article").
        setFormId("selectContentTypeForm").   
        setField("contentType","exo:article").
        setField("op","changeType").
        addValidator(new ExpectTextValidator("Article"))
    ) ;
    
    addWebUnit(
        new SubmitFormUnit("SaveNewDocument", "click on save button").
        setFormId("contentForm").   
        setField("contentName","relationDoc").
        setField("title","relationDoc").
        setField("op","save")
        
    ) ;
    if (UserECMFileExplorerSuite.isEnableJCR) {
      //Click on preferences icon
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;
      //Save session
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }     
                  //-----Back to current user   
    addWebUnit(
      new ContextManagementUnit(ContextManagementUnit.RESTORE, "Restore Context")
    );
    addWebSuite(new UserAddFolderSuite());    
    //Create acrticle Document
     addWebUnit(
        new ClickURLLinkUnit("AddNewDocument", "Click on add document icon").
        setURL("uicomponent=UIToolbar&op=addDocument").               
        addValidator(new ExpectFormIdValidator("contentForm"))      
    ) ; 
    addWebUnit(
        new SubmitFormUnit("SelectDocumentType", "select document type is exo:article").
        setFormId("selectContentTypeForm").   
        setField("contentType","exo:article").
        setField("op","changeType").
        addValidator(new ExpectTextValidator("Article"))
    ) ;
    addWebUnit(
        new SubmitFormUnit("SaveNewDocument", "click on save button").
        setFormId("contentForm").   
        setField("contentName","test-doc").
        setField("title","test-doc").
        setField("op","save").
        addValidator(new ExpectTextValidator("test-doc"))
    ) ;
    if (UserECMFileExplorerSuite.isEnableJCR) {
      //Click on preferences icon
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;
      //Save session
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }     
    // Goto test-doc and add relation
    addWebUnit(
         new ClickURLLinkUnit("SelectTest-doc ", "Click on test-doc link").
         setURL("uicomponent=UIContentManager&op=changeNode&objectId=/home/users/#{client.name}/test-folder/test-doc").        
         addValidator(new ExpectTextValidator("test-doc"))      
     ) ;
     
    addWebUnit(
        new ClickURLLinkUnit("SelectAdminTab", "Select Admin tab").
        setURL("uicomponent=UIViewController&op=selectNode&objectId=Admin").        
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIToolbar&op=manageRelations"))      
    ) ;
    
    addWebUnit(
        new ClickURLLinkUnit("SelectManageRelations", "Select Manage Relation").
        setURL("uicomponent=UIToolbar&op=manageRelations").        
        addValidator(new ExpectLinkWithURLValidator("uicomponent=UIRelationsList&op=add"))      
    ) ;
    
    addWebUnit(
        new ClickURLLinkUnit("ClickOnAddRelation", "Click on add relation icon").
        setURL("uicomponent=UIRelationsList&op=add").        
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=JcrRelationBrowser&op=changeNode&objectId=/cms"))      
    ) ;   
    
    addWebUnit(
        new ClickURLLinkUnit("ClickOnCMS", "Click on CMS link").
        setURL("&uicomponent=JcrRelationBrowser&op=changeNode&objectId=/cms").        
        addValidator(new ExpectLinkWithURLValidator("uicomponent=JcrRelationBrowser&op=changeNode&objectId=/cms/publications"))      
    ) ;
    
    addWebUnit(
        new ClickURLLinkUnit("ClickOnPublications", "Click on add relation icon").
        setURL("&uicomponent=JcrRelationBrowser&op=changeNode&objectId=/cms/publications").        
        addValidator(new ExpectLinkWithURLValidator("uicomponent=JcrRelationBrowser&op=selectNode&objectId=/cms/publications/relationDoc"))      
    ) ; 
     
    addWebUnit(
        new ClickURLLinkUnit("SelectRelationsDoc", "Select relationDoc").
        setURL("uicomponent=JcrRelationBrowser&op=selectNode&objectId=/cms/publications/relationDoc").        
        addValidator(new ExpectTextValidator("/cms/publications/relationDoc"))      
    ) ;
    if (UserECMFileExplorerSuite.isEnableJCR) {
      //Click on preferences icon
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;
      //Save session
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }     
    addWebUnit(
     new ClickURLLinkUnit("SelectInfoTab ", "Select Info tab").
     setURL("uicomponent=UIViewController&op=selectNode&objectId=Info").        
     addValidator(new ExpectLinkWithURLValidator("uicomponent=UIToolbar&op=viewRelations"))      
    ) ;
   
    addWebUnit(
     new ClickURLLinkUnit("ClickOnViewRelation ", "Click on view relation").
     setURL("uicomponent=UIToolbar&op=viewRelations").        
     addValidator(new ExpectNoTextValidator("This node hasn't been relationable yet")).      
     addValidator(new ExpectTextValidator("Related Document"))
    ) ;
   
    addWebUnit(
     new ClickURLLinkUnit("ClickOnrelationDoc ", "Click on relationDoc").
     setURL("uicomponent=UIContentManager&op=viewNode&objectId=/cms/publications/relationDoc").                                  
     addValidator(new ExpectTextValidator("relationDoc")).
     addValidator(new ExpectNoTextValidator("Related Document"))
    ) ;
    addWebUnit(
       new ClickURLLinkUnit("Back", "Click on Back To test folder").
       setURL("&uicomponent=MainToolbar&op=back").                
       addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=viewNode&objectId=/cms/publications/relationDoc"))
    ) ; 
    addWebUnit(
     new ClickURLLinkUnit("Back to test folder", "Click on Back To test folder").
     setURL("&uicomponent=MainToolbar&op=back").                
     addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/#{client.name}/test-folder/test-doc"))      
   ) ;     
   addWebUnit(
     new ContextManagementUnit(ContextManagementUnit.SAVE, "Save Context")
   );        
                   //-----Back to exoadmin account      
   String[] admin = ["user", "admin"] ;
   addWebSuite(new UserLoginSuite("exoadmin", "exo", admin)) ; 
   goToPage("/Home/ecm") ;
   addWebUnit(
     new ClickURLLinkUnit("SelectDrive", "select Drive button").
     setURL("&uicomponent=MainToolbar&op=selectDrive").
     addCondition(new HaveLinkWithURLCondition("&uicomponent=MainToolbar&op=selectDrive")).
     addValidator(new ExpectTextValidator("Drives"))      
   ) ;
   addWebUnit(
     new ClickURLLinkUnit("ClickOnDrive-test", "click on drive-test button").
     setURL("&uicomponent=UIDriveBrowser&op=selectWorkspace&objectId=drive-test").               
     addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=selectDrive"))      
   ) ;  
   //Select view
   addWebUnit(
     new SubmitFormUnit("SelectCreatedNewView", "select created new view").
     setFormId("UISelectViewForm").            
     setField("views","view-test").
     setField("op","save").
     addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIViewController&op=selectNode&objectId=Admin"))      
   ) ;   
   // Goto publications folder and rename relationDoc to newRelationDoc
    addWebUnit(
       new ClickURLLinkUnit("ClickOnCms", "click on home cms").
       setURL("uicomponent=UIContentManager&op=changeNode&objectId=/cms").             
       addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/publications"))           
   ) ;   
   addWebUnit(
       new ClickURLLinkUnit("ClickOnPublications", "click on publications link").
       setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/cms/publications").
       addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIViewController&op=selectNode&objectId=Actions"))
   ) ;
   addWebUnit(
       new ClickURLLinkUnit("DeleteNewRelationDoc", "click on delete icon if newRelationDoc existed").
	   setURL("&uicomponent=UIContentManager&op=removeNode&objectId=/cms/publications/newRelationDoc").
	   addCondition(new HaveLinkWithURLCondition("&uicomponent=UIContentManager&op=removeNode&objectId=/cms/publications/newRelationDoc")).    		
	   addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIContentManager&op=removeNode&objectId=/cms/publications/newRelationDoc", 0))
    ) ;
   addWebUnit(
     new ClickURLLinkUnit("RenameRelationdoc", "Rename relationDoc").
     setURL("&uicomponent=UIContentManager&op=rename&objectId=/cms/publications/relationDoc").                  
     addValidator(new ExpectFormIdValidator("renameForm"))      
   ) ;     
   addWebUnit(
     new SubmitFormUnit("SaveRename", "click on save button").
     setFormId("renameForm").   
     setField("oldName","relationDoc").
     setField("newName","newRelationDoc").
     setField("op","save").
     addValidator(new ExpectTextValidator("The node \"relationDoc\" was renamed to \"newRelationDoc\" successfully."))
   ) ;
   
                  //-----Back to current user
   addWebUnit(
     new ContextManagementUnit(ContextManagementUnit.RESTORE, "Restore Context")
   );
   addWebUnit(
     new ClickURLLinkUnit("RenameTest-doc", "Rename test-doc").
     setURL("&uicomponent=UIContentManager&op=rename&objectId=/home/users/#{client.name}/test-folder/test-doc").       
     addValidator(new ExpectFormIdValidator("renameForm"))      
   ) ;     
   addWebUnit(
     new SubmitFormUnit("SaveRename", "click on save button").
     setFormId("renameForm").   
     setField("oldName","test-doc").
     setField("newName","newtest-doc").
     setField("op","save").
     addValidator(new ExpectTextValidator("The node \"test-doc\" was renamed to \"newtest-doc\" successfully."))     
   ) ; 
   COMMENT = "Have bug when you click on newtest-doc, so you can't select newRelationDoc";
   addWebUnit(
     new TODO("HaveBugWhenClickOnNewtest-doc", COMMENT).
     setComment(COMMENT).
     setStatus(WebUnit.STATUS_INCOMPLETE)
   ) ;
   addWebUnit(
     new ClickURLLinkUnit("SelectNewtest-doc", "Select newtest-doc").
     setURL("&uicomponent=UIContentManager&op=changeNode&objectId=/home/users/#{client.name}/test-folder/newtest-doc")     
   ) ;           
   addWebUnit(
    new ClickURLLinkUnit("SelectInfoTab ", "Select Info tab").
    setURL("uicomponent=UIViewController&op=selectNode&objectId=Info").        
    addValidator(new ExpectLinkWithURLValidator("uicomponent=UIToolbar&op=viewRelations"))      
   ) ;
  
   addWebUnit(
    new ClickURLLinkUnit("ClickOnViewRelation ", "Click on view relation").
    setURL("uicomponent=UIToolbar&op=viewRelations").        
    addValidator(new ExpectNoTextValidator("This node hasn't been relationable yet")).      
    addValidator(new ExpectTextValidator("Related Document"))
   ) ;
  
   addWebUnit(
    new ClickURLLinkUnit("ClickOnnewRelationDoc ", "Click on newRelationDoc").
    setURL("uicomponent=UIContentManager&op=viewNode&objectId=/cms/publications/newRelationDoc").                                  
    addValidator(new ExpectTextValidator("newRelationDoc")).
    addValidator(new ExpectNoTextValidator("Related Document"))
   ) ;
   
   //Return to drive selection
   addWebUnit(
     new ClickURLLinkUnit("SelectDrive", "select Drive button").
     setURL("&uicomponent=MainToolbar&op=selectDrive").                  
     addValidator(new ExpectTextValidator("Drives"))      
   ) ;
  }                 
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user"] ;
    return [new NewAccountSuite(), new UserLoginSuite("#{client.name}", "#{client.name}", roles)] ;
  }  
}