/***************************************************************************
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package suites.ecm.user;

import condition.* ;
import suites.share.* ;
import org.exoplatform.test.web.*;
import org.exoplatform.test.web.unit.*;
import org.exoplatform.test.web.validator.*;
import suites.runner.ECMUserSuites;

/**
 * Created by The eXo Platform SARL
 * Author : Dung Hoang
 *          dunghoang@exoplatform.com, jobdung@yahoo.com
 * Dec 27, 2005
 */
public class UserViewPermissionsSuite extends WebUnitSuite {
  static String DESC = "Go to Ecm Portlet User page, click on Info tab, click on View Permissions button and to do somethings" ;
  static String NAME = "UserViewPermissionsSuite" ;
  String COMMENT  = "You must run AdminViewSuite and AdminDriveSuite before run this suite";
  
  public UserViewPermissionsSuite() {    
    super(NAME, DESC) ;
    addWebSuite(new UserAddFolderSuite());
    //-----Select Info tab
    addWebUnit(
      new ClickURLLinkUnit("SelectInfoTab", "click on info tab").
      setURL("&uicomponent=UIViewController&op=selectNode&objectId=Info").  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIToolbar&op=viewPermissions"))
    ) ;   
    
    //-----View Permisions
    addWebUnit(
      new ClickURLLinkUnit("ClickOnViewPermisions", "click on View Permisions button").
      setURL("&uicomponent=UIToolbar&op=viewPermissions").                  
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIPermissionsInfo&op=removePermission&objectId=#{client.name}")).
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIPermissionsInfo&op=removePermission&objectId=*:/admin")).      
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIPermissionsInfo&op=addPermission"))      
    ) ;
    
    //----------Add Permisions
    addWebUnit(
      new ClickURLLinkUnit("ClickOnAddPermisions", "click on Add Permisions button").
      setURL("&uicomponent=UIPermissionsInfo&op=addPermission").                  
      addValidator(new ExpectFormIdValidator("permissionForm"))      
    ) ;    
    
    //---------------Search Group
    addWebUnit(
      new SubmitFormUnit("SearchGroup", "click on search button").
      setFormId("permissionForm").   
      setField("op","searchGroup").               
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIExoPermissionBrowser&op=selectGroup&objectId=/user"))      
    ) ; 
       
    addWebUnit(
      new ClickURLLinkUnit("SelectGroup", "select a group").
      setURL("&uicomponent=UIExoPermissionBrowser&op=selectGroup&objectId=/admin").              
      addValidator(new ExpectFormIdValidator("permissionForm"))      
    ) ; 
                
    //---------------Save Permisions
    addWebUnit(
      new SubmitFormUnit("SavePermission", "click on save button").
      setFormId("permissionForm").   
      setField("userOrGroup","*:/user").
      setCheckBox("read",true).
      setCheckBox("add_node",true).
      setCheckBox("set_property",true).
      setCheckBox("remove",true).
      setField("op","save").
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIPermissionsInfo&op=removePermission&objectId=*:/user"))
    ) ;  
               
    //Delete created permission
    addWebUnit(
      new ClickURLLinkUnit("DeletePermission", "click on delete icon").
      setURL("&uicomponent=UIPermissionsInfo&op=removePermission&objectId=*:/user").        
      addValidator(new ExpectLinkWithURLValidator("&uicomponent=UIPermissionsInfo&op=removePermission&objectId=*:/user",0))           										  
    ) ;
    if (UserECMFileExplorerSuite.isEnableJCR) {
      //Click on preferences icon
      addWebUnit(
        new ClickURLLinkUnit("ClickOnPreferences", "click on preferences icon").
        setURL("uicomponent=MainToolbar&op=preferences").                  
        addValidator(new ExpectFormIdValidator("preferencesForm"))      
      ) ; 
      addWebUnit(
        new SubmitFormUnit("SelectEnableJCRLikeStructure", "Select Enable JCR like structure and save").
        setFormId("preferencesForm").
        setCheckBox("checkbox",true).
        setField("op","save").
        addValidator(new ExpectLinkWithURLValidator("&uicomponent=MainToolbar&op=saveSession"))
      ) ;
      //Save session
      addWebUnit(
        new ClickURLLinkUnit("SavedSession", "click on save Session button").
        setURL("&uicomponent=MainToolbar&op=saveSession").        
        addValidator(new ExpectTextValidator("All the changes have been saved"))      
      ) ;
    }     
    //Select Drive
    addWebUnit(
      new ClickURLLinkUnit("SelectDrive", "select Drive button").
      setURL("&uicomponent=MainToolbar&op=selectDrive").                  
      addValidator(new ExpectTextValidator("Drives"))      
    ) ;      
  }                 
  public java.util.List getRequiredSuites() { 
    String[] roles = ["user"] ;
    return [new NewAccountSuite(), new UserLoginSuite("#{client.name}", "#{client.name}", roles)] ;
  }  
}